<?php

namespace Ids\Andreani\Setup;

use Magento\Eav\Setup\EavSetup;
use Magento\Eav\Setup\EavSetupFactory;
use Magento\Customer\Setup\CustomerSetupFactory;
use Magento\Framework\Setup\InstallDataInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Customer\Model\Customer;
use Magento\Eav\Model\Entity\Attribute\Set as AttributeSet;
use Magento\Eav\Model\Entity\Attribute\SetFactory as AttributeSetFactory;
use Magento\Eav\Api\AttributeRepositoryInterface;

/**
 * @codeCoverageIgnore
 */
class InstallData implements InstallDataInterface
{
    /**
     * EAV setup factory
     *
     * @var EavSetupFactory
     */
    private $eavSetupFactory;

    /**
     * @var CustomerSetupFactory
     */
    protected $customerSetupFactory;

    /**
     * @var AttributeSetFactory
     */
    private $attributeSetFactory;

    /**
     * @var AttributeSetFactory
     */
    private $attributeRepositoryInterface;

    /**
     * InstallData constructor.
     * @param EavSetupFactory $eavSetupFactory
     * @param CustomerSetupFactory $customerSetupFactory
     * @param AttributeSetFactory $attributeSetFactory
     * @param AttributeRepositoryInterface $attributeRepositoryInterface
     */
    public function __construct(
        EavSetupFactory $eavSetupFactory,
        CustomerSetupFactory $customerSetupFactory,
        AttributeSetFactory $attributeSetFactory,
        AttributeRepositoryInterface $attributeRepositoryInterface
    )
    {
        $this->eavSetupFactory              = $eavSetupFactory;
        $this->customerSetupFactory         = $customerSetupFactory;
        $this->attributeSetFactory          = $attributeSetFactory;
        $this->attributeRepositoryInterface = $attributeRepositoryInterface;
    }

    /**
     * {@inheritdoc}
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     */
    public function install(ModuleDataSetupInterface $setup, ModuleContextInterface $context)
    {
        $setup->startSetup();

        /** @var EavSetup $eavSetup */
        $eavSetup = $this->eavSetupFactory->create(['setup' => $setup]);

        $eavSetup->addAttribute(
            \Magento\Catalog\Model\Product::ENTITY,
            'volumen',
            [
                'frontend'  => '',
                'label'     => 'Volumen',
                'input'     => 'text',
                'class'     => '',
                'global'    => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
                'visible'   => true,
                'required'  => false,
                'user_defined' => false,
                'default'   => '',
                'apply_to'  => '',
                'visible_on_front'        => false,
                'is_used_in_grid'         => false,
                'is_visible_in_grid'      => false,
                'is_filterable_in_grid'   => false,
                'used_in_product_listing' => true
            ]
        );

        /**
         * Atributos customer_address
         */
        $customerAddressSetup = $this->customerSetupFactory->create(['setup' => $setup]);

        $customerAddressEntity = $customerAddressSetup->getEavConfig()->getEntityType(
            \Magento\Customer\Api\AddressMetadataInterface::ENTITY_TYPE_ADDRESS
        );
        $attributeCustomerAddressSetId      = $customerAddressEntity->getDefaultAttributeSetId();
        $attributeCustomerAddressSet        = $this->attributeSetFactory->create();
        $attributeCustomerAddressGroupId    = $attributeCustomerAddressSet->getDefaultGroupId($attributeCustomerAddressSetId);

        $attributesAddressInfo = [
            'dni' => [
                'label'     => 'DNI',
                'type'      => 'int',
                'input'     => 'text',
                'global'    => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
                'visible'   => true,
                'required'  => true,
                'user_defined' => true,
                'frontend_class'     => 'validate-number',
                'validate_rules'     => '{"max_text_length":14, "min_text_length":7}',
                'visible_on_front'   => true,
                'sort_order'  => 118,
                'position'    => 118,
                'system'      => 0,
            ],
            'celular' => [
                'type' => 'int',
                'label' => 'Celular',
                'input'     => 'text',
                'global'    => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
                'visible'   => true,
                'required'  => false,
                'user_defined' => true,
                'frontend_class'     => 'validate-number',
                'validate_rules'     => '{"max_text_length":20}',
                'visible_on_front'   => true,
                'sort_order'  => 200,
                'position'    => 200,
                'system'      => 0,
            ],
            'altura' => [
                'type' => 'int',
                'label' => 'Altura',
                'input' => 'text',
                'global'    => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
                'visible'   => true,
                'required'  => true,
                'user_defined' => true,
                'frontend_class'     => 'validate-number',
                'validate_rules'     => '{"min_text_length":1}',
                'visible_on_front'   => true,
                'sort_order'  => 70,
                'position'    => 70,
                'system'      => 0,
            ],
            'piso' => [
                'type' => 'int',
                'label' => 'Piso',
                'input' => 'text',
                'global'    => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
                'visible'   => true,
                'required'  => false,
                'user_defined' => true,
                'visible_on_front'   => true,
                'sort_order'  => 71,
                'position'    => 71,
                'system'      => 0,
            ],
            'departamento' => [
                'type' => 'varchar',
                'label' => 'Departamento',
                'input' => 'text',
                'global'    => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
                'visible'   => true,
                'required'  => false,
                'user_defined' => true,
                'visible_on_front'   => true,
                'sort_order'  => 72,
                'position'    => 72,
                'system'      => 0,
            ],
            'observaciones' => [
                'type' => 'varchar',
                'label' => 'Observaciones',
                'input' => 'text',
                'global'    => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
                'visible'   => true,
                'required'  => false,
                'user_defined' => true,
                'visible_on_front'   => true,
                'sort_order'  => 1001,
                'position'    => 1001,
                'system'      => 0,
            ],
          
        ];

        foreach ($attributesAddressInfo as $attributeCode => $attributeParams) {
                $customerAddressSetup->addAttribute(
                    'customer_address',
                    $attributeCode,
                    $attributeParams
                );
                $customerAddressAttribute = $customerAddressSetup->getEavConfig()->getAttribute(
                    \Magento\Customer\Api\AddressMetadataInterface::ENTITY_TYPE_ADDRESS,
                    $attributeCode
                )
                    ->addData([
                        'attribute_set_id'   => $attributeCustomerAddressSetId,
                        'attribute_group_id' => $attributeCustomerAddressGroupId,
                        'used_in_forms'      =>
                            [
                                'customer_address_edit',
                                'adminhtml_customer_address'
                            ],
                    ]);
                $customerAddressAttribute->save();
            

        }
        $infoIdsAndreniProvincia =
            [
                [ 'provincia_id'=>2, 'nombre' => 'BUENOS AIRES'],
                [ 'provincia_id'=>3, 'nombre' => 'CORDOBA'],
                [ 'provincia_id'=>4, 'nombre' => 'CHUBUT'],
                [ 'provincia_id'=>5, 'nombre' => 'CHACO'],
                [ 'provincia_id'=>6, 'nombre' => 'CORRIENTES'],
                [ 'provincia_id'=>7, 'nombre' => 'CATAMARCA'],
                [ 'provincia_id'=>8, 'nombre' => 'ENTRE RIOS'],
                [ 'provincia_id'=>9, 'nombre' => 'LA PAMPA'],
                [ 'provincia_id'=>10, 'nombre' => 'JUJUY'],
                [ 'provincia_id'=>11, 'nombre' => 'LA RIOJA'],
                [ 'provincia_id'=>12, 'nombre' => 'MENDOZA'],
                [ 'provincia_id'=>13, 'nombre' => 'FORMOSA'],
                [ 'provincia_id'=>14, 'nombre' => 'MISIONES'],
                [ 'provincia_id'=>15, 'nombre' => 'NUEQUEN'],
                [ 'provincia_id'=>16, 'nombre' => 'RIO NEGRO'],
                [ 'provincia_id'=>17, 'nombre' => 'SALTA'],
                [ 'provincia_id'=>18, 'nombre' => 'SAN JUAN'],
                [ 'provincia_id'=>19, 'nombre' => 'SAN LUIS'],
                [ 'provincia_id'=>20, 'nombre' => 'SANTA CRUZ'],
                [ 'provincia_id'=>21, 'nombre' => 'SANTA FE'],
                [ 'provincia_id'=>22, 'nombre' => 'SANTIAGO DEL ESTERO'],
                [ 'provincia_id'=>23, 'nombre' => 'TIERRA DEL FUEGO'],
                [ 'provincia_id'=>24, 'nombre' => 'TUCUMAN'],
            ];

        $setup->getConnection()
            ->insertArray($setup->getTable('ids_andreani_provincia'),['provincia_id','nombre'], $infoIdsAndreniProvincia);

        $infoIdsAndreniZona =
            [
                [ 'nombre' => 'Local'],
                [ 'nombre' => 'Interior 1'],
                [ 'nombre' => 'Interior 2']
            ];

        $setup->getConnection()
            ->insertArray($setup->getTable('ids_andreani_zona'),['nombre'], $infoIdsAndreniZona);

        $infoIdsAndreniTarifa =
            [
                [1000,130,120,200,1],
                [1400,130,120,200,1],
                [1920,150,150,200,1],
                [2440,150,150,200,1],
                [3004,150,150,200,1],
                [5290,360,240,200,1],
                [1000,130,120,220,2],
                [1400,130,120,220,2],
                [1920,200,150,220,2],
                [2440,200,200,250,2],
                [3004,240,200,250,2],
                [5290,360,360,260,2],
                [1000,130,130,250,3],
                [1400,130,130,250,3],
                [1920,200,150,250,3],
                [2440,200,200,280,3],
                [3004,240,240,280,3],
                [5290,360,360,280,3]
            ];

        $setup->getConnection()
            ->insertArray($setup->getTable('ids_andreani_tarifa'),
                ['rango','valor_estandar','valor_sucursal','valor_urgente','zona_id'], $infoIdsAndreniTarifa);


        $infoIdsAndreniCodigoPostal =
            [
                ['BARRACAS','C.A.B.A.','1130','2','1'],
                ['BARRACAS','C.A.B.A.','1131','2','1'],
                ['BARRACAS','C.A.B.A.','1132','2','1'],
                ['BARRACAS','C.A.B.A.','1133','2','1'],
                ['BARRACAS','C.A.B.A.','1134','2','1'],
                ['BARRACAS','C.A.B.A.','1135','2','1'],
                ['BARRACAS','C.A.B.A.','1136','2','1'],
                ['BARRACAS','C.A.B.A.','1137','2','1'],
                ['BARRACAS','C.A.B.A.','1138','2','1'],
                ['BARRACAS','C.A.B.A.','1139','2','1'],
                ['BARRACAS','C.A.B.A.','1140','2','1'],
                ['BARRACAS','C.A.B.A.','1141','2','1'],
                ['BARRACAS','C.A.B.A.','1142','2','1'],
                ['BARRACAS','C.A.B.A.','1143','2','1'],
                ['BARRACAS','C.A.B.A.','1144','2','1'],
                ['BARRACAS','C.A.B.A.','1145','2','1'],
                ['BARRACAS','C.A.B.A.','1146','2','1'],
                ['BARRACAS','C.A.B.A.','1147','2','1'],
                ['BARRACAS','C.A.B.A.','1148','2','1'],
                ['BARRACAS','C.A.B.A.','1149','2','1'],
                ['BARRACAS','C.A.B.A.','1150','2','1'],
                ['BARRACAS','C.A.B.A.','1151','2','1'],
                ['BARRACAS','C.A.B.A.','1152','2','1'],
                ['BARRACAS','C.A.B.A.','1153','2','1'],
                ['BARRACAS','C.A.B.A.','1154','2','1'],
                ['BARRACAS','C.A.B.A.','1155','2','1'],
                ['BARRACAS','C.A.B.A.','1156','2','1'],
                ['BARRACAS','C.A.B.A.','1157','2','1'],
                ['BARRACAS','C.A.B.A.','1158','2','1'],
                ['BARRACAS','C.A.B.A.','1159','2','1'],
                ['BARRACAS','C.A.B.A.','1160','2','1'],
                ['BARRACAS','C.A.B.A.','1161','2','1'],
                ['BARRACAS','C.A.B.A.','1162','2','1'],
                ['BARRACAS','C.A.B.A.','1163','2','1'],
                ['BARRACAS','C.A.B.A.','1164','2','1'],
                ['BARRACAS','C.A.B.A.','1165','2','1'],
                ['BARRACAS','C.A.B.A.','1166','2','1'],
                ['BARRACAS','C.A.B.A.','1167','2','1'],
                ['BARRACAS','C.A.B.A.','1168','2','1'],
                ['BARRACAS','C.A.B.A.','1169','2','1'],
                ['BARRACAS','C.A.B.A.','1237','2','1'],
                ['BARRACAS','C.A.B.A.','1238','2','1'],
                ['BARRACAS','C.A.B.A.','1239','2','1'],
                ['BARRACAS','C.A.B.A.','1240','2','1'],
                ['BARRACAS','C.A.B.A.','1241','2','1'],
                ['BARRACAS','C.A.B.A.','1242','2','1'],
                ['BARRACAS','C.A.B.A.','1243','2','1'],
                ['BARRACAS','C.A.B.A.','1244','2','1'],
                ['BARRACAS','C.A.B.A.','1245','2','1'],
                ['BARRACAS','C.A.B.A.','1246','2','1'],
                ['BARRACAS','C.A.B.A.','1247','2','1'],
                ['BARRACAS','C.A.B.A.','1248','2','1'],
                ['BARRACAS','C.A.B.A.','1249','2','1'],
                ['BARRACAS','C.A.B.A.','1250','2','1'],
                ['BARRACAS','C.A.B.A.','1251','2','1'],
                ['BARRACAS','C.A.B.A.','1252','2','1'],
                ['BARRACAS','C.A.B.A.','1253','2','1'],
                ['BARRACAS','C.A.B.A.','1254','2','1'],
                ['BARRACAS','C.A.B.A.','1255','2','1'],
                ['BARRACAS','C.A.B.A.','1256','2','1'],
                ['BARRACAS','C.A.B.A.','1257','2','1'],
                ['BARRACAS','C.A.B.A.','1258','2','1'],
                ['BARRACAS','C.A.B.A.','1259','2','1'],
                ['BARRACAS','C.A.B.A.','1260','2','1'],
                ['BARRACAS','C.A.B.A.','1261','2','1'],
                ['BARRACAS','C.A.B.A.','1262','2','1'],
                ['BARRACAS','C.A.B.A.','1263','2','1'],
                ['BARRACAS','C.A.B.A.','1264','2','1'],
                ['BARRACAS','C.A.B.A.','1265','2','1'],
                ['BARRACAS','C.A.B.A.','1266','2','1'],
                ['BARRACAS','C.A.B.A.','1267','2','1'],
                ['BARRACAS','C.A.B.A.','1268','2','1'],
                ['BARRACAS','C.A.B.A.','1269','2','1'],
                ['BARRACAS','C.A.B.A.','1270','2','1'],
                ['BARRACAS','C.A.B.A.','1271','2','1'],
                ['BARRACAS','C.A.B.A.','1272','2','1'],
                ['BARRACAS','C.A.B.A.','1273','2','1'],
                ['BARRACAS','C.A.B.A.','1274','2','1'],
                ['BARRACAS','C.A.B.A.','1275','2','1'],
                ['BARRACAS','C.A.B.A.','1276','2','1'],
                ['BARRACAS','C.A.B.A.','1277','2','1'],
                ['BARRACAS','C.A.B.A.','1278','2','1'],
                ['BARRACAS','C.A.B.A.','1279','2','1'],
                ['BARRACAS','C.A.B.A.','1280','2','1'],
                ['BARRACAS','C.A.B.A.','1281','2','1'],
                ['BARRACAS','C.A.B.A.','1282','2','1'],
                ['BARRACAS','C.A.B.A.','1283','2','1'],
                ['BARRACAS','C.A.B.A.','1284','2','1'],
                ['BARRACAS','C.A.B.A.','1285','2','1'],
                ['BARRACAS','C.A.B.A.','1286','2','1'],
                ['BARRACAS','C.A.B.A.','1287','2','1'],
                ['BARRACAS','C.A.B.A.','1288','2','1'],
                ['BARRACAS','C.A.B.A.','1289','2','1'],
                ['BARRACAS','C.A.B.A.','1290','2','1'],
                ['BARRACAS','C.A.B.A.','1291','2','1'],
                ['BARRACAS','C.A.B.A.','1292','2','1'],
                ['BARRACAS','C.A.B.A.','1293','2','1'],
                ['BARRACAS','C.A.B.A.','1294','2','1'],
                ['BARRACAS','C.A.B.A.','1295','2','1'],
                ['BARRACAS','C.A.B.A.','1296','2','1'],
                ['BARRACAS','C.A.B.A.','1298','2','1'],
                ['BARRACAS','C.A.B.A.','1422','2','1'],
                ['BARRACAS','C.A.B.A.','1432','2','1'],
                ['BARRACAS','C.A.B.A.','1433','2','1'],
                ['BARRACAS','C.A.B.A.','1434','2','1'],
                ['BARRACAS','C.A.B.A.','1436','2','1'],
                ['BARRACAS','C.A.B.A.','1437','2','1'],
                ['COLEGIALES','C.A.B.A.','1426','2','1'],
                ['COLEGIALES','C.A.B.A.','1428','2','1'],
                ['COLEGIALES','C.A.B.A.','1429','2','1'],
                ['COLEGIALES','C.A.B.A.','1430','2','1'],
                ['FLORENCIO VARELA','BERAZATEGUI','1880','2','1'],
                ['FLORENCIO VARELA','BERAZATEGUI','1884','2','1'],
                ['FLORENCIO VARELA','EL PATO','1893','2','1'],
                ['FLORENCIO VARELA','FLORENCIO VARELA','1888','2','1'],
                ['FLORENCIO VARELA','GOBERNADOR COSTA','1859','2','1'],
                ['FLORENCIO VARELA','HUDSON','1885','2','1'],
                ['FLORENCIO VARELA','INGENIERO ALLAN','1891','2','1'],
                ['FLORENCIO VARELA','JOSE MARIA GUTIERREZ','1890','2','1'],
                ['FLORENCIO VARELA','KM. 40','1889','2','1'],
                ['FLORENCIO VARELA','PLATANOS','1861','2','1'],
                ['FLORENCIO VARELA','RANELAGH','1886','2','1'],
                ['FLORENCIO VARELA','SAN JOSE','1844','2','1'],
                ['FLORENCIO VARELA','SOURIGUES','1837','2','1'],
                ['FLORENCIO VARELA','VILLA ESPAÑA','1860','2','1'],
                ['LA PLATA','ABASTO','1933','2','1'],
                ['LA PLATA','ABBOTT','7228','2','1'],
                ['LA PLATA','ADELA','7136','2','1'],
                ['LA PLATA','ALEJANDRO PETION','1811','2','1'],
                ['LA PLATA','ARANA','1909','2','1'],
                ['LA PLATA','ARTURO SEGUI','1895','2','1'],
                ['LA PLATA','ATALAYA','1947','2','1'],
                ['LA PLATA','BALNEARIO PUNTA LARA','1931','2','1'],
                ['LA PLATA','BERISO','1924','2','1'],
                ['LA PLATA','BERISSO','1923','2','1'],
                ['LA PLATA','CHASCOMUS','7130','2','1'],
                ['LA PLATA','CITY BELL','1896','2','1'],
                ['LA PLATA','CORONEL BRANDSEN','1980','2','1'],
                ['LA PLATA','DOMSELAAR','1984','2','1'],
                ['LA PLATA','DON CIPRIANO','7135','2','1'],
                ['LA PLATA','EL CARMEN','1907','2','1'],
                ['LA PLATA','EMPALME MAGDALENA','1948','2','1'],
                ['LA PLATA','ENSENADA','1925','2','1'],
                ['LA PLATA','ENSENADA','1926','2','1'],
                ['LA PLATA','ESCUELA NAVAL MILITAR RIO SANTIAGO','1927','2','1'],
                ['LA PLATA','ESTACION MORENO','1901','2','1'],
                ['LA PLATA','GENERAL BELGRANO','7223','2','1'],
                ['LA PLATA','GOBERNADOR UDAONDO','7221','2','1'],
                ['LA PLATA','GOMES','1983','2','1'],
                ['LA PLATA','ISLA SANTIAGO','1929','2','1'],
                ['LA PLATA','LA PLATA','1900','2','1'],
                ['LA PLATA','LA PLATA','1902','2','1'],
                ['LA PLATA','LA PLATA','1904','2','1'],
                ['LA PLATA','LA PLATA','1906','2','1'],
                ['LA PLATA','LA PLATA','1908','2','1'],
                ['LA PLATA','LA PLATA','1910','2','1'],
                ['LA PLATA','LA PLATA','1912','2','1'],
                ['LA PLATA','LA PLATA','1914','2','1'],
                ['LA PLATA','LISANDRO OLMOS ETCHEVERRI','1901','2','1'],
                ['LA PLATA','LOMA VERDE','1981','2','1'],
                ['LA PLATA','LOS TALAS','1923','2','1'],
                ['LA PLATA','LOS TALAS','1942','2','1'],
                ['LA PLATA','MALVINAS ARGENTINAS','1916','2','1'],
                ['LA PLATA','OLIDEN','1972','2','1'],
                ['LA PLATA','PEREYRA','1894','2','1'],
                ['LA PLATA','PIPINAS','1921','2','1'],
                ['LA PLATA','PIPINAS','1954','2','1'],
                ['LA PLATA','POBLET','1905','2','1'],
                ['LA PLATA','PUNTA INDIO','1919','2','1'],
                ['LA PLATA','PUNTA INDIO','1952','2','1'],
                ['LA PLATA','ROSAS','7205','2','1'],
                ['LA PLATA','VERONICA','1917','2','1'],
                ['LA PLATA','VIEYTES','1915','2','1'],
                ['LA PLATA','VILLANUEVA','7225','2','1'],
                ['LA PLATA','ZENON VIDELA DORNA','7226','2','1'],
                ['LANUS','AVELLANEDA','1870','2','1'],
                ['LANUS','CRUCESITA','1873','2','1'],
                ['LANUS','DOCK SUD','1871','2','1'],
                ['LANUS','GERLI','1823','2','1'],
                ['LANUS','GERLI','1869','2','1'],
                ['LANUS','LANUS','1824','2','1'],
                ['LANUS','PIÑEYRO','1868','2','1'],
                ['LANUS','REMEDIOS DE ESCALADA','1826','2','1'],
                ['LANUS','SARANDI','1872','2','1'],
                ['MONTE GRANDE','9 DE ABRIL','1839','2','1'],
                ['MONTE GRANDE','AEROPUERTO EZEIZA','1802','2','1'],
                ['MONTE GRANDE','CANNING','1801','2','1'],
                ['MONTE GRANDE','CANNING','1807','2','1'],
                ['MONTE GRANDE','COLONIA SANTA ROSA','1816','2','1'],
                ['MONTE GRANDE','EL JAGÑEL','1805','2','1'],
                ['MONTE GRANDE','EZEIZA','1804','2','1'],
                ['MONTE GRANDE','LUIS GUILLON','1838','2','1'],
                ['MONTE GRANDE','MAXIMO PAZ','1812','2','1'],
                ['MONTE GRANDE','MONTE GRANDE','1841','2','1'],
                ['MONTE GRANDE','MONTE GRANDE','1842','2','1'],
                ['MONTE GRANDE','VICENTE CASARES','1808','2','1'],
                ['MONTE GRANDE','VILLA TRANSRADIO','1776','2','1'],
                ['PALERMO','C.A.B.A.','1176','2','1'],
                ['PALERMO','C.A.B.A.','1414','2','1'],
                ['PALERMO','C.A.B.A.','1425','2','1'],
                ['QUILMES','BERNAL','1876','2','1'],
                ['QUILMES','BERNAL ','1883','2','1'],
                ['QUILMES','EZPELETA','1882','2','1'],
                ['QUILMES','EZPELETA OESTE','1840','2','1'],
                ['QUILMES','MALVINAS ARGENTINAS','1851','2','1'],
                ['QUILMES','PEREYRA','1857','2','1'],
                ['QUILMES','QUILMES','1878','2','1'],
                ['QUILMES','QUILMES OESTE','1879','2','1'],
                ['QUILMES','SAN FRANCISCO SOLANO','1881','2','1'],
                ['QUILMES','VILLA BROWN','1892','2','1'],
                ['QUILMES','VILLA DOMINICO','1874','2','1'],
                ['QUILMES','VILLA LA FLORIDA','1855','2','1'],
                ['QUILMES','WILDE','1875','2','1'],
                ['QUILMES','ZEBALLOS','1863','2','1'],
                ['SUIPACHA','C.A.B.A.','1001','2','1'],
                ['SUIPACHA','C.A.B.A.','1002','2','1'],
                ['SUIPACHA','C.A.B.A.','1003','2','1'],
                ['SUIPACHA','C.A.B.A.','1004','2','1'],
                ['SUIPACHA','C.A.B.A.','1005','2','1'],
                ['SUIPACHA','C.A.B.A.','1006','2','1'],
                ['SUIPACHA','C.A.B.A.','1007','2','1'],
                ['SUIPACHA','C.A.B.A.','1008','2','1'],
                ['SUIPACHA','C.A.B.A.','1009','2','1'],
                ['SUIPACHA','C.A.B.A.','1010','2','1'],
                ['SUIPACHA','C.A.B.A.','1011','2','1'],
                ['SUIPACHA','C.A.B.A.','1012','2','1'],
                ['SUIPACHA','C.A.B.A.','1013','2','1'],
                ['SUIPACHA','C.A.B.A.','1014','2','1'],
                ['SUIPACHA','C.A.B.A.','1015','2','1'],
                ['SUIPACHA','C.A.B.A.','1016','2','1'],
                ['SUIPACHA','C.A.B.A.','1017','2','1'],
                ['SUIPACHA','C.A.B.A.','1018','2','1'],
                ['SUIPACHA','C.A.B.A.','1019','2','1'],
                ['SUIPACHA','C.A.B.A.','1020','2','1'],
                ['SUIPACHA','C.A.B.A.','1021','2','1'],
                ['SUIPACHA','C.A.B.A.','1022','2','1'],
                ['SUIPACHA','C.A.B.A.','1023','2','1'],
                ['SUIPACHA','C.A.B.A.','1024','2','1'],
                ['SUIPACHA','C.A.B.A.','1025','2','1'],
                ['SUIPACHA','C.A.B.A.','1026','2','1'],
                ['SUIPACHA','C.A.B.A.','1027','2','1'],
                ['SUIPACHA','C.A.B.A.','1028','2','1'],
                ['SUIPACHA','C.A.B.A.','1029','2','1'],
                ['SUIPACHA','C.A.B.A.','1030','2','1'],
                ['SUIPACHA','C.A.B.A.','1031','2','1'],
                ['SUIPACHA','C.A.B.A.','1032','2','1'],
                ['SUIPACHA','C.A.B.A.','1033','2','1'],
                ['SUIPACHA','C.A.B.A.','1034','2','1'],
                ['SUIPACHA','C.A.B.A.','1035','2','1'],
                ['SUIPACHA','C.A.B.A.','1036','2','1'],
                ['SUIPACHA','C.A.B.A.','1037','2','1'],
                ['SUIPACHA','C.A.B.A.','1038','2','1'],
                ['SUIPACHA','C.A.B.A.','1039','2','1'],
                ['SUIPACHA','C.A.B.A.','1040','2','1'],
                ['SUIPACHA','C.A.B.A.','1041','2','1'],
                ['SUIPACHA','C.A.B.A.','1042','2','1'],
                ['SUIPACHA','C.A.B.A.','1043','2','1'],
                ['SUIPACHA','C.A.B.A.','1044','2','1'],
                ['SUIPACHA','C.A.B.A.','1045','2','1'],
                ['SUIPACHA','C.A.B.A.','1046','2','1'],
                ['SUIPACHA','C.A.B.A.','1047','2','1'],
                ['SUIPACHA','C.A.B.A.','1048','2','1'],
                ['SUIPACHA','C.A.B.A.','1049','2','1'],
                ['SUIPACHA','C.A.B.A.','1050','2','1'],
                ['SUIPACHA','C.A.B.A.','1051','2','1'],
                ['SUIPACHA','C.A.B.A.','1052','2','1'],
                ['SUIPACHA','C.A.B.A.','1053','2','1'],
                ['SUIPACHA','C.A.B.A.','1054','2','1'],
                ['SUIPACHA','C.A.B.A.','1055','2','1'],
                ['SUIPACHA','C.A.B.A.','1056','2','1'],
                ['SUIPACHA','C.A.B.A.','1057','2','1'],
                ['SUIPACHA','C.A.B.A.','1058','2','1'],
                ['SUIPACHA','C.A.B.A.','1059','2','1'],
                ['SUIPACHA','C.A.B.A.','1060','2','1'],
                ['SUIPACHA','C.A.B.A.','1061','2','1'],
                ['SUIPACHA','C.A.B.A.','1062','2','1'],
                ['SUIPACHA','C.A.B.A.','1063','2','1'],
                ['SUIPACHA','C.A.B.A.','1064','2','1'],
                ['SUIPACHA','C.A.B.A.','1065','2','1'],
                ['SUIPACHA','C.A.B.A.','1066','2','1'],
                ['SUIPACHA','C.A.B.A.','1067','2','1'],
                ['SUIPACHA','C.A.B.A.','1068','2','1'],
                ['SUIPACHA','C.A.B.A.','1069','2','1'],
                ['SUIPACHA','C.A.B.A.','1070','2','1'],
                ['SUIPACHA','C.A.B.A.','1071','2','1'],
                ['SUIPACHA','C.A.B.A.','1072','2','1'],
                ['SUIPACHA','C.A.B.A.','1073','2','1'],
                ['SUIPACHA','C.A.B.A.','1074','2','1'],
                ['SUIPACHA','C.A.B.A.','1075','2','1'],
                ['SUIPACHA','C.A.B.A.','1076','2','1'],
                ['SUIPACHA','C.A.B.A.','1077','2','1'],
                ['SUIPACHA','C.A.B.A.','1078','2','1'],
                ['SUIPACHA','C.A.B.A.','1079','2','1'],
                ['SUIPACHA','C.A.B.A.','1080','2','1'],
                ['SUIPACHA','C.A.B.A.','1081','2','1'],
                ['SUIPACHA','C.A.B.A.','1082','2','1'],
                ['SUIPACHA','C.A.B.A.','1083','2','1'],
                ['SUIPACHA','C.A.B.A.','1084','2','1'],
                ['SUIPACHA','C.A.B.A.','1085','2','1'],
                ['SUIPACHA','C.A.B.A.','1086','2','1'],
                ['SUIPACHA','C.A.B.A.','1087','2','1'],
                ['SUIPACHA','C.A.B.A.','1088','2','1'],
                ['SUIPACHA','C.A.B.A.','1089','2','1'],
                ['SUIPACHA','C.A.B.A.','1090','2','1'],
                ['SUIPACHA','C.A.B.A.','1091','2','1'],
                ['SUIPACHA','C.A.B.A.','1092','2','1'],
                ['SUIPACHA','C.A.B.A.','1093','2','1'],
                ['SUIPACHA','C.A.B.A.','1094','2','1'],
                ['SUIPACHA','C.A.B.A.','1095','2','1'],
                ['SUIPACHA','C.A.B.A.','1096','2','1'],
                ['SUIPACHA','C.A.B.A.','1097','2','1'],
                ['SUIPACHA','C.A.B.A.','1098','2','1'],
                ['SUIPACHA','C.A.B.A.','1099','2','1'],
                ['SUIPACHA','C.A.B.A.','1100','2','1'],
                ['SUIPACHA','C.A.B.A.','1101','2','1'],
                ['SUIPACHA','C.A.B.A.','1102','2','1'],
                ['SUIPACHA','C.A.B.A.','1103','2','1'],
                ['SUIPACHA','C.A.B.A.','1104','2','1'],
                ['SUIPACHA','C.A.B.A.','1105','2','1'],
                ['SUIPACHA','C.A.B.A.','1106','2','1'],
                ['SUIPACHA','C.A.B.A.','1107','2','1'],
                ['SUIPACHA','C.A.B.A.','1110','2','1'],
                ['SUIPACHA','C.A.B.A.','1111','2','1'],
                ['SUIPACHA','C.A.B.A.','1112','2','1'],
                ['SUIPACHA','C.A.B.A.','1113','2','1'],
                ['SUIPACHA','C.A.B.A.','1114','2','1'],
                ['SUIPACHA','C.A.B.A.','1115','2','1'],
                ['SUIPACHA','C.A.B.A.','1116','2','1'],
                ['SUIPACHA','C.A.B.A.','1117','2','1'],
                ['SUIPACHA','C.A.B.A.','1118','2','1'],
                ['SUIPACHA','C.A.B.A.','1119','2','1'],
                ['SUIPACHA','C.A.B.A.','1120','2','1'],
                ['SUIPACHA','C.A.B.A.','1121','2','1'],
                ['SUIPACHA','C.A.B.A.','1122','2','1'],
                ['SUIPACHA','C.A.B.A.','1123','2','1'],
                ['SUIPACHA','C.A.B.A.','1124','2','1'],
                ['SUIPACHA','C.A.B.A.','1125','2','1'],
                ['SUIPACHA','C.A.B.A.','1126','2','1'],
                ['SUIPACHA','C.A.B.A.','1127','2','1'],
                ['SUIPACHA','C.A.B.A.','1128','2','1'],
                ['SUIPACHA','C.A.B.A.','1129','2','1'],
                ['SUIPACHA','C.A.B.A.','1300','2','1'],
                ['SUIPACHA','C.A.B.A.','1301','2','1'],
                ['SUIPACHA','C.A.B.A.','1302','2','1'],
                ['SUIPACHA','C.A.B.A.','1303','2','1'],
                ['SUIPACHA','C.A.B.A.','1304','2','1'],
                ['SUIPACHA','C.A.B.A.','1305','2','1'],
                ['SUIPACHA','C.A.B.A.','1306','2','1'],
                ['SUIPACHA','C.A.B.A.','1307','2','1'],
                ['SUIPACHA','C.A.B.A.','1308','2','1'],
                ['SUIPACHA','C.A.B.A.','1309','2','1'],
                ['SUIPACHA','C.A.B.A.','1310','2','1'],
                ['SUIPACHA','C.A.B.A.','1311','2','1'],
                ['SUIPACHA','C.A.B.A.','1312','2','1'],
                ['SUIPACHA','C.A.B.A.','1313','2','1'],
                ['SUIPACHA','C.A.B.A.','1314','2','1'],
                ['SUIPACHA','C.A.B.A.','1315','2','1'],
                ['SUIPACHA','C.A.B.A.','1316','2','1'],
                ['SUIPACHA','C.A.B.A.','1317','2','1'],
                ['SUIPACHA','C.A.B.A.','1318','2','1'],
                ['SUIPACHA','C.A.B.A.','1319','2','1'],
                ['SUIPACHA','C.A.B.A.','1320','2','1'],
                ['SUIPACHA','C.A.B.A.','1321','2','1'],
                ['SUIPACHA','C.A.B.A.','1322','2','1'],
                ['SUIPACHA','C.A.B.A.','1323','2','1'],
                ['SUIPACHA','C.A.B.A.','1324','2','1'],
                ['SUIPACHA','C.A.B.A.','1325','2','1'],
                ['SUIPACHA','C.A.B.A.','1326','2','1'],
                ['SUIPACHA','C.A.B.A.','1327','2','1'],
                ['SUIPACHA','C.A.B.A.','1328','2','1'],
                ['SUIPACHA','C.A.B.A.','1329','2','1'],
                ['SUIPACHA','C.A.B.A.','1330','2','1'],
                ['SUIPACHA','C.A.B.A.','1331','2','1'],
                ['SUIPACHA','C.A.B.A.','1332','2','1'],
                ['SUIPACHA','C.A.B.A.','1333','2','1'],
                ['SUIPACHA','C.A.B.A.','1334','2','1'],
                ['SUIPACHA','C.A.B.A.','1335','2','1'],
                ['SUIPACHA','C.A.B.A.','1336','2','1'],
                ['SUIPACHA','C.A.B.A.','1337','2','1'],
                ['SUIPACHA','C.A.B.A.','1338','2','1'],
                ['SUIPACHA','C.A.B.A.','1339','2','1'],
                ['SUIPACHA','C.A.B.A.','1340','2','1'],
                ['SUIPACHA','C.A.B.A.','1341','2','1'],
                ['SUIPACHA','C.A.B.A.','1342','2','1'],
                ['SUIPACHA','C.A.B.A.','1343','2','1'],
                ['SUIPACHA','C.A.B.A.','1344','2','1'],
                ['SUIPACHA','C.A.B.A.','1345','2','1'],
                ['SUIPACHA','C.A.B.A.','1346','2','1'],
                ['SUIPACHA','C.A.B.A.','1347','2','1'],
                ['SUIPACHA','C.A.B.A.','1348','2','1'],
                ['SUIPACHA','C.A.B.A.','1349','2','1'],
                ['SUIPACHA','C.A.B.A.','1350','2','1'],
                ['SUIPACHA','C.A.B.A.','1351','2','1'],
                ['SUIPACHA','C.A.B.A.','1352','2','1'],
                ['SUIPACHA','C.A.B.A.','1353','2','1'],
                ['SUIPACHA','C.A.B.A.','1354','2','1'],
                ['SUIPACHA','C.A.B.A.','1355','2','1'],
                ['SUIPACHA','C.A.B.A.','1356','2','1'],
                ['SUIPACHA','C.A.B.A.','1357','2','1'],
                ['SUIPACHA','C.A.B.A.','1358','2','1'],
                ['SUIPACHA','C.A.B.A.','1359','2','1'],
                ['SUIPACHA','C.A.B.A.','1360','2','1'],
                ['SUIPACHA','C.A.B.A.','1361','2','1'],
                ['SUIPACHA','C.A.B.A.','1362','2','1'],
                ['SUIPACHA','C.A.B.A.','1363','2','1'],
                ['SUIPACHA','C.A.B.A.','1364','2','1'],
                ['SUIPACHA','C.A.B.A.','1365','2','1'],
                ['SUIPACHA','C.A.B.A.','1366','2','1'],
                ['SUIPACHA','C.A.B.A.','1367','2','1'],
                ['SUIPACHA','C.A.B.A.','1368','2','1'],
                ['SUIPACHA','C.A.B.A.','1369','2','1'],
                ['SUIPACHA','C.A.B.A.','1370','2','1'],
                ['SUIPACHA','C.A.B.A.','1371','2','1'],
                ['SUIPACHA','C.A.B.A.','1372','2','1'],
                ['SUIPACHA','C.A.B.A.','1373','2','1'],
                ['SUIPACHA','C.A.B.A.','1374','2','1'],
                ['SUIPACHA','C.A.B.A.','1375','2','1'],
                ['SUIPACHA','C.A.B.A.','1376','2','1'],
                ['SUIPACHA','C.A.B.A.','1377','2','1'],
                ['SUIPACHA','C.A.B.A.','1378','2','1'],
                ['SUIPACHA','C.A.B.A.','1379','2','1'],
                ['SUIPACHA','C.A.B.A.','1380','2','1'],
                ['SUIPACHA','C.A.B.A.','1381','2','1'],
                ['SUIPACHA','C.A.B.A.','1382','2','1'],
                ['SUIPACHA','C.A.B.A.','1383','2','1'],
                ['SUIPACHA','C.A.B.A.','1384','2','1'],
                ['SUIPACHA','C.A.B.A.','1385','2','1'],
                ['SUIPACHA','C.A.B.A.','1386','2','1'],
                ['SUIPACHA','C.A.B.A.','1387','2','1'],
                ['SUIPACHA','C.A.B.A.','1388','2','1'],
                ['SUIPACHA','C.A.B.A.','1389','2','1'],
                ['SUIPACHA','C.A.B.A.','1390','2','1'],
                ['SUIPACHA','C.A.B.A.','1391','2','1'],
                ['SUIPACHA','C.A.B.A.','1392','2','1'],
                ['SUIPACHA','C.A.B.A.','1393','2','1'],
                ['SUIPACHA','C.A.B.A.','1394','2','1'],
                ['SUIPACHA','C.A.B.A.','1395','2','1'],
                ['SUIPACHA','C.A.B.A.','1396','2','1'],
                ['SUIPACHA','C.A.B.A.','1397','2','1'],
                ['SUIPACHA','C.A.B.A.','1398','2','1'],
                ['SUIPACHA','C.A.B.A.','1399','2','1'],
                ['SUIPACHA','C.A.B.A.','1453','2','1'],
                ['SUIPACHA','C.A.B.A.','1599','2','1'],
                ['TEMPERLEY','BANFIELD','1821','2','1'],
                ['TEMPERLEY','BANFIELD','1827','2','1'],
                ['TEMPERLEY','BANFIELD','1828','2','1'],
                ['TEMPERLEY','INGENIERO BUDGE','1773','2','1'],
                ['TEMPERLEY','LOMAS DE ZAMORA','1832','2','1'],
                ['TEMPERLEY','TURDERA','1833','2','1'],
                ['TEMPERLEY','VILLA FIORITO','1831','2','1'],
                ['VILLA PUEYRREDON','C.A.B.A.','1417','2','1'],
                ['VILLA PUEYRREDON','C.A.B.A.','1419','2','1'],
                ['VILLA PUEYRREDON','C.A.B.A.','1427','2','1'],
                ['VILLA PUEYRREDON','C.A.B.A.','1431','2','1'],
                ['FLORES','C.A.B.A.','1400','2','1'],
                ['FLORES','C.A.B.A.','1401','2','1'],
                ['FLORES','C.A.B.A.','1402','2','1'],
                ['FLORES','C.A.B.A.','1406','2','1'],
                ['FLORES','C.A.B.A.','1407','2','1'],
                ['FLORES','C.A.B.A.','1408','2','1'],
                ['FLORES','C.A.B.A.','1409','2','1'],
                ['FLORES','C.A.B.A.','1410','2','1'],
                ['FLORES','C.A.B.A.','1411','2','1'],
                ['FLORES','C.A.B.A.','1412','2','1'],
                ['FLORES','C.A.B.A.','1416','2','1'],
                ['FLORES','C.A.B.A.','1420','2','1'],
                ['FLORES','C.A.B.A.','1421','2','1'],
                ['FLORES','C.A.B.A.','1424','2','1'],
                ['FLORES','C.A.B.A.','1439','2','1'],
                ['FLORES','C.A.B.A.','1440','2','1'],
                ['MORENO','20 DE JULIO','1786','2','1'],
                ['MORENO','9 DE ABRIL','1735','2','1'],
                ['MORENO','AGUSTIN FERRARI','1723','2','1'],
                ['MORENO','BARRIO LOS AROMOS SAN PATRICIO','1792','2','1'],
                ['MORENO','CUARTEL V','1740','2','1'],
                ['MORENO','EL PAMPERO','1866','2','1'],
                ['MORENO','ESTACION GOMEZ','1918','2','1'],
                ['MORENO','ESTACION MORENO','1935','2','1'],
                ['MORENO','FRANCISCO ALVAREZ','1746','2','1'],
                ['MORENO','GENERAL LAS HERAS','1741','2','1'],
                ['MORENO','KM 77','1737','2','1'],
                ['MORENO','LA REJA','1738','2','1'],
                ['MORENO','LA REJA','1745','2','1'],
                ['MORENO','M PAZ BARRIO BERNASCONI','1791','2','1'],
                ['MORENO','M PAZ BARRIO EL MARTILLO','1790','2','1'],
                ['MORENO','M PAZ BARRIO LA LONJA','1783','2','1'],
                ['MORENO','M PAZ BARRIO LA MILAGROSA','1784','2','1'],
                ['MORENO','M PAZ BARRIO MARTIN FIERRO','1782','2','1'],
                ['MORENO','MARCOS PAZ','1727','2','1'],
                ['MORENO','MARIANO ACOSTA','1723','2','1'],
                ['MORENO','MARIANO ACOSTA','1724','2','1'],
                ['MORENO','PASO DEL REY','1742','2','1'],
                ['MORENO','PONTEVEDRA','1761','2','1'],
                ['MORENO','TRUJUI','1664','2','1'],
                ['MORENO','TRUJUI','1736','2','1'],
                ['MORENO','VILLARS','1731','2','1'],
                ['MORON','CASTELAR','1710','2','1'],
                ['MORON','CASTELAR','1712','2','1'],
                ['MORON','DOMINGO FAUSTINO SARMIENTO','1707','2','1'],
                ['MORON','HAEDO','1706','2','1'],
                ['MORON','HURLINGHAM','1686','2','1'],
                ['MORON','ITUZAINGO','1714','2','1'],
                ['MORON','JOSE INGENIEROS','1703','2','1'],
                ['MORON','LIBERTAD','1716','2','1'],
                ['MORON','MERLO','1722','2','1'],
                ['MORON','MORON','1708','2','1'],
                ['MORON','SAN ANTONIO DE PADUA','1718','2','1'],
                ['MORON','VILLA GOBERNADOR UDAONDO','1713','2','1'],
                ['MORON','VILLA TESEY','1688','2','1'],
                ['PILAR','ARROYO DE LA CRUZ','2813','2','1'],
                ['PILAR','BARRIO EL CAZADOR','1626','2','1'],
                ['PILAR','CAPILLA DEL SEÑOR','2812','2','1'],
                ['PILAR','DEL VISO','1669','2','1'],
                ['PILAR','MANZONE','1633','2','1'],
                ['PILAR','PILAR','1629','2','1'],
                ['PILAR','PILAR','1630','2','1'],
                ['PILAR','PILAR','1634','2','1'],
                ['PILAR','PRESIDENTE DERQUI','1635','2','1'],
                ['PILAR','VILLA ROSA','1631','2','1'],
                ['SAN JUSTO','ALDO BONZI','1770','2','1'],
                ['SAN JUSTO','ALDO BONZI','1785','2','1'],
                ['SAN JUSTO','BARRIO EL ZORZAL','1793','2','1'],
                ['SAN JUSTO','CIUDAD EVITA','1778','2','1'],
                ['SAN JUSTO','ISIDRO CASNOVA','1756','2','1'],
                ['SAN JUSTO','LA SALADA','1774','2','1'],
                ['SAN JUSTO','LOMAS DEL MIRADOR','1752','2','1'],
                ['SAN JUSTO','RAMOS MEJIA','1704','2','1'],
                ['SAN JUSTO','SAN JUSTO','1754','2','1'],
                ['SAN JUSTO','TAPIALES','1770','2','1'],
                ['SAN JUSTO','VILLA INSUPERABLE','1751','2','1'],
                ['SAN JUSTO','VILLA MADERO','1768','2','1'],
                ['SAN JUSTO','VIRREY DEL PINO','1764','2','1'],
                ['SAN MARTIN','11 DE SEPTIEMBRE','1690','2','1'],
                ['SAN MARTIN','EL LIBERTADOR','1692','2','1'],
                ['SAN MARTIN','JOSE LEON SUAREZ','1655','2','1'],
                ['SAN MARTIN','LOMA HERMOSA','1657','2','1'],
                ['SAN MARTIN','MARTIN CORONADO','1682','2','1'],
                ['SAN MARTIN','MARTIN CORONADO','1683','2','1'],
                ['SAN MARTIN','PABLO PODESTA','1687','2','1'],
                ['SAN MARTIN','SAENZ PEÑA','1674','2','1'],
                ['SAN MARTIN','SAN ANDRES','1651','2','1'],
                ['SAN MARTIN','SAN MARTIN','1650','2','1'],
                ['SAN MARTIN','SAN MARTIN','1654','2','1'],
                ['SAN MARTIN','SANTOS LUGARES','1676','2','1'],
                ['SAN MARTIN','TRES DE FEBRERO','1652','2','1'],
                ['SAN MARTIN','VILLA BALLESTER','1653','2','1'],
                ['SAN MARTIN','VILLA BOSCH','1682','2','1'],
                ['SAN MIGUEL','ADOLFO SORDEAUX','1612','2','1'],
                ['SAN MIGUEL','BELLA VISTA','1661','2','1'],
                ['SAN MIGUEL','CAMPO DE MAYO','1659','2','1'],
                ['SAN MIGUEL','DON TORCUATO','1611','2','1'],
                ['SAN MIGUEL','JOSE C PAZ','1660','2','1'],
                ['SAN MIGUEL','JOSE C PAZ','1666','2','1'],
                ['SAN MIGUEL','JOSE C.PAZ','1665','2','1'],
                ['SAN MIGUEL','LOS POLVORINES','1613','2','1'],
                ['SAN MIGUEL','PABLO NOGUES','1616','2','1'],
                ['SAN MIGUEL','SAN MIGUEL','1663','2','1'],
                ['VICENTE LOPEZ','CARAPACHAY','1606','2','1'],
                ['VICENTE LOPEZ','ISLA MARTIN GARCIA','1601','2','1'],
                ['VICENTE LOPEZ','MUNRO','1605','2','1'],
                ['VICENTE LOPEZ','OLIVOS','1636','2','1'],
                ['VICENTE LOPEZ','OLIVOS','1637','2','1'],
                ['VICENTE LOPEZ','VICENTE LOPEZ','1638','2','1'],
                ['VICENTE LOPEZ','VILLA ADELINA','1607','2','1'],
                ['VICTORIA','ACASSUSO','1641','2','1'],
                ['VICTORIA','BECCAR','1643','2','1'],
                ['VICTORIA','DELTA SAN FERNANDO','1647','2','1'],
                ['VICTORIA','MARTINEZ','1639','2','1'],
                ['VICTORIA','MARTINEZ','1640','2','1'],
                ['VICTORIA','SAN FERNANDO','1646','2','1'],
                ['VICTORIA','TIGRE','1649','2','1'],
                ['VICTORIA','VICTORIA','1644','2','1'],
                ['VICTORIA','VIRREYES','1645','2','1'],
                ['BARILOCHE','CHOLILA','9217','4','2'],
                ['BARILOCHE','CUSHAMEN','8211','4','2'],
                ['BARILOCHE','CUSHAMEN','9211','4','2'],
                ['BARILOCHE','EL BOLSON','8430','4','2'],
                ['BARILOCHE','EL MAITEN','9210','4','2'],
                ['BARILOCHE','ESQUEL','9200','4','2'],
                ['BARILOCHE','GOBERNADOR COSTA','9223','4','2'],
                ['BARILOCHE','LELEQUE','9213','4','2'],
                ['BARILOCHE','MANUEL CHOIQUE','8415','4','2'],
                ['BARILOCHE','PARQUE NACIONAL LOS ALERCES','9201','4','2'],
                ['BARILOCHE','TREVELIN','9203','4','2'],
                ['BARILOCHE','DINA HUAPI','8404','15','2'],
                ['BARILOCHE','DINA HUAPI','8406','15','2'],
                ['BARILOCHE','JUNIN DE LOS ANDES','8371','15','2'],
                ['BARILOCHE','PAMPA DEL MALLEO','8373','15','2'],
                ['BARILOCHE','TRAFUL','8403','15','2'],
                ['BARILOCHE','VILLA LA ANGOSTURA','8407','15','2'],
                ['BARILOCHE','VILLA LOS COLIHUES','8405','15','2'],
                ['BARILOCHE','AGUADA DE GUERRA','8424','16','2'],
                ['BARILOCHE','CAÑADON CHILENO','8417','16','2'],
                ['BARILOCHE','CLEMENTE ONELLI','8416','16','2'],
                ['BARILOCHE','MAQUINCHAO','8422','16','2'],
                ['BARILOCHE','PILCANIYEU','8412','16','2'],
                ['BARILOCHE','BARILOCHE','8400','16','2'],
                ['COMODORO RIVADAVIA','ALDEA BELEIRO','9037','4','2'],
                ['COMODORO RIVADAVIA','BUEN PASTO','9023','4','2'],
                ['COMODORO RIVADAVIA','COLHUE HUAPI','9021','4','2'],
                ['COMODORO RIVADAVIA','DIADEMA ARGENTINA','9009','4','2'],
                ['COMODORO RIVADAVIA','DR.RICARDO ROJAS','9035','4','2'],
                ['COMODORO RIVADAVIA','LAGO BLANCO','9039','4','2'],
                ['COMODORO RIVADAVIA','SARMIENTO','9020','4','2'],
                ['COMODORO RIVADAVIA','BAJO CARACOLES','9315','20','2'],
                ['COMODORO RIVADAVIA','CAÑADON SECO','9013','20','2'],
                ['COMODORO RIVADAVIA','EL HUECO','9051','20','2'],
                ['COMODORO RIVADAVIA','JARAMILLO','9053','20','2'],
                ['COMODORO RIVADAVIA','KOLUEL KAYKE','9019','20','2'],
                ['COMODORO RIVADAVIA','LAS HERAS','9017','20','2'],
                ['COMODORO RIVADAVIA','LOS ANTIGUOS','9041','20','2'],
                ['COMODORO RIVADAVIA','PERITO MORENO','9040','20','2'],
                ['COMODORO RIVADAVIA','PUERTO DESEADO','9050','20','2'],
                ['GENERAL ROCA','ALEJANDRO STEFANELLI','8335','9','2'],
                ['GENERAL ROCA','CHICHINALES ','8337','9','2'],
                ['GENERAL ROCA','GENERAL FERNANDEZ','8325','9','2'],
                ['GENERAL ROCA','VILLA REGINA','8336','9','2'],
                ['GENERAL ROCA','CHOELE CHOEL','8360','15','2'],
                ['GENERAL ROCA','CERVANTES','8326','16','2'],
                ['GENERAL ROCA','CHELFORO','8366','16','2'],
                ['GENERAL ROCA','FRAY LUIS BELTRAN','8361','16','2'],
                ['GENERAL ROCA','GENERAL LORENZO VINTTER','8514','16','3'],
                ['GENERAL ROCA','GENERAL ROCA','8332','16','2'],
                ['GENERAL ROCA','LAMARQUE','8363','16','2'],
                ['GENERAL ROCA','CORONEL BELISLE','8364','16','3'],
                ['GENERAL ROCA','CORONEL JJ GOMEZ','8333','16','2'],
                ['GENERAL ROCA','POMONA','8363','16','3'],
                ['NUEQUEN','CATRIEL','8307','9','2'],
                ['NUEQUEN','COLONIA 25 DE MAYO','8201','9','3'],
                ['NUEQUEN','ALUMINE','8345','15','2'],
                ['NUEQUEN','BARDA DEL MEDIO','8305','15','2'],
                ['NUEQUEN','CAVIAHUE','8346','15','2'],
                ['NUEQUEN','CENTENARIO','8309','15','2'],
                ['NUEQUEN','CIPOLETTI','8327','15','2'],
                ['NUEQUEN','COLONIA VALENTINA','8301','15','2'],
                ['NUEQUEN','COPAHUE','8348','15','2'],
                ['NUEQUEN','CUTRAL CO','8322','15','2'],
                ['NUEQUEN','EL CHOLAR','8353','15','2'],
                ['NUEQUEN','EL HUECU','8349','15','2'],
                ['NUEQUEN','GENTE GRANDE','8379','15','2'],
                ['NUEQUEN','LAS COLORADAS','8341','15','2'],
                ['NUEQUEN','LAS LAJAS','8347','15','2'],
                ['NUEQUEN','MARIANO MORENO','8351','15','2'],
                ['NUEQUEN','MOQUEHUA','8344','15','2'],
                ['NUEQUEN','NUEQUEN','8300','15','2'],
                ['NUEQUEN','NUEQUEN','8302','15','2'],
                ['NUEQUEN','NUEQUEN','8304','15','2'],
                ['NUEQUEN','NUEQUEN','8375','15','2'],
                ['NUEQUEN','PASO AGUERRE','8313','15','2'],
                ['NUEQUEN','PIEDRA DEL AGUILA','8315','15','2'],
                ['NUEQUEN','PLAZA HUINCUL','8318','15','2'],
                ['NUEQUEN','PLOTIER','8316','15','2'],
                ['NUEQUEN','RINCON DE LOS SAUCES','8319','15','2'],
                ['NUEQUEN','SAN IGNACIO','8375','15','2'],
                ['NUEQUEN','SENILLOSA','8320','15','2'],
                ['NUEQUEN','ZAPALA','8340','15','2'],
                ['NUEQUEN','LONCOPUE','8349','15','3'],
                ['NUEQUEN','CHOSMALAL','8353','15','2'],
                ['NUEQUEN','CINCO SALTOS','8303','16','2'],
                ['NUEQUEN','GENERAL FERNANDEZ ORO','8324','16','2'],
                ['NUEQUEN','VILLA MANZANO','8308','16','2'],
                ['RIO GALLEGOS','CHILQUENILAHUE','9227','4','2'],
                ['RIO GALLEGOS','VALLE HONDO','9221','4','2'],
                ['RIO GALLEGOS','28 DE NOVIEMBRE','9408','20','2'],
                ['RIO GALLEGOS','BELLA VISTA','9401','20','2'],
                ['RIO GALLEGOS','CARA MALA','9313','20','2'],
                ['RIO GALLEGOS','COMANDANTE LUIS PIEDRABUENA','9303','20','2'],
                ['RIO GALLEGOS','GOBERNADRO GREGORES','9311','20','2'],
                ['RIO GALLEGOS','PUERTO SAN JULIAN','9310','20','2'],
                ['RIO GALLEGOS','PUERTO 20','9300','20','2'],
                ['RIO GALLEGOS','RIO GALLEGOS','9400','20','2'],
                ['RIO GALLEGOS','RIO GALLEGOS','9402','20','2'],
                ['RIO GALLEGOS','RIO TURBIO','9407','20','2'],
                ['RIO GALLEGOS','TRES LAGOS','9301','20','2'],
                ['RIO GRANDE','RIO GRANDE','9420','20','2'],
                ['RIO GRANDE','ANTARTIDA E ISLAS DEL SUR','9411','23','2'],
                ['RIO GRANDE','FRIGORIFICO CAP','9421','23','2'],
                ['TRELEW','ARROYO VERDE','9123','4','2'],
                ['TRELEW','CAMARONES','9111','4','2'],
                ['TRELEW','PASO DE INDIOS','9207','4','2'],
                ['TRELEW','PUENTE HENDRE','9101','4','2'],
                ['TRELEW','PUERTO PIRAMIDES','9121','4','2'],
                ['TRELEW','RAWSON','9103','4','2'],
                ['TRELEW','SIERRA GRANDE','8532','4','2'],
                ['TRELEW','TRELEW','9100','4','2'],
                ['TRELEW','TRELEW','9102','4','2'],
                ['USHUAIA','BARRIO SAN VICENTE DE PAUL','9412','23','2'],
                ['USHUAIA','PUERTO ARGENTINO','9409','23','2'],
                ['USHUAIA','USHUAIA','9410','23','2'],
                ['USHUAIA','USHUAIA','9413','23','2'],
                ['USHUAIA','RIO GRANDE','9410','23','2'],
                ['USHUAIA','TOLHUIN','9412','23','2'],
                ['BAHIA BLANCA','ALGARROBO','8136','2','2'],
                ['BAHIA BLANCA','APARICIO','8158','2','2'],
                ['BAHIA BLANCA','ARANO','6443','2','2'],
                ['BAHIA BLANCA','ARIAS','8101','2','2'],
                ['BAHIA BLANCA','ARROYO CORTO','8172','2','2'],
                ['BAHIA BLANCA','ARROYO VENADO','6437','2','2'],
                ['BAHIA BLANCA','ARTURO VATTEGNE','6433','2','2'],
                ['BAHIA BLANCA','AZOPARDO','8181','2','2'],
                ['BAHIA BLANCA','BAHIA BLANCA','8000','2','2'],
                ['BAHIA BLANCA','BAHIA BLANCA','8001','2','2'],
                ['BAHIA BLANCA','BAHIA BLANCA','8002','2','2'],
                ['BAHIA BLANCA','BAHIA BLANCA','8003','2','2'],
                ['BAHIA BLANCA','BAHIA BLANCA','8100','2','2'],
                ['BAHIA BLANCA','BAJO HONDO','8115','2','2'],
                ['BAHIA BLANCA','BALNEARIO CLAROMECO','7505','2','2'],
                ['BAHIA BLANCA','BONIFACIO','6439','2','2'],
                ['BAHIA BLANCA','BORDENAVE','8187','2','2'],
                ['BAHIA BLANCA','CABILDO','8118','2','2'],
                ['BAHIA BLANCA','CARHUE','6430','2','2'],
                ['BAHIA BLANCA','CASBAS','6417','2','2'],
                ['BAHIA BLANCA','CHASICO','8017','2','2'],
                ['BAHIA BLANCA','CHASICO','8117','2','2'],
                ['BAHIA BLANCA','COMANDANTE ESPORA','8107','2','2'],
                ['BAHIA BLANCA','COPETONAS','7511','2','2'],
                ['BAHIA BLANCA','CORONEL DORREGO','8150','2','2'],
                ['BAHIA BLANCA','CORONEL SUAREZ','7540','2','2'],
                ['BAHIA BLANCA','DARRAGUEIRA','8183','2','2'],
                ['BAHIA BLANCA','DE BARY','6348','2','2'],
                ['BAHIA BLANCA','DE LA GARMA','7515','2','2'],
                ['BAHIA BLANCA','EL PENSAMIENTO','7531','2','2'],
                ['BAHIA BLANCA','ESPARTILLAR','8171','2','2'],
                ['BAHIA BLANCA','ESTACION LAGO EPECUEY','6431','2','2'],
                ['BAHIA BLANCA','ESTEBAN A. GASCON','8185','2','2'],
                ['BAHIA BLANCA','GENERAL CERRI','8105','2','2'],
                ['BAHIA BLANCA','GIL','8151','2','2'],
                ['BAHIA BLANCA','GOYENA','8175','2','2'],
                ['BAHIA BLANCA','GUAMINI','6435','2','2'],
                ['BAHIA BLANCA','HILARIO ASCASUBI','8142','2','2'],
                ['BAHIA BLANCA','HUANGUELEN','7545','2','2'],
                ['BAHIA BLANCA','INDIO RICO','7501','2','2'],
                ['BAHIA BLANCA','INGENIERO THOMPSON','6337','2','2'],
                ['BAHIA BLANCA','JUAN A.GUISASOLA','8156','2','2'],
                ['BAHIA BLANCA','JUAN E.BARRA','7517','2','2'],
                ['BAHIA BLANCA','LA PRIMAVERA','7543','2','2'],
                ['BAHIA BLANCA','LA RESERVA','7536','2','2'],
                ['BAHIA BLANCA','MEDANOS','8132','2','2'],
                ['BAHIA BLANCA','MICAELA CASCALLARES','7507','2','2'],
                ['BAHIA BLANCA','MONTE HERMOSO','8153','2','2'],
                ['BAHIA BLANCA','PASMAN','7547','2','2'],
                ['BAHIA BLANCA','PEDRO LURO','8148','2','2'],
                ['BAHIA BLANCA','PIGUE','8170','2','2'],
                ['BAHIA BLANCA','PUERTO BELGRANO','8111','2','2'],
                ['BAHIA BLANCA','QUIÑIHUAL','7533','2','2'],
                ['BAHIA BLANCA','RIVADERO','8127','2','2'],
                ['BAHIA BLANCA','RIVERA','6441','2','2'],
                ['BAHIA BLANCA','SAAVEDRA','8174','2','2'],
                ['BAHIA BLANCA','SALDUNGARAY','8166','2','2'],
                ['BAHIA BLANCA','SALIQUELLO','6339','2','2'],
                ['BAHIA BLANCA','SAN GERMAN','8124','2','2'],
                ['BAHIA BLANCA','SAN JOSE','8136','2','2'],
                ['BAHIA BLANCA','SAN MAYOL','7519','2','2'],
                ['BAHIA BLANCA','SAN ROMAN','8154','2','2'],
                ['BAHIA BLANCA','SIERRA DE LA VENTANA','8168','2','2'],
                ['BAHIA BLANCA','SUNDBLAD','6401','2','2'],
                ['BAHIA BLANCA','TENIENTE ORIGONE','8144','2','2'],
                ['BAHIA BLANCA','TORQUINST','8160','2','2'],
                ['BAHIA BLANCA','TREINTA DE AGOSTO','6405','2','2'],
                ['BAHIA BLANCA','TRES ARROYOS','7500','2','2'],
                ['BAHIA BLANCA','TRES LOMAS','6409','2','2'],
                ['BAHIA BLANCA','TRES PICOS','8162','2','2'],
                ['BAHIA BLANCA','VILLA IRIS','8126','2','2'],
                ['BAHIA BLANCA','CAÑADON DE LOS MOGOTES','8184','3','2'],
                ['BAHIA BLANCA','JACINTO ARAUZ','8208','9','2'],
                ['BAHIA BLANCA','LA ROSA','8133','9','2'],
                ['BAHIA BLANCA','RIO COLORADO','8138','9','2'],
                ['AZUL','16 DE JULIO','7313','2','2'],
                ['AZUL','AZUL','7300','2','2'],
                ['AZUL','CACHARI','7214','2','2'],
                ['AZUL','MARTIN COLMAN','7201','2','2'],
                ['AZUL','GENERAL LAMADRID','7406','2','2'],
                ['AZUL','HINOJO','7318','2','2'],
                ['AZUL','LA COLINA','7408','2','2'],
                ['AZUL','LIBANO','7407','2','2'],
                ['AZUL','LOMA NEGRA','7403','2','2'],
                ['AZUL','LOMA NEGRA','7436','2','2'],
                ['AZUL','PARDO','7212','2','2'],
                ['AZUL','SAN JORGE','7404','2','2'],
                ['AZUL','SIERRAS BAYAS','7403','2','2'],
                ['AZUL','TAPALQUE','7303','2','2'],
                ['AZUL','VOLUNTAD','7412','2','2'],
                ['MAR DEL PLATA','CARI LARQUEA','7119','2','2'],
                ['MAR DEL PLATA','CASTELLI','7114','2','2'],
                ['MAR DEL PLATA','CEVIGNE','7101','2','2'],
                ['MAR DEL PLATA','CHAPADMALAL','7605','2','2'],
                ['MAR DEL PLATA','CHAPADMALAL','7619','2','2'],
                ['MAR DEL PLATA','COMANDANTE NICANOR OTAMENDI','7603','2','2'],
                ['MAR DEL PLATA','COOPER','7639','2','2'],
                ['MAR DEL PLATA','CORONEL VIDAL','7174','2','2'],
                ['MAR DEL PLATA','DOLORES','7100','2','2'],
                ['MAR DEL PLATA','ENERGIA','7640','2','2'],
                ['MAR DEL PLATA','ENERGIA','7641','2','2'],
                ['MAR DEL PLATA','GENERAL GUIDO','7118','2','2'],
                ['MAR DEL PLATA','JUAN N.FERNANDEZ','7011','2','2'],
                ['MAR DEL PLATA','JUANCHO','7169','2','2'],
                ['MAR DEL PLATA','LA DULCE','7637','2','2'],
                ['MAR DEL PLATA','LABARDEN','7161','2','2'],
                ['MAR DEL PLATA','LAS ARMAS','7172','2','2'],
                ['MAR DEL PLATA','LAS ARMAS','7176','2','2'],
                ['MAR DEL PLATA','LEZAMA','7116','2','2'],
                ['MAR DEL PLATA','MAR DEL PLATA','7600','2','2'],
                ['MAR DEL PLATA','MAR DEL PLATA','7602','2','2'],
                ['MAR DEL PLATA','MAR DEL PLATA','7604','2','2'],
                ['MAR DEL PLATA','MAR DEL PLATA','7606','2','2'],
                ['MAR DEL PLATA','MAR DEL PLATA','7608','2','2'],
                ['MAR DEL PLATA','MAR DEL PLATA','7610','2','2'],
                ['MAR DEL PLATA','MAR DEL PLATA','7611','2','2'],
                ['MAR DEL PLATA','MIRAMAR','7607','2','2'],
                ['MAR DEL PLATA','NAHUEL RUCA','7613','2','2'],
                ['MAR DEL PLATA','NECOCHEA','7630','2','2'],
                ['MAR DEL PLATA','NECOCHEA','7632','2','2'],
                ['MAR DEL PLATA','ORENSE','7503','2','2'],
                ['MAR DEL PLATA','PILA','7110','2','2'],
                ['MAR DEL PLATA','PILA','7116','2','2'],
                ['MAR DEL PLATA','QUEQUEN','7631','2','2'],
                ['MAR DEL PLATA','RAUCH','7203','2','3'],
                ['MAR DEL PLATA','SAN ALBERTO','7503','2','2'],
                ['MAR DEL PLATA','SANTA CLARA DEL MAR','7609','2','2'],
                ['TANDIL','AYACUCHO','7150','2','2'],
                ['TANDIL','BALCARCE','7620','2','3'],
                ['TANDIL','BARKER','7005','2','2'],
                ['TANDIL','BARKER','7033','2','2'],
                ['TANDIL','BENITO JUAREZ','7020','2','2'],
                ['TANDIL','CHILLAR','7311','2','2'],
                ['TANDIL','DE LA CANAL','7013','2','2'],
                ['TANDIL','EL TRIGO','7207','2','2'],
                ['TANDIL','IRAOLA','7009','2','2'],
                ['TANDIL','LA PASTORA','7001','2','2'],
                ['TANDIL','LAS FLORES','7200','2','2'],
                ['TANDIL','LOPEZ','7021','2','2'],
                ['TANDIL','NAPALEUFU','7007','2','2'],
                ['TANDIL','OLAVARRIA','7400','2','2'],
                ['TANDIL','OLAVARRIA','7436','2','2'],
                ['TANDIL','TANDIL','7000','2','2'],
                ['TANDIL','UDAQUIOLA','7151','2','2'],
                ['TANDIL','VELA','7003','2','2'],
                ['TANDIL','LOS PINOS','7623','2','3'],
                ['MENDOZA','12 DE OCTUBRE','5597','12','2'],
                ['MENDOZA','3 DE MAYO','5548','12','2'],
                ['MENDOZA','ALPATACAL','5591','12','2'],
                ['MENDOZA','BARRIO LENCINA','5564','12','2'],
                ['MENDOZA','BARRIO RIVADAVIA','5554','12','2'],
                ['MENDOZA','BARRIO SAN EDUARDO','5578','12','2'],
                ['MENDOZA','BUEN ORDEN','5670','12','2'],
                ['MENDOZA','CARRIZAL DE ABAJO','5556','12','2'],
                ['MENDOZA','CARRODILLA','5505','12','2'],
                ['MENDOZA','CARTELLONE','5658','12','2'],
                ['MENDOZA','CHACRAS DE CORIA','5505','12','2'],
                ['MENDOZA','CHACRAS DE CORIA','5528','12','2'],
                ['MENDOZA','CIUDAD DE JUNIN','5573','12','2'],
                ['MENDOZA','COLONIA CANO','5524','12','2'],
                ['MENDOZA','COLONIA LAMBARE','5653','12','2'],
                ['MENDOZA','COLONIA LAS ROSAS','5565','12','2'],
                ['MENDOZA','COLONIA LAS ROSAS','5662','12','2'],
                ['MENDOZA','COMPUERTAS','5550','12','2'],
                ['MENDOZA','CORDON DE PLATA','5661','12','2'],
                ['MENDOZA','CORONEL DORREGO','5626','12','2'],
                ['MENDOZA','COSTA DE ARAUJO','5535','12','2'],
                ['MENDOZA','CRUZ DE PIEDRA','5586','12','2'],
                ['MENDOZA','E PLUMERILLO','5541','12','2'],
                ['MENDOZA','EL ALTO SALVADOR','5654','12','2'],
                ['MENDOZA','EL BORBOLLON','5666','12','2'],
                ['MENDOZA','EL CENTRAL','5580','12','2'],
                ['MENDOZA','EL DIVISADERO','5583','12','2'],
                ['MENDOZA','EL MEDANITO','5675','12','2'],
                ['MENDOZA','EL MIRADOR','5568','12','2'],
                ['MENDOZA','EUGENIO  BUSTOS','5569','12','2'],
                ['MENDOZA','FRAY LUIS BELTRAN','5531','12','2'],
                ['MENDOZA','GENERAL BENEGAS','5544','12','2'],
                ['MENDOZA','GENERAL GUTIERREZ','5511','12','2'],
                ['MENDOZA','GODOY CRUZ','5501','12','2'],
                ['MENDOZA','INGENIERO GIAGNONI','5582','12','2'],
                ['MENDOZA','JESUS NAZARENO','5638','12','2'],
                ['MENDOZA','LA CENTRAL','5579','12','2'],
                ['MENDOZA','LA CENTRAL RIVADAVIA','5574','12','2'],
                ['MENDOZA','LA COLONIA','5566','12','2'],
                ['MENDOZA','LA COLONIA','5572','12','2'],
                ['MENDOZA','LA CONSULTA','5567','12','2'],
                ['MENDOZA','LA ISLA','5685','12','2'],
                ['MENDOZA','LA PAZ','5590','12','2'],
                ['MENDOZA','LAVALLE VILLA TULUMAYA','5664','12','2'],
                ['MENDOZA','LOS ARBOLES','5559','12','2'],
                ['MENDOZA','LOS ARBOLES','5575','12','2'],
                ['MENDOZA','LOS ARBOLES DE VILLEGAS','5563','12','2'],
                ['MENDOZA','LOS BARRIALES','5585','12','2'],
                ['MENDOZA','LOS BARRIALES','5668','12','2'],
                ['MENDOZA','LOS CAMPAMENTOS','5526','12','2'],
                ['MENDOZA','LOS CORRALITOS','5527','12','2'],
                ['MENDOZA','LOS SAUCES','5663','12','2'],
                ['MENDOZA','LUNLUTA','5593','12','2'],
                ['MENDOZA','MAIPU','5515','12','2'],
                ['MENDOZA','MEDANO DE ORO','5676','12','2'],
                ['MENDOZA','MEDRANO','5585','12','2'],
                ['MENDOZA','MENDOZA','5500','12','2'],
                ['MENDOZA','MENDOZA','5502','12','2'],
                ['MENDOZA','MENDOZA','5504','12','2'],
                ['MENDOZA','MINELLI','5532','12','2'],
                ['MENDOZA','MUNDO NUEVO','5536','12','2'],
                ['MENDOZA','MUNDO NUEVO ','5579','12','2'],
                ['MENDOZA','ÑACUÑAN','5595','12','2'],
                ['MENDOZA','PAREDITAS','5510','12','2'],
                ['MENDOZA','PEDREGAL','5651','12','2'],
                ['MENDOZA','PERDRIEL  ','5509','12','2'],
                ['MENDOZA','PHILIPPS','5538','12','2'],
                ['MENDOZA','PLUMERILLO','5542','12','2'],
                ['MENDOZA','POLVAREDA','5551','12','2'],
                ['MENDOZA','PRIMAVERA','5641','12','2'],
                ['MENDOZA','PUENTE DEL INCA','5555','12','2'],
                ['MENDOZA','PUNTA DE VACAS','5553','12','2'],
                ['MENDOZA','QUINTO CUARTEL','5674','12','2'],
                ['MENDOZA','RIVADAVIA','5577','12','2'],
                ['MENDOZA','RODEO DE LA CRUZ  ','5525','12','2'],
                ['MENDOZA','RODEO DEL MEDIO','5529','12','2'],
                ['MENDOZA','RUSSELL','5517','12','2'],
                ['MENDOZA','RUSSELL','5581','12','2'],
                ['MENDOZA','SAN CARLOS','5512','12','2'],
                ['MENDOZA','SAN JOSE','5530','12','2'],
                ['MENDOZA','SAN JOSE DE TUPUNGATO','5667','12','2'],
                ['MENDOZA','SAN JUAN BAUTISTA USNO','5682','12','2'],
                ['MENDOZA','SAN MARTIN','5570','12','2'],
                ['MENDOZA','SAN MIGUEL','5537','12','2'],
                ['MENDOZA','SANTA MARIA DE ORO','5576','12','2'],
                ['MENDOZA','SANTA ROSA','5596','12','2'],
                ['MENDOZA','SEGOVIA','5525','12','2'],
                ['MENDOZA','TRES DE MAYO','5543','12','2'],
                ['MENDOZA','TRES ESQUINAS','5514','12','2'],
                ['MENDOZA','TUNUYAN','5560','12','2'],
                ['MENDOZA','TUPUNGATO','5561','12','2'],
                ['MENDOZA','UGARTECHE','5509','12','2'],
                ['MENDOZA','UGARTECHE','5546','12','2'],
                ['MENDOZA','USPALLATA','5545','12','2'],
                ['MENDOZA','VILLA BASTIAS','5562','12','2'],
                ['MENDOZA','VILLA NUEVA DE GUAYMALLEN','5521','12','2'],
                ['MENDOZA','VILLA SECA','5588','12','2'],
                ['MENDOZA','VISTA FLORES','5665','12','2'],
                ['SAN JUAN','CHELCOS','5475','11','2'],
                ['SAN JUAN','CAMPO DE BATALLA','5435','12','2'],
                ['SAN JUAN','TUDCUM','5686','12','2'],
                ['SAN JUAN','ALBARDON','5419','18','2'],
                ['SAN JUAN','ALTO DE SIERRA','5438','18','2'],
                ['SAN JUAN','ASTICA','5447','18','2'],
                ['SAN JUAN','CALINGASTA','5403','18','2'],
                ['SAN JUAN','CAMPO AFUERA','5426','18','2'],
                ['SAN JUAN','CAÑADO HONDA','5431','18','2'],
                ['SAN JUAN','CAUCETE','5441','18','2'],
                ['SAN JUAN','CAUCETE','5442','18','2'],
                ['SAN JUAN','CHACRITAS','5456','18','2'],
                ['SAN JUAN','CHIMBAS','5413','18','2'],
                ['SAN JUAN','COLONIA ZAPATA','5436','18','2'],
                ['SAN JUAN','DIFUNTA CORREA','5443','18','2'],
                ['SAN JUAN','DOS ACEQUIAS','5439','18','2'],
                ['SAN JUAN','EL BOSQUE','5455','18','2'],
                ['SAN JUAN','EL FICAL','5461','18','2'],
                ['SAN JUAN','EL FICAL','5464','18','2'],
                ['SAN JUAN','ESTACION LA RINCONADA','5433','18','2'],
                ['SAN JUAN','IGLESIA','5467','18','2'],
                ['SAN JUAN','JACHAI','5460','18','2'],
                ['SAN JUAN','LA CAÑADA','5430','18','2'],
                ['SAN JUAN','LA CHIMBERA','5466','18','2'],
                ['SAN JUAN','LAS CHACRITAS','5417','18','2'],
                ['SAN JUAN','LAS FLORES','5458','18','2'],
                ['SAN JUAN','LAS TAPIAS','5440','18','2'],
                ['SAN JUAN','MARAYES','5446','18','2'],
                ['SAN JUAN','MARQUESADO','5407','18','2'],
                ['SAN JUAN','PAMPA DEL CHAÑAR','5468','18','2'],
                ['SAN JUAN','PAMPA VIEJA','5469','18','2'],
                ['SAN JUAN','PIE DE PALO','5444','18','2'],
                ['SAN JUAN','POCITO','5428','18','2'],
                ['SAN JUAN','PUNTA DEL MONTE','5454','18','2'],
                ['SAN JUAN','RAWSON','5420','18','2'],
                ['SAN JUAN','RAWSON','5423','18','2'],
                ['SAN JUAN','RINCONDADA','5429','18','2'],
                ['SAN JUAN','RODEO','5465','18','2'],
                ['SAN JUAN','S LUCIA','5411','18','2'],
                ['SAN JUAN','SAN ISIDRO','5416','18','2'],
                ['SAN JUAN','SAN MARTIN','5418','18','2'],
                ['SAN JUAN','ULLUM','5414','18','2'],
                ['SAN JUAN','ULLUN','5409','18','2'],
                ['SAN JUAN','VALLE  FERTIL','5449','18','2'],
                ['SAN JUAN','VILLA ABERASTAIN','5427','18','2'],
                ['SAN JUAN','VILLA KRAUSE','5425','18','2'],
                ['SAN JUAN','VILLA LERGA','5424','18','2'],
                ['SAN JUAN','VILLA PALERMO','5410','18','2'],
                ['SAN LUIS','BALDE DE ESCUDERO','5715','7','2'],
                ['SAN LUIS','EL CALDEN','5717','11','2'],
                ['SAN LUIS','AUTO GRANDE','5722','19','2'],
                ['SAN LUIS','BALDE','5724','19','2'],
                ['SAN LUIS','CANDELARIA','5713','19','2'],
                ['SAN LUIS','CAROLINA','5725','19','2'],
                ['SAN LUIS','CARPINTERIA','5883','19','2'],
                ['SAN LUIS','CARPINTERIA','5888','19','2'],
                ['SAN LUIS','CONCARAN','5770','19','2'],
                ['SAN LUIS','E CHORRILLO','5701','19','2'],
                ['SAN LUIS','ESTACION ZANJITAS','5721','19','2'],
                ['SAN LUIS','GENERAL URQUIZA','5763','19','2'],
                ['SAN LUIS','JUANA KOSLAY','5758','19','2'],
                ['SAN LUIS','LA FLORIDA','5726','19','2'],
                ['SAN LUIS','LA PETRA','5751','19','2'],
                ['SAN LUIS','LAFINUR','5872','19','2'],
                ['SAN LUIS','LAS CHACRAS','5752','19','2'],
                ['SAN LUIS','LAS CHACRAS','5880','19','2'],
                ['SAN LUIS','LOS CERRILLOS','5867','19','2'],
                ['SAN LUIS','LUJAN','5709','19','2'],
                ['SAN LUIS','MANANTIAL DE FLORES','5775','19','2'],
                ['SAN LUIS','MERLO','5881','19','2'],
                ['SAN LUIS','NASCHEL','5759','19','2'],
                ['SAN LUIS','NUEVA ESCOCIA','5743','19','2'],
                ['SAN LUIS','PUERTA COLORADA','5771','19','2'],
                ['SAN LUIS','QUEBRACHO LADEADO','5874','19','2'],
                ['SAN LUIS','QUINES','5711','19','2'],
                ['SAN LUIS','SAN FRANCISCO DEL MONTE DE ORO','5705','19','2'],
                ['SAN LUIS','SAN JERONIMO','5719','19','2'],
                ['SAN LUIS','SAN LORENZO','5757','19','2'],
                ['SAN LUIS','SAN LUIS','5700','19','2'],
                ['SAN LUIS','SAN LUIS','5702','19','2'],
                ['SAN LUIS','SAN LUIS','5710','19','2'],
                ['SAN LUIS','SAN LUIS','5780','19','2'],
                ['SAN LUIS','SAN LUIS','5886','19','2'],
                ['SAN LUIS','SAN MARTIN','5755','19','2'],
                ['SAN LUIS','SANTA ROSA DEL CONLARA','5777','19','2'],
                ['SAN LUIS','TILISARAO','5773','19','2'],
                ['SAN LUIS','VILLA DE LA QUEBRADA','5716','19','2'],
                ['SAN LUIS','VILLA DE PRAGA','5762','19','2'],
                ['SAN LUIS','VILLA DEL CARMEN','5835','19','2'],
                ['SAN RAFAEL','CARMENSA','5621','9','2'],
                ['SAN RAFAEL','25 DE MAYO','5615','12','2'],
                ['SAN RAFAEL','BOWEN','5634','12','2'],
                ['SAN RAFAEL','CANALEJAS','5636','12','2'],
                ['SAN RAFAEL','COLONIA SANTA TERESA','5649','12','2'],
                ['SAN RAFAEL','CORRAL DE LORCA','5637','12','2'],
                ['SAN RAFAEL','EL SOSNEADO','5611','12','2'],
                ['SAN RAFAEL','GENERAL ALVEAR','5620','12','2'],
                ['SAN RAFAEL','GENERAL ORTEGA','5606','12','2'],
                ['SAN RAFAEL','INGENIERO BALLOFFET','5630','12','2'],
                ['SAN RAFAEL','LAS MALVINAS','5635','12','2'],
                ['SAN RAFAEL','LLAVE NUEVA','5608','12','2'],
                ['SAN RAFAEL','LOS CAVAOS','5614','12','2'],
                ['SAN RAFAEL','LOS COMPARTOS','5648','12','2'],
                ['SAN RAFAEL','MALARGUE','5613','12','2'],
                ['SAN RAFAEL','MEDRANO','5633','12','2'],
                ['SAN RAFAEL','MONTE COMAN','5609','12','2'],
                ['SAN RAFAEL','PEDRO VARGAS','5603','12','2'],
                ['SAN RAFAEL','PEDRO VARGAS','5617','12','2'],
                ['SAN RAFAEL','PUEBLO ECHEVARRIETA','5618','12','2'],
                ['SAN RAFAEL','PUNTA DEL AGUA','5627','12','2'],
                ['SAN RAFAEL','RAMA CAIDA','5619','12','2'],
                ['SAN RAFAEL','REAL DEL PADRE','5624','12','2'],
                ['SAN RAFAEL','RESOLANA','5601','12','2'],
                ['SAN RAFAEL','SALTO DE LAS ROSAS','5603','12','2'],
                ['SAN RAFAEL','SAN RAFAEL','5600','12','2'],
                ['SAN RAFAEL','SAN RAFAEL','5602','12','2'],
                ['SAN RAFAEL','VALLE DE LAS LEÑAS','5612','12','2'],
                ['SAN RAFAEL','VILLA 25 DE MAYO','5625','12','2'],
                ['SAN RAFAEL','VILLA ATUEL','5622','12','2'],
                ['VILLA MERCEDES','JUSTO DARACT','5738','3','2'],
                ['VILLA MERCEDES','FRAGA','5736','SAN LUIS','2'],
                ['VILLA MERCEDES','JUAN JORBA','5731','SAN LUIS','2'],
                ['VILLA MERCEDES','JUAN LLERENA','5735','SAN LUIS','2'],
                ['VILLA MERCEDES','LA GAMA','5732','SAN LUIS','2'],
                ['VILLA MERCEDES','LA RIBERA','5734','SAN LUIS','2'],
                ['VILLA MERCEDES','VILLA MERCEDES','5730','SAN LUIS','2'],
                ['VILLA MERCEDES','VILLA REYNOLDS','5733','SAN LUIS','2'],
                ['CAÑADA DE GOMEZ','ARMSTRONG','2508','21','2'],
                ['CAÑADA DE GOMEZ','BERRETA','2501','21','2'],
                ['CAÑADA DE GOMEZ','CAÑADA DE GOMEZ','2500','21','2'],
                ['CAÑADA DE GOMEZ','CAÑADA ROSQUIN','2454','21','2'],
                ['CAÑADA DE GOMEZ','CARLOS PELLEGRINI','2453','21','2'],
                ['CAÑADA DE GOMEZ','CENTENO','2148','21','2'],
                ['CAÑADA DE GOMEZ','CORREA','2136','21','2'],
                ['CAÑADA DE GOMEZ','CORREA','2506','21','2'],
                ['CAÑADA DE GOMEZ','IBARLUCEA','2142','21','2'],
                ['CAÑADA DE GOMEZ','LAS PAREJAS','2505','21','2'],
                ['CAÑADA DE GOMEZ','LUCIO V. LOPEZ','2142','21','2'],
                ['CAÑADA DE GOMEZ','MARIA SUSANA','2527','21','2'],
                ['CAÑADA DE GOMEZ','MONTES DE OCA','2521','21','2'],
                ['CAÑADA DE GOMEZ','PIAMONTE','2529','21','2'],
                ['CAÑADA DE GOMEZ','ROLDAN','2134','21','2'],
                ['CAÑADA DE GOMEZ','SAN GENARO','2146','21','2'],
                ['CAÑADA DE GOMEZ','SAN GENARO','2147','21','2'],
                ['CAÑADA DE GOMEZ','SAN GENARO NORTE','2147','21','2'],
                ['CAÑADA DE GOMEZ','SAN JERONIMO SUD  ','2136','21','2'],
                ['CAÑADA DE GOMEZ','TOTORAS','2144','21','2'],
                ['CAÑADA DE GOMEZ','VILLA ELOISA','2503','21','2'],
                ['CASILDA','CAMILO ALDAO','2585','3','2'],
                ['CASILDA','CORRAL DE BUSTOS','2645','3','2'],
                ['CASILDA','LOS SURGENTES','2581','3','2'],
                ['CASILDA','OHIGGINS','2645','3','2'],
                ['CASILDA','ALBARELLOS','2101','21','2'],
                ['CASILDA','AREQUITO  ','2183','21','2'],
                ['CASILDA','ARTEAGA','2187','21','2'],
                ['CASILDA','BERABEVU','2639','21','2'],
                ['CASILDA','BOMBAL','2179','21','2'],
                ['CASILDA','CAFFERATA ','2643','21','2'],
                ['CASILDA','CARMEN DEL SAUCE','2109','21','3'],
                ['CASILDA','CASILDA','2170','21','2'],
                ['CASILDA','CHABAS','2173','21','2'],
                ['CASILDA','CHAÑAR LADEADO','2139','21','2'],
                ['CASILDA','CHAÑAR LADEADO','2643','21','2'],
                ['CASILDA','CORONEL DOMINGUEZ','2114','21','2'],
                ['CASILDA','ERASTO','2119','21','2'],
                ['CASILDA','FIRMAT','2630','21','2'],
                ['CASILDA','FUENTES','2123','21','2'],
                ['CASILDA','Godeken','2640','21','2'],
                ['CASILDA','LOS NOGALES','2183','21','2'],
                ['CASILDA','LOS NOGALES','2184','21','2'],
                ['CASILDA','LOS QUIRQUINCHOS','2637','21','2'],
                ['CASILDA','MAXIMO PAZ','2115','21','2'],
                ['CASILDA','PUEBLO MIGUEL TORRES','2631','21','2'],
                ['CASILDA','SAN JOSE DE LA ESQUINA','2185','21','2'],
                ['CASILDA','SAN JOSE DE LA ESQUINA','2186','21','2'],
                ['CASILDA','SANFORD','2173','21','2'],
                ['CASILDA','SANFORD','2174','21','2'],
                ['CASILDA','VILLA AMELIA','2102','21','2'],
                ['CASILDA','VILLADA','2173','21','2'],
                ['CASILDA','VILLADA','2176','21','2'],
                ['CASILDA','ZABALLA','2125','21','2'],
                ['CASILDA','CORONEL ARNOLD','2123','21','2'],
                ['CASILDA','CORONEL DOMINGUEZ','2105','21','2'],
                ['CASILDA','ESTACION BERNARD','2119','21','3'],
                ['CASILDA','MAIZALES','2119','21','3'],
                ['ROSARIO','MUTTI','2909','2','2'],
                ['ROSARIO','CAMILO ALDAO','2586','3','2'],
                ['ROSARIO','ALVAREZ','2107','21','2'],
                ['ROSARIO','CARMEN DEL SAUCE','2135','21','2'],
                ['ROSARIO','CASAS','2149','21','2'],
                ['ROSARIO','FIGHIERA  ','2126','21','2'],
                ['ROSARIO','FUNES','2132','21','2'],
                ['ROSARIO','LA SALADA','2141','21','2'],
                ['ROSARIO','LAS BANDURRIAS','2153','21','2'],
                ['ROSARIO','LOS QUIRQUINCHOS','2537','21','2'],
                ['ROSARIO','LUCIO V LOPEZ','2150','21','2'],
                ['ROSARIO','MARIA TERESA','2610','21','2'],
                ['ROSARIO','PEREZ','2121','21','2'],
                ['ROSARIO','PUEBLO ESTHER','2126','21','2'],
                ['ROSARIO','PUEBLO ESTHER','2129','21','2'],
                ['ROSARIO','ROSARIO','2000','21','2'],
                ['ROSARIO','ROSARIO','2001','21','2'],
                ['ROSARIO','ROSARIO','2002','21','2'],
                ['ROSARIO','ROSARIO','2003','21','2'],
                ['ROSARIO','ROSARIO','2004','21','2'],
                ['ROSARIO','ROSARIO','2005','21','2'],
                ['ROSARIO','ROSARIO','2006','21','2'],
                ['ROSARIO','ROSARIO','2007','21','2'],
                ['ROSARIO','ROSARIO','2008','21','2'],
                ['ROSARIO','ROSARIO','2009','21','2'],
                ['ROSARIO','ROSARIO','2010','21','2'],
                ['ROSARIO','ROSARIO','2011','21','2'],
                ['ROSARIO','ROSARIO','2012','21','2'],
                ['ROSARIO','ROSARIO','2013','21','2'],
                ['ROSARIO','ROSARIO','2014','21','2'],
                ['ROSARIO','ROSARIO','2015','21','2'],
                ['ROSARIO','ROSARIO','2108','21','2'],
                ['ROSARIO','ROSARIO','2127','21','2'],
                ['ROSARIO','ROSARIO','2130','21','2'],
                ['ROSARIO','ROSARIO','2131','21','2'],
                ['ROSARIO','ROSARIO','2143','21','2'],
                ['ROSARIO','SAN EUGENIO','2256','21','2'],
                ['ROSARIO','SOLDINI','2107','21','2'],
                ['ROSARIO','VILLA GOBERNADOR GALVEZ','2124','21','2'],
                ['SAN LORENZO','ALDAO','2214','21','2'],
                ['SAN LORENZO','AROCENA','2244','21','2'],
                ['SAN LORENZO','CLARKE','2218','21','2'],
                ['SAN LORENZO','COLONIA BELGRANO','2257','21','2'],
                ['SAN LORENZO','DESVIO ARIJON','2242','21','2'],
                ['SAN LORENZO','DIAZ','2222','21','2'],
                ['SAN LORENZO','GESSLER','2254','21','2'],
                ['SAN LORENZO','GOBERNADOR BAIGORRIA','2152','21','2'],
                ['SAN LORENZO','LOMA ALTA','2253','21','2'],
                ['SAN LORENZO','MACIEL','2208','21','2'],
                ['SAN LORENZO','MACIEL','2209','21','2'],
                ['SAN LORENZO','MONJE','2212','21','2'],
                ['SAN LORENZO','OLIVEROS','2206','21','2'],
                ['SAN LORENZO','PUERTO GENERAL SAN MARTIN','2202','21','2'],
                ['SAN LORENZO','SAN FABIAN','2243','21','2'],
                ['SAN LORENZO','SAN LORENZO','2200','21','2'],
                ['SAN LORENZO','SAN MARTIN DE LAS ESCOBAS','2449','21','2'],
                ['SAN LORENZO','SANTA CLARA DE BUENA VISTA','2258','21','2'],
                ['SAN LORENZO','SERODINO','2216','21','2'],
                ['SAN LORENZO','TIMBUES','2204','21','2'],
                ['SAN LORENZO','YRIGOYEN','2249','21','2'],
                ['VENADO TUERTO','FORTIN TIBURCIO','6024','2','2'],
                ['VENADO TUERTO','GERMANIA','6053','2','2'],
                ['VENADO TUERTO','IRIARTE','6042','2','2'],
                ['VENADO TUERTO','JUAN B.ALBERDI','6034','2','2'],
                ['VENADO TUERTO','JUNCAL','2723','2','2'],
                ['VENADO TUERTO','PEARSON','2711','2','2'],
                ['VENADO TUERTO','ARIAS','2624','3','2'],
                ['VENADO TUERTO','CAVANAGH','2625','3','2'],
                ['VENADO TUERTO','GUATIMOZIN','2627','3','2'],
                ['VENADO TUERTO','LEGUIZAMON','6128','3','2'],
                ['VENADO TUERTO','ROSSI','6128','3','2'],
                ['VENADO TUERTO','AARON CASTELLANOS','6106','21','2'],
                ['VENADO TUERTO','ALCORTA','2117','21','3'],
                ['VENADO TUERTO','AMENABAR','6103','21','2'],
                ['VENADO TUERTO','CARRERAS','2729','21','2'],
                ['VENADO TUERTO','CHAPUY','2603','21','2'],
                ['VENADO TUERTO','CHOVET','2633','21','2'],
                ['VENADO TUERTO','christophersen','6039','21','2'],
                ['VENADO TUERTO','DIEGO DE ALVEAR','6036','21','2'],
                ['VENADO TUERTO','ELORTONDO','2732','21','2'],
                ['VENADO TUERTO','LA CHISPA','2602','21','2'],
                ['VENADO TUERTO','LABORDEBOY','2726','21','2'],
                ['VENADO TUERTO','MAGGIOLO','2622','21','2'],
                ['VENADO TUERTO','MARIA TERESA','2609','21','3'],
                ['VENADO TUERTO','MELINCUE','2728','21','2'],
                ['VENADO TUERTO','RUFINO','6100','21','2'],
                ['VENADO TUERTO','SAN EDUARDO','2615','21','2'],
                ['VENADO TUERTO','SANCTI SPIRITU','2617','21','2'],
                ['VENADO TUERTO','SANTA ISABEL','2605','21','2'],
                ['VENADO TUERTO','TEODOLINA','6009','21','2'],
                ['VENADO TUERTO','VENADO TUERTO','2600','21','2'],
                ['VENADO TUERTO','WHEELWRIGHT','2722','21','3'],
                ['CONCORDIA','LOS PINOS','3272','2','3'],
                ['CONCORDIA','ALFONSO LOMA ','3471','6','2'],
                ['CONCORDIA','ALVEAR','3344','6','2'],
                ['CONCORDIA','BAIBIENE','3466','6','2'],
                ['CONCORDIA','CABRED','3232','6','2'],
                ['CONCORDIA','CHAVARRIA','3474','6','2'],
                ['CONCORDIA','CURUZU CUATIA','3460','6','2'],
                ['CONCORDIA','LA CRUZ','3346','6','2'],
                ['CONCORDIA','MERCEDES','3470','6','2'],
                ['CONCORDIA','MOCORETA','3226','6','2'],
                ['CONCORDIA','PERUGORRIA','3461','6','2'],
                ['CONCORDIA','SAN JAIME DE LA FRONTERA','3085','6','3'],
                ['CONCORDIA','SAN JAIME DE LA FRONTERA','3185','6','2'],
                ['CONCORDIA','SAUCE','3463','6','2'],
                ['CONCORDIA','BENITO LEGEREN','3203','8','2'],
                ['CONCORDIA','CHAJARI','3228','8','2'],
                ['CONCORDIA','COLON','3280','8','2'],
                ['CONCORDIA','COLONIA E.GOUCHON','3269','8','2'],
                ['CONCORDIA','COLONIA LA GLORIA','3204','8','2'],
                ['CONCORDIA','CONCORDIA','3200','8','2'],
                ['CONCORDIA','CONCORDIA','3202','8','2'],
                ['CONCORDIA','CONCORDIA','3205','8','2'],
                ['CONCORDIA','CONCORDIA','3207','8','2'],
                ['CONCORDIA','CRUCECITAS 8A SECCION','3175','8','2'],
                ['CONCORDIA','EL BRILLANTE','3284','8','2'],
                ['CONCORDIA','EL MARTILLO','3166','8','2'],
                ['CONCORDIA','GENERAL CAMPOS','3206','8','3'],
                ['CONCORDIA','GENERAL CAMPOS','3216','8','2'],
                ['CONCORDIA','LA CLARITA','3269','8','2'],
                ['CONCORDIA','LA CLARITA','3271','8','2'],
                ['CONCORDIA','LA CRIOLLA','3212','8','3'],
                ['CONCORDIA','LA CRIOLLA','3217','8','2'],
                ['CONCORDIA','LIEBIG','3281','8','2'],
                ['CONCORDIA','LIEBIG','3282','8','2'],
                ['CONCORDIA','LOS CHARRUAS','3212','8','2'],
                ['CONCORDIA','LOS CHARRUAS','3215','8','2'],
                ['CONCORDIA','LOS CONQUISTADORES','3083','8','3'],
                ['CONCORDIA','LOS CONQUISTADORES','3183','8','2'],
                ['CONCORDIA','MOREIRA','3181','8','2'],
                ['CONCORDIA','OSVALDO MAGÑASCO','3202','8','2'],
                ['CONCORDIA','OSVALDO MAGÑASCO','3212','8','2'],
                ['CONCORDIA','PARADA PUCHETA','3235','8','2'],
                ['CONCORDIA','PRONUNCIAMIENTO','3263','8','2'],
                ['CONCORDIA','PUEBLO COLORADO','3286','8','2'],
                ['CONCORDIA','SAN JOSE','3283','8','2'],
                ['CONCORDIA','SAN JOSE DE FELICIANO','3087','8','3'],
                ['CONCORDIA','SAN JOSE DE FELICIANO','3187','8','2'],
                ['CONCORDIA','SAN SALVADOR','3218','8','2'],
                ['CONCORDIA','SANTA ANA','3208','8','2'],
                ['CONCORDIA','UBAJAY','3287','8','2'],
                ['CONCORDIA','VILLA CLARA','3252','8','2'],
                ['CONCORDIA','VILLA ERRECABORDE','3250','8','2'],
                ['CONCORDIA','VILLA PERPER','3088','8','3'],
                ['CONCORDIA','VILLA PERPER','3188','8','2'],
                ['CONCORDIA','VILLA SAN MARCIAL','3248','8','2'],
                ['CONCORDIA','VILLAGUAY','3240','8','2'],
                ['CONCORDIA','VILLAGUAY ESTE','3244','8','2'],
                ['CONCORDIA','YUQUERI','3204','8','3'],
                ['CONCORDIA','YUQUERI','3214','8','2'],
                ['CONCORDIA','LAS TEJAS','3165','SANTIAGO DEL ESTERO','2'],
                ['CORRIENTES','ARROYITO','3448','6','2'],
                ['CORRIENTES','BARRIO VILLA 3','3442','6','2'],
                ['CORRIENTES','BERON DE ASTRADA','3481','6','2'],
                ['CORRIENTES','BUENA VISTA','3454','6','2'],
                ['CORRIENTES','CAPILLITA','3407','6','2'],
                ['CORRIENTES','CARRIZAL NORTE','3433','6','2'],
                ['CORRIENTES','COLONIA CARLOS PELLEGRINI','3473','6','2'],
                ['CORRIENTES','COLONIA DEL CARMEN','3451','6','2'],
                ['CORRIENTES','COLONIA PANDO','3449','6','2'],
                ['CORRIENTES','CONCEPCION','3423','6','2'],
                ['CORRIENTES','CORRIENTES','3400','6','2'],
                ['CORRIENTES','CORRIENTES','3402','6','2'],
                ['CORRIENTES','CORRIENTES','3404','6','2'],
                ['CORRIENTES','CORRIENTES','3408','6','2'],
                ['CORRIENTES','CORRIENTES','3410','6','2'],
                ['CORRIENTES','CRUZ DE LOS MILAGROS','3441','6','2'],
                ['CORRIENTES','EMPEDRADO','3418','6','2'],
                ['CORRIENTES','ENSENADA GRANDE','3412','6','2'],
                ['CORRIENTES','ESTACION SALADAS','3428','6','2'],
                ['CORRIENTES','GOBERNADOR MARTINEZ  ','3445','6','2'],
                ['CORRIENTES','GOBERNADOR.JUAN E MARTINEZ','3444','6','2'],
                ['CORRIENTES','GOYA','3450','6','2'],
                ['CORRIENTES','ITA IBATE','3480','6','2'],
                ['CORRIENTES','ITATI','3414','6','2'],
                ['CORRIENTES','LAGUNA BRAVA','3411','6','2'],
                ['CORRIENTES','LOMAS DE AGUIRRE','3405','6','2'],
                ['CORRIENTES','LORETO','3483','6','2'],
                ['CORRIENTES','MANANTIALES','3427','6','2'],
                ['CORRIENTES','PUESTO DE ISLA','3487','6','2'],
                ['CORRIENTES','RIACHUELO','3416','6','2'],
                ['CORRIENTES','SALADAS','3420','6','2'],
                ['CORRIENTES','SAN MIGUEL','3485','6','2'],
                ['CORRIENTES','SANTA ANA','3401','6','2'],
                ['CORRIENTES','SANTA LUCIA','3440','6','2'],
                ['CORRIENTES','SANTA LUCIA','3445','6','2'],
                ['CORRIENTES','SANTA ROSA','3421','6','2'],
                ['CORRIENTES','SANTA ROSA','3424','6','2'],
                ['CORRIENTES','TATACUA','3426','6','2'],
                ['CORRIENTES','TATACUA  ','3421','6','2'],
                ['CORRIENTES','VILLA OLIVARI','3486','6','2'],
                ['FORMOSA','BARRIO SARMIENTO','3699','CHACO','2'],
                ['FORMOSA','CENTRO FRONTERIZO CLORINDA','3642','CHACO','2'],
                ['FORMOSA','COMANDANTE FONTANA','3620','CHACO','2'],
                ['FORMOSA','FORTIN LUGONES','3621','CHACO','2'],
                ['FORMOSA','GATO COLORADO','3647','CHACO','2'],
                ['FORMOSA','LA EDUVIGES','3648','CHACO','2'],
                ['FORMOSA','LAGUNA YEMA','3634','CHACO','2'],
                ['FORMOSA','VILLA SARMIENTO','3657','CHACO','2'],
                ['FORMOSA','GARCETE CUE','3641','6','2'],
                ['FORMOSA','BRUCHARD','3622','FORMOSA','2'],
                ['FORMOSA','BUENA VISTA','3617','FORMOSA','2'],
                ['FORMOSA','CLORINDA','3610','FORMOSA','2'],
                ['FORMOSA','EL ESPINILLO','3615','FORMOSA','2'],
                ['FORMOSA','ESTANISLAO DEL CAMPO','3626','FORMOSA','2'],
                ['FORMOSA','FORMOSA','3600','FORMOSA','2'],
                ['FORMOSA','FORMOSA','3616','FORMOSA','2'],
                ['FORMOSA','GENERAL FRANCISCO BASILIANO BOSCH','3632','FORMOSA','2'],
                ['FORMOSA','GENERAL MOSCONI','3637','FORMOSA','2'],
                ['FORMOSA','HERRADURA','3601','FORMOSA','2'],
                ['FORMOSA','IBARRETA','3624','FORMOSA','2'],
                ['FORMOSA','LAGUNA NAICK NECK','3640','FORMOSA','2'],
                ['FORMOSA','MARIANO BOEDO','3604','FORMOSA','2'],
                ['FORMOSA','MISION TACAAGLE','3614','FORMOSA','2'],
                ['FORMOSA','PALO SANTO','3608','FORMOSA','2'],
                ['FORMOSA','PIRANE','3606','FORMOSA','2'],
                ['FORMOSA','POZO DEL TIGRE','3628','FORMOSA','2'],
                ['FORMOSA','PUERTO PILCOMAYO','3611','FORMOSA','2'],
                ['FORMOSA','PUERTO PILCOMAYO','3639','FORMOSA','2'],
                ['FORMOSA','SAN FRANCISCO DE LAISHI','3605','FORMOSA','2'],
                ['FORMOSA','SAN HILARIO','3618','FORMOSA','2'],
                ['FORMOSA','SAN MARTIN II','3627','FORMOSA','2'],
                ['FORMOSA','SIETE PALMAS','3612','FORMOSA','2'],
                ['FORMOSA','VILLA DOS TRECE','3602','FORMOSA','2'],
                ['FORMOSA','VILLA DOS TRECE','3603','FORMOSA','2'],
                ['FORMOSA','VILLA ESCOLAR','3548','FORMOSA','2'],
                ['FORMOSA','LANTERI','3576','21','2'],
                ['GUALEGUAYCHU','ALDEA ANTONIO','2827','8','2'],
                ['GUALEGUAYCHU','ALDEA ASUNCION','2841','8','2'],
                ['GUALEGUAYCHU','CASEROS','3262','8','2'],
                ['GUALEGUAYCHU','CEIBAS','2823','8','2'],
                ['GUALEGUAYCHU','COLONIA ELIA','3261','8','2'],
                ['GUALEGUAYCHU','COLONIA NUEVA MONTEVIDEO','2828','8','2'],
                ['GUALEGUAYCHU','CONCEPCION DEL URUGUAY','3260','8','2'],
                ['GUALEGUAYCHU','CONCEPCION DEL URUGUAY','3264','8','2'],
                ['GUALEGUAYCHU','ENRIQUE CARBO','2852','8','2'],
                ['GUALEGUAYCHU','GENERAL GALARZA','2843','8','2'],
                ['GUALEGUAYCHU','GLIBERTESCRIÑA  ','2828','8','2'],
                ['GUALEGUAYCHU','GOBERNADOR ECHAGUE','2847','8','2'],
                ['GUALEGUAYCHU','GOBERNADOR MANSILLA','2845','8','2'],
                ['GUALEGUAYCHU','GUALEGUAY','2840','8','3'],
                ['GUALEGUAYCHU','GUALEGUAYCHU','2820','8','2'],
                ['GUALEGUAYCHU','GUALEGUAYCHU','2822','8','2'],
                ['GUALEGUAYCHU','GUALEYAN','2821','8','2'],
                ['GUALEGUAYCHU','HERRERA','3272','8','2'],
                ['GUALEGUAYCHU','IBICUY','2846','8','2'],
                ['GUALEGUAYCHU','IRAZUSTA ESTACION','2851','8','2'],
                ['GUALEGUAYCHU','LAZO','2841','8','2'],
                ['GUALEGUAYCHU','MEDANOS','2848','8','2'],
                ['GUALEGUAYCHU','PERDICES','2823','8','2'],
                ['GUALEGUAYCHU','RINCON DEL GATO','2821','8','2'],
                ['GUALEGUAYCHU','ROCAMORA','3172','8','2'],
                ['GUALEGUAYCHU','ROSARIO DEL TALA','3174','8','2'],
                ['GUALEGUAYCHU','VILLA PARANACITO','2823','8','2'],
                ['GUALEGUAYCHU','VILLA MANTERO','3272','8','2'],
                ['GUALEGUAYCHU','VILLA SAN JUSTO','3262','8','2'],
                ['GUALEGUAYCHU','ALDEA SAN JUAN','2826','8','2'],
                ['GUALEGUAYCHU','ECHAGUE','2845','8','2'],
                ['PARANA','ESQUINA','3196','6','2'],
                ['PARANA','GUAYQUIRARO','3194','6','2'],
                ['PARANA','LIBERTADOR','3197','6','2'],
                ['PARANA','ALCARAZ','3138','8','2'],
                ['PARANA','ALDEA PROTESTANTE','3115','8','2'],
                ['PARANA','ALDEA SAN RAFAEL','3135','8','2'],
                ['PARANA','ALDEA SPATZENKUTTER','3101','8','2'],
                ['PARANA','ALDEA STA MARIA','3123','8','2'],
                ['PARANA','ARANGUREN','3162','8','2'],
                ['PARANA','BETBEDER  ','3156','8','2'],
                ['PARANA','BOVRIL','3142','8','2'],
                ['PARANA','COLIONIA MARIA LUISA','3114','8','2'],
                ['PARANA','COLONIA BERRO','3128','8','2'],
                ['PARANA','COLONIA CERRITO','3122','8','2'],
                ['PARANA','CRESPO','3116','8','2'],
                ['PARANA','DURAZNO','3178','8','2'],
                ['PARANA','EL PINGO','3132','8','2'],
                ['PARANA','EL RAMBLON','3109','8','2'],
                ['PARANA','EL SOLAR','3137','8','2'],
                ['PARANA','GENERAL RACEDO','3130','8','2'],
                ['PARANA','GENERAL RAMIREZ','3164','8','3'],
                ['PARANA','GUARDAMONTE','3179','8','2'],
                ['PARANA','HASENKAMP','3134','8','3'],
                ['PARANA','HERNANDARIAS','3127','8','3'],
                ['PARANA','HERNANDEZ','3156','8','2'],
                ['PARANA','LA PAZ','3190','8','2'],
                ['PARANA','LA PICADA','3118','8','2'],
                ['PARANA','LUCAS GONZALEZ','3145','8','2'],
                ['PARANA','LUCAS GONZALEZ','3158','8','2'],
                ['PARANA','NOGOYA','3150','8','3'],
                ['PARANA','PARANA','3100','8','2'],
                ['PARANA','PARANA','3102','8','2'],
                ['PARANA','PARANA','3104','8','2'],
                ['PARANA','PARANA','3106','8','2'],
                ['PARANA','PARANA','3108','8','2'],
                ['PARANA','PARANA','3141','8','2'],
                ['PARANA','PIEDRAS BLANCAS','3129','8','3'],
                ['PARANA','PUEBLO BELLOCQ','3136','8','2'],
                ['PARANA','PUEBLO BRUGO','3125','8','2'],
                ['PARANA','RAMON A.PARERA','3118','8','2'],
                ['PARANA','SAN BENITO','3107','8','2'],
                ['PARANA','SAN BENITO','3120','8','2'],
                ['PARANA','SAN GUSTAVO','3184','8','2'],
                ['PARANA','SAN GUSTAVO','3191','8','2'],
                ['PARANA','SAUCE','3152','8','2'],
                ['PARANA','SAUCE DE LUNA','3144','8','2'],
                ['PARANA','SEGUI','3117','8','2'],
                ['PARANA','SOLA','3176','8','3'],
                ['PARANA','TABOSSI','3111','8','2'],
                ['PARANA','TABOSSI','3124','8','2'],
                ['PARANA','VIALE  ','3109','8','2'],
                ['PARANA','VILLA FONTANA','3114','8','2'],
                ['PARANA','VILLA URQUIZA','3113','8','2'],
                ['PARANA','VILLA URQUIZA','3126','8','2'],
                ['PARANA','CERRITO','3122','8','2'],
                ['POSADAS','COLONIA GARABI','3342','6','3'],
                ['POSADAS','COLONIA GARABI','3343','6','2'],
                ['POSADAS','CORRIENTES','3406','6','2'],
                ['POSADAS','GOBERNADOR INGENIERO VALENTIN VIRASORO','3342','6','2'],
                ['POSADAS','ITUZAINGO','3302','6','2'],
                ['POSADAS','SAN CARLOS','3307','6','2'],
                ['POSADAS','SAN CARLOS','3345','6','2'],
                ['POSADAS','SAN JOSE','3306','6','2'],
                ['POSADAS','SAN JOSE','3307','6','2'],
                ['POSADAS','ALBA POSSE','3363','14','2'],
                ['POSADAS','ALMIRANTE BROWN','3385','14','2'],
                ['POSADAS','APOSTOLES','3350','14','2'],
                ['POSADAS','ARISTOBULO DEL VALLE','3364','14','2'],
                ['POSADAS','CABUREI','3371','14','2'],
                ['POSADAS','CAMPO GRANDE','3362','14','2'],
                ['POSADAS','CAMPO VIERA','3362','14','2'],
                ['POSADAS','CAMPO VIERA','3375','14','2'],
                ['POSADAS','CANDELARIA','3308','14','2'],
                ['POSADAS','CAPIOVY','3332','14','2'],
                ['POSADAS','CERRO CORA','3309','14','2'],
                ['POSADAS','CERRO CORA','3319','14','2'],
                ['POSADAS','COLONIA ALMAFUERTE','3336','14','2'],
                ['POSADAS','COLONIA CARAGUATAY','3386','14','2'],
                ['POSADAS','COLONIA DOMINGO SAVIO','3323','14','2'],
                ['POSADAS','COLONIA MARTIRES','3318','14','2'],
                ['POSADAS','COLONIA POLANA','3321','14','2'],
                ['POSADAS','COLONIA POLANA','3328','14','2'],
                ['POSADAS','COLONIA VICTORIA','3388','14','2'],
                ['POSADAS','CONCEPCION DE LA SIERRA','3355','14','2'],
                ['POSADAS','CORPUS','3327','14','2'],
                ['POSADAS','EL ALCAZAR','3384','14','2'],
                ['POSADAS','EL ALCAZAR','3390','14','2'],
                ['POSADAS','EL DORADO','3380','14','2'],
                ['POSADAS','ESTACION APOSTOLES','3359','14','2'],
                ['POSADAS','FRIGO','3337','14','2'],
                ['POSADAS','GARRUCHOS','3391','14','2'],
                ['POSADAS','GARUHAPE','3334','14','2'],
                ['POSADAS','GARUHAPE','3396','14','2'],
                ['POSADAS','GENERAL ALVEAR','3361','14','2'],
                ['POSADAS','GOBERNADOR ROCA','3324','14','2'],
                ['POSADAS','GUARANI','3361','14','3'],
                ['POSADAS','GUARANI','3369','14','2'],
                ['POSADAS','HIPOLITO YRIGOYEN','3328','14','2'],
                ['POSADAS','HIPOLITO YRIGOYEN','3335','14','2'],
                ['POSADAS','LEANDRO N ALEM','3315','14','2'],
                ['POSADAS','MARIANO MORENO','3338','14','2'],
                ['POSADAS','MBOPICUA','3331','14','2'],
                ['POSADAS','MIGUEL LANUS','3310','14','2'],
                ['POSADAS','MOJON GRANDE','3325','14','2'],
                ['POSADAS','OBERA','3360','14','2'],
                ['POSADAS','OBERA','3365','14','2'],
                ['POSADAS','OLEGARIO V.ANDRADE','3311','14','2'],
                ['POSADAS','PANAMBI','3361','14','2'],
                ['POSADAS','PANAMBI','3373','14','2'],
                ['POSADAS','PLAYADITO','3358','14','2'],
                ['POSADAS','POSADAS','3300','14','2'],
                ['POSADAS','POSADAS','3301','14','2'],
                ['POSADAS','POSADAS','3303','14','2'],
                ['POSADAS','POSADAS','3305','14','2'],
                ['POSADAS','POSADAS','3329','14','2'],
                ['POSADAS','POSADAS','3330','14','2'],
                ['POSADAS','PUERTO ESPAÑA','3320','14','2'],
                ['POSADAS','PUERTO ESPERANZA','3378','14','2'],
                ['POSADAS','PUERTO LEONI','3333','14','2'],
                ['POSADAS','PUERTO LIBERTAD','3374','14','2'],
                ['POSADAS','PUERTO LINARES','3382','14','2'],
                ['POSADAS','PUERTO PINARES','3389','14','2'],
                ['POSADAS','PUERTO PIRAY','3381','14','2'],
                ['POSADAS','PUERTO RICO','3334','14','2'],
                ['POSADAS','SALTO ENCANTADO','3339','14','2'],
                ['POSADAS','SAN ANTONIO','3366','14','2'],
                ['POSADAS','SAN ANTONIO','3383','14','2'],
                ['POSADAS','SAN JAVIER','3357','14','2'],
                ['POSADAS','SAN PEDRO','3352','14','2'],
                ['POSADAS','SAN VICENTE','3356','14','2'],
                ['POSADAS','SAN VICENTE','3364','14','2'],
                ['POSADAS','SANTA ANA','3316','14','2'],
                ['POSADAS','SANTA RITA','3363','14','2'],
                ['POSADAS','SANTA RITA','3377','14','2'],
                ['POSADAS','SANTO PIPPO','3326','14','2'],
                ['POSADAS','TRES CAPONES','3353','14','2'],
                ['POSADAS','TRES CAPONES','3354','14','2'],
                ['POSADAS','VILLA BONITA','3361','14','2'],
                ['POSADAS','VILLA BONITA','3368','14','2'],
                ['POSADAS','9 DE JULIO','3363','14','2'],
                ['POSADAS','9 DE JULIO','3380','14','3'],
                ['POSADAS','ACARAGUA','3361','14','3'],
                ['POSADAS','ALBERDI','3361','14','3'],
                ['POSADAS','ALICIA','3363','14','2'],
                ['POSADAS','AMEGHINO','3361','14','3'],
                ['POSADAS','ANDRESITO','3364','14','2'],
                ['POSADAS','BERNARDO DE IRIGOYEN','3366','14','3'],
                ['POSADAS','COLONIA AURORA','3363','14','2'],
                ['POSADAS','COLONIA OASIS','3328','14','2'],
                ['POSADAS','EL SOBERVIO','3364','14','3'],
                ['POSADAS','GENERAL URQUIZA','3326','14','2'],
                ['POSADAS','LOS HELECHOS','3361','14','2'],
                ['POSADAS','RUIZ DE MONTOYA','3332','14','2'],
                ['POSADAS','RUIZ DE MONTOYA','3334','14','2'],
                ['POSADAS','SAN FRANCISCO','3363','14','3'],
                ['POSADAS','SAN MARTIN','3361','14','2'],
                ['RAFAELA','BELLA ITALIA','2301','21','2'],
                ['RAFAELA','CAPIVA','2311','21','2'],
                ['RAFAELA','CASABLANCA','2317','21','2'],
                ['RAFAELA','COLONIA CASTELLANOS','2310','21','2'],
                ['RAFAELA','COLONIA CLARA','3025','21','2'],
                ['RAFAELA','EGUSQUIZA','2319','21','2'],
                ['RAFAELA','ELISA','3029','21','2'],
                ['RAFAELA','EUSEBIA','2317','21','2'],
                ['RAFAELA','EUSEBIA','2330','21','2'],
                ['RAFAELA','GALISTEO','2307','21','2'],
                ['RAFAELA','HIPATIA','3023','21','2'],
                ['RAFAELA','HUANQUEROS','3076','21','2'],
                ['RAFAELA','HUMBERTO PRIMERO','2309','21','2'],
                ['RAFAELA','LA CABRAL','3074','21','2'],
                ['RAFAELA','MANUCHO','3023','21','2'],
                ['RAFAELA','MOISES VILLE','2313','21','2'],
                ['RAFAELA','ÑANDUCITA','3072','21','2'],
                ['RAFAELA','PROGRESO','3023','21','2'],
                ['RAFAELA','PUEBLO SAN ANTONIO','2320','21','2'],
                ['RAFAELA','RAFAELA','2300','21','2'],
                ['RAFAELA','SAN CRISTOBAL','3070','21','2'],
                ['RAFAELA','SANTURCE','3074','21','2'],
                ['RAFAELA','SARMIENTO','3024','21','2'],
                ['RAFAELA','SOLEDAD','3025','21','2'],
                ['RAFAELA','SUSANA','2302','21','2'],
                ['RAFAELA','VILA','2306','21','2'],
                ['RAFAELA','VIRGINIA','2311','21','2'],
                ['RESISTENCIA','BARRANQUERAS ','3503','CHACO','2'],
                ['RESISTENCIA','CAMPO LARGO','3716','CHACO','2'],
                ['RESISTENCIA','CHARATA','3730','CHACO','2'],
                ['RESISTENCIA','CHOROTIS','3735','CHACO','2'],
                ['RESISTENCIA','COLONIAS UNIDAS','3515','CHACO','2'],
                ['RESISTENCIA','COLONIAS UNIDAS','3532','CHACO','2'],
                ['RESISTENCIA','CORZUELA','3718','CHACO','2'],
                ['RESISTENCIA','COTE LAI','3513','CHACO','2'],
                ['RESISTENCIA','EL SAUZALITO','3704','CHACO','2'],
                ['RESISTENCIA','ENRIQUE URIEN','3544','CHACO','2'],
                ['RESISTENCIA','FORTIN LAVALLE  ','3703','CHACO','2'],
                ['RESISTENCIA','GANCEDO','3734','CHACO','2'],
                ['RESISTENCIA','GENERAL VEDIA','3520','CHACO','2'],
                ['RESISTENCIA','GENERAL VEDIA','3522','CHACO','2'],
                ['RESISTENCIA','GRAL PINEDO','3732','CHACO','2'],
                ['RESISTENCIA','GRAL SAN MARTIN','3509','CHACO','2'],
                ['RESISTENCIA','ITIN','3733','CHACO','2'],
                ['RESISTENCIA','JUAN JOSE CASTELLI','3705','CHACO','2'],
                ['RESISTENCIA','LA CLOTILDE','3725','CHACO','2'],
                ['RESISTENCIA','LA EDUVIGES','3507','CHACO','2'],
                ['RESISTENCIA','LA LEONESA','3514','CHACO','2'],
                ['RESISTENCIA','LA LEONESA','3518','CHACO','2'],
                ['RESISTENCIA','LA VERDE','3527','CHACO','2'],
                ['RESISTENCIA','LAS BREÑAS','3722','CHACO','2'],
                ['RESISTENCIA','LAS BREÑAS','3724','CHACO','2'],
                ['RESISTENCIA','LAS GARCITAS','3533','CHACO','2'],
                ['RESISTENCIA','LAS PALMAS','3523','CHACO','2'],
                ['RESISTENCIA','MAKALLE','3528','CHACO','2'],
                ['RESISTENCIA','MESON DE FIERRO','3729','CHACO','2'],
                ['RESISTENCIA','NAPENAY','3706','CHACO','2'],
                ['RESISTENCIA','PAMPA DEL INDIO','3531','CHACO','2'],
                ['RESISTENCIA','PAMPA DEL INFIERNO','3708','CHACO','2'],
                ['RESISTENCIA','PAMPA DEL INFIERNO','3721','CHACO','2'],
                ['RESISTENCIA','PAMPA SOMMER','3728','CHACO','2'],
                ['RESISTENCIA','PRESIDENCIA DE LA PLAZA','3536','CHACO','2'],
                ['RESISTENCIA','PRESIDENTE ROCA','3511','CHACO','2'],
                ['RESISTENCIA','PRESIDENTE ROQUE SAENZ PEÑA','3700','CHACO','2'],
                ['RESISTENCIA','PUERTO BERMEJO','3524','CHACO','2'],
                ['RESISTENCIA','PUERTO TIROL ','3505','CHACO','2'],
                ['RESISTENCIA','SAN BERNARDO','3701','CHACO','2'],
                ['RESISTENCIA','SAN BERNARDO','3726','CHACO','2'],
                ['RESISTENCIA','TRES ESTACAS','3727','CHACO','2'],
                ['RESISTENCIA','TRES ISLETAS','3703','CHACO','2'],
                ['RESISTENCIA','VILLA ANGELA','3540','CHACO','2'],
                ['RESISTENCIA','VILLA BERTHET','3545','CHACO','2'],
                ['RESISTENCIA','WICHI','3702','CHACO','2'],
                ['RESISTENCIA','HERMOSO CAMPO','3733','CHACO','3'],
                ['RESISTENCIA','GENERAL CAPDEVILA','3732','CHACO','3'],
                ['RESISTENCIA','LA TIGRA','3701','CHACO','3'],
                ['RESISTENCIA','AVIA TERAI','3706','CHACO','3'],
                ['RESISTENCIA','CONCEPCION DEL BERMEJO','3708','CHACO','3'],
                ['RESISTENCIA','CORONEL DUGRATY','3541','CHACO','2'],
                ['RESISTENCIA','MIRAFLORES','3705','CHACO','3'],
                ['RESISTENCIA','MISION NUEVA POMPEYA','3705','CHACO','3'],
                ['RESISTENCIA','FONTANA','3514','CHACO','2'],
                ['RESISTENCIA','VILELAS','3753','SANTIAGO DEL ESTERO','2'],
                ['SANTA FE','ANGEL GALLARDO','3014','21','2'],
                ['SANTA FE','ANTONIO PINI','3061','21','2'],
                ['SANTA FE','ARROYO AGUIAR','3031','21','2'],
                ['SANTA FE','CALCHAQUI','3050','21','2'],
                ['SANTA FE','CAMPO ANDINO','3021','21','2'],
                ['SANTA FE','CANDIOTI','2469','21','2'],
                ['SANTA FE','CANDIOTI','3018','21','2'],
                ['SANTA FE','CAVOUR','3081','21','2'],
                ['SANTA FE','CAVOUR','3082','21','2'],
                ['SANTA FE','CAYASTACITO','3038','21','2'],
                ['SANTA FE','CERES','2340','21','2'],
                ['SANTA FE','CORONDA','2240','21','2'],
                ['SANTA FE','EMILIA','3036','21','2'],
                ['SANTA FE','EMILIA','3092','21','2'],
                ['SANTA FE','EMPALME SAN CARLOS','3007','21','2'],
                ['SANTA FE','ESPERANZA','3080','21','2'],
                ['SANTA FE','FELICIA','3087','21','2'],
                ['SANTA FE','FORTIN OLMOS','3562','21','2'],
                ['SANTA FE','GOBERNADOR CRESPO','3044','21','2'],
                ['SANTA FE','GRUTLY','3083','21','2'],
                ['SANTA FE','HIPATIA','3049','21','2'],
                ['SANTA FE','INDEPENDENCIA','3060','21','2'],
                ['SANTA FE','INTIYACO','3552','21','2'],
                ['SANTA FE','KILOMETRO 302','3097','21','2'],
                ['SANTA FE','LAGUNA PAIVA','3020','21','2'],
                ['SANTA FE','LOS LAURELES','3584','21','2'],
                ['SANTA FE','LOS SALADILLOS','3041','21','2'],
                ['SANTA FE','MARCELINO ESCALADA','3042','21','2'],
                ['SANTA FE','MARIA LUISA CORREA','2502','21','2'],
                ['SANTA FE','MARIANO SAAVEDRA','3039','21','2'],
                ['SANTA FE','NELSON','3032','SANTA FE','2'],
                ['SANTA FE','NUEVO TORINO','3088','21','2'],
                ['SANTA FE','PETRONILLA','3046','21','2'],
                ['SANTA FE','PILAR','3085','21','2'],
                ['SANTA FE','POZO BORRADO','3067','21','2'],
                ['SANTA FE','RAMAYON','3042','21','2'],
                ['SANTA FE','RAMAYON','3094','21','2'],
                ['SANTA FE','RECREO','3018','21','2'],
                ['SANTA FE','SAN CARLOS','3013','21','2'],
                ['SANTA FE','SAN CARLOS NORTE','3009','21','2'],
                ['SANTA FE','SAN GERONIMO NORTE','3011','21','2'],
                ['SANTA FE','SAN JAVIER','3005','21','2'],
                ['SANTA FE','SAN JAVIER','3036','21','3'],
                ['SANTA FE','SANTA FE','3000','21','2'],
                ['SANTA FE','SANTA FE','3002','21','2'],
                ['SANTA FE','SANTA FE','3004','21','2'],
                ['SANTA FE','SANTA FE','3006','21','2'],
                ['SANTA FE','SANTA FE','3008','21','2'],
                ['SANTA FE','SANTA FE','3012','21','2'],
                ['SANTA FE','SANTA FE','3015','21','2'],
                ['SANTA FE','SANTA MARGARITA','3061','21','2'],
                ['SANTA FE','SANTA MARGARITA','3068','21','2'],
                ['SANTA FE','SANTA ROSA','3022','21','2'],
                ['SANTA FE','SANTO TOME','3016','21','2'],
                ['SANTA FE','SAUCE VIEJO','3017','21','2'],
                ['SANTA FE','SAUCE VIEJO','3034','21','2'],
                ['SANTA FE','TACUARENDI','3588','21','2'],
                ['SANTA FE','TOSTADO','3060','21','2'],
                ['SANTA FE','VERA','3550','21','2'],
                ['SANTA FE','VERA Y PINTADO','3054','21','2'],
                ['SANTA FE','VILLA MINETTI','3061','21','2'],
                ['SANTA FE','VUELTA DEL PIRATA','3058','21','2'],
                ['SANTA FE','FORTIN INCA','3063','SANTIAGO DEL ESTERO','2'],
                ['SANTA FE','MALBRAN','2355','SANTIAGO DEL ESTERO','3'],
                ['SANTA FE','UNION','2357','SANTIAGO DEL ESTERO','2'],
                ['SANTA FE','PALO NEGRO','2354','SANTIAGO DEL ESTERO','3'],
                ['CORDOBA','AGUA DEL TALA','5243','3','2'],
                ['CORDOBA','ALTO GRANDE','5893','3','2'],
                ['CORDOBA','AMBUL','5299','3','2'],
                ['CORDOBA','ARROYITO','2434','3','2'],
                ['CORDOBA','BAÑADO DE SOTO','5285','3','2'],
                ['CORDOBA','CALCHIN OESTE','5964','3','2'],
                ['CORDOBA','CALCHIN OESTE','5965','3','2'],
                ['CORDOBA','CAMINIAGA','5245','3','2'],
                ['CORDOBA','CANDELARIA NORTE','5235','3','2'],
                ['CORDOBA','CANTERAS DE QUILPO','5281','3','2'],
                ['CORDOBA','CAÑADA DE LUQUE','5229','3','2'],
                ['CORDOBA','CAPILLA DE SITON','5231','3','2'],
                ['CORDOBA','CASA GRANDE','5162','3','2'],
                ['CORDOBA','CHARBONIER','5282','3','2'],
                ['CORDOBA','CHURQUI','5246','3','2'],
                ['CORDOBA','COLAZO','5971','3','2'],
                ['CORDOBA','COLAZO','5981','3','2'],
                ['CORDOBA','COLONIA CAROYA','5223','3','2'],
                ['CORDOBA','COMECHINGONES','5129','3','2'],
                ['CORDOBA','CONLARA','5873','3','2'],
                ['CORDOBA','COPACABANA','5201','3','2'],
                ['CORDOBA','CORDOBA','5000','3','2'],
                ['CORDOBA','CORDOBA','5001','3','2'],
                ['CORDOBA','CORDOBA','5002','3','2'],
                ['CORDOBA','CORDOBA','5003','3','2'],
                ['CORDOBA','CORDOBA','5004','3','2'],
                ['CORDOBA','CORDOBA','5005','3','2'],
                ['CORDOBA','CORDOBA','5006','3','2'],
                ['CORDOBA','CORDOBA','5007','3','2'],
                ['CORDOBA','CORDOBA','5008','3','2'],
                ['CORDOBA','CORDOBA','5010','3','2'],
                ['CORDOBA','CORDOBA','5011','3','2'],
                ['CORDOBA','CORDOBA','5012','3','2'],
                ['CORDOBA','CORDOBA','5013','3','2'],
                ['CORDOBA','CORDOBA','5014','3','2'],
                ['CORDOBA','CORDOBA','5015','3','2'],
                ['CORDOBA','CORDOBA','5016','3','2'],
                ['CORDOBA','CORDOBA','5017','3','2'],
                ['CORDOBA','CORDOBA','5018','3','2'],
                ['CORDOBA','CORDOBA','5019','3','2'],
                ['CORDOBA','CORDOBA','5020','3','2'],
                ['CORDOBA','CORDOBA','5023','3','2'],
                ['CORDOBA','CORDOBA','5024','3','2'],
                ['CORDOBA','CORDOBA','5103','3','2'],
                ['CORDOBA','CORONEL OLMEDO ','5119','3','3'],
                ['CORDOBA','CRUZ DE CAÑA','5287','3','3'],
                ['CORDOBA','DEAN FUNES','5200','3','2'],
                ['CORDOBA','DIEGO DE ROJAS','5136','3','2'],
                ['CORDOBA','EL CERRITO','5205','3','2'],
                ['CORDOBA','EL MANZANO','5114','3','2'],
                ['CORDOBA','EL QUEBRACHAL','5101','3','2'],
                ['CORDOBA','ESQUINA','5131','3','2'],
                ['CORDOBA','EXPEDICIÑN CO','5100','3','2'],
                ['CORDOBA','FERREYRA','5123','3','2'],
                ['CORDOBA','GUIÑAZU  ','5145','3','2'],
                ['CORDOBA','ISLA VERDE','2660','3','2'],
                ['CORDOBA','JAMES CRAIK','5984','3','2'],
                ['CORDOBA','JESUS MARIA','5220','3','2'],
                ['CORDOBA','JUAREZ CELMAN','5145','3','2'],
                ['CORDOBA','LA AGUADA','5211','3','2'],
                ['CORDOBA','LA BOLSA','5689','3','2'],
                ['CORDOBA','LA COYUNDA  ','2435','3','2'],
                ['CORDOBA','LA LAGUNA','5283','3','2'],
                ['CORDOBA','LA PALMA','5299','3','2'],
                ['CORDOBA','LA PARA','5137','3','2'],
                ['CORDOBA','LA PAZ','5879','3','3'],
                ['CORDOBA','LA POBLACION','5875','3','3'],
                ['CORDOBA','LA POSTA','5227','3','2'],
                ['CORDOBA','LA PUERTA','5102','3','2'],
                ['CORDOBA','LA PUERTA','5138','3','2'],
                ['CORDOBA','LA PUERTA  ','5137','3','2'],
                ['CORDOBA','LA TORDILLA','2435','3','2'],
                ['CORDOBA','LAGUNA LARGA','5974','3','2'],
                ['CORDOBA','LAS AVERIAS','5135','3','2'],
                ['CORDOBA','LAS PEÑAS','5238','3','2'],
                ['CORDOBA','LAS RABONAS','5885','3','3'],
                ['CORDOBA','LAS TAPIAS','5885','3','3'],
                ['CORDOBA','LOS ALAMOS ','5244','3','2'],
                ['CORDOBA','LOS HORNILLOS','5885','3','3'],
                ['CORDOBA','LOS HOYOS','5249','3','2'],
                ['CORDOBA','LOZADA','5104','3','2'],
                ['CORDOBA','MALAGUEÑO','5101','3','2'],
                ['CORDOBA','MALAGUEÑO','5110','3','2'],
                ['CORDOBA','MANFREDI','5988','3','2'],
                ['CORDOBA','MARULL','5139','3','2'],
                ['CORDOBA','MATORRALES','5970','3','2'],
                ['CORDOBA','MAYU SUMAJ','5146','3','2'],
                ['CORDOBA','MI GRANJA','5128','3','2'],
                ['CORDOBA','MINA CLAVERO','5889','3','2'],
                ['CORDOBA','MONTE CRISTO  ','5125','3','2'],
                ['CORDOBA','MONTE RALO','5118','3','2'],
                ['CORDOBA','MONTE RALO','5119','3','3'],
                ['CORDOBA','NONO','5887','3','2'],
                ['CORDOBA','OLIVA','5980','3','2'],
                ['CORDOBA','ONCATIVO','5986','3','2'],
                ['CORDOBA','PANAHOLMA','5893','3','2'],
                ['CORDOBA','PASO VIEJO','5284','3','2'],
                ['CORDOBA','PIEDRA PINTADA','5871','3','3'],
                ['CORDOBA','PILAR','5972','3','2'],
                ['CORDOBA','PIQUILLIN','5130','3','2'],
                ['CORDOBA','POCHO','5294','3','2'],
                ['CORDOBA','PUESTO DE VERA','5271','3','2'],
                ['CORDOBA','QUILINO','5214','3','2'],
                ['CORDOBA','SAN CARLOS MINAS','5291','3','2'],
                ['CORDOBA','SAN ESTEBAN','5188','3','2'],
                ['CORDOBA','SAN JOSE','5865','3','3'],
                ['CORDOBA','SAN JOSE','5871','3','3'],
                ['CORDOBA','SAN JOSE DE LA DORMIDA','5244','3','2'],
                ['CORDOBA','SANTA ANA','5112','3','2'],
                ['CORDOBA','SANTA ROSA DEL RIO 1ï¾°','5133','3','2'],
                ['CORDOBA','SANTA TERESA','5221','3','2'],
                ['CORDOBA','SERREZUELA','5270','3','2'],
                ['CORDOBA','SIMBOLAR','5242','3','2'],
                ['CORDOBA','SINSACATE','5221','3','2'],
                ['CORDOBA','TAJAMARES','5233','3','2'],
                ['CORDOBA','TALA CAÑADA','5297','3','2'],
                ['CORDOBA','TRANSITO','2436','3','2'],
                ['CORDOBA','TUCLAME','5286','3','2'],
                ['CORDOBA','TULUMBA','5203','3','2'],
                ['CORDOBA','VILLA ANISACATE','5167','3','2'],
                ['CORDOBA','VILLA CURA BROCHERO','5891','3','2'],
                ['CORDOBA','VILLA DE SOTO','5284','3','2'],
                ['CORDOBA','VILLA DEL ROSARIO','5963','3','2'],
                ['CORDOBA','VILLA DEL TOTORAL','5236','3','2'],
                ['CORDOBA','VILLA DOLORES','5870','3','3'],
                ['CORDOBA','VILLA EL DESCANSO','5169','3','2'],
                ['CORDOBA','VILLA FONTANA','5134','3','2'],
                ['CORDOBA','VILLA LA BOLSA','5170','3','2'],
                ['CORDOBA','VILLA M.DEL RIO SECO','5248','3','2'],
                ['CORDOBA','VILLA QUILINO','5215','3','2'],
                ['CORDOBA','VILLA RIO YCHO CRUZ','5159','3','2'],
                ['CORDOBA','YACANTO','5877','3','3'],
                ['CORDOBA','YCHO CRUZ SIERRAS','5161','3','2'],
                ['CORDOBA','BAZAN','5306','11','2'],
                ['CORDOBA','SANTA CLARA','5157','SAN LUIS','2'],
                ['RIO CUARTO','LA CESIRA','6101','2','2'],
                ['RIO CUARTO','VILLA SABOYA','6101','2','2'],
                ['RIO CUARTO','VILLA SABOYA','6102','2','2'],
                ['RIO CUARTO','ACHIRAS','5833','3','2'],
                ['RIO CUARTO','ADELIA MARIA','5843','3','2'],
                ['RIO CUARTO','ALCIRA','5813','3','2'],
                ['RIO CUARTO','ALEJANDRO','2686','3','2'],
                ['RIO CUARTO','ASSUNTA','2671','3','2'],
                ['RIO CUARTO','BENJAMIN GOULD','2664','3','2'],
                ['RIO CUARTO','CANALS','2650','3','2'],
                ['RIO CUARTO','CARNERILLO','5805','3','2'],
                ['RIO CUARTO','CHARRAS','5807','3','2'],
                ['RIO CUARTO','CHARRAS','5814','3','2'],
                ['RIO CUARTO','COLONIA BISMAR','2651','3','2'],
                ['RIO CUARTO','COLONIA BREMEN','2651','3','2'],
                ['RIO CUARTO','COLONIA ITALIANA','2646','3','2'],
                ['RIO CUARTO','COLONIA MAIPU','2685','3','2'],
                ['RIO CUARTO','CORONEL BAIGORRIA','5811','3','2'],
                ['RIO CUARTO','CORONEL MOLDES','5847','3','2'],
                ['RIO CUARTO','EL RASTREADOR','6121','3','2'],
                ['RIO CUARTO','ELENA','5815','3','2'],
                ['RIO CUARTO','ESTACION ACHIRAS','5831','3','2'],
                ['RIO CUARTO','GENERAL LEVALLE','6132','3','2'],
                ['RIO CUARTO','HIPOLITO BOUCHARD','6225','3','2'],
                ['RIO CUARTO','HOLMBERG','5825','3','2'],
                ['RIO CUARTO','HUANCHILLA','6121','3','2'],
                ['RIO CUARTO','HUINCA RENANCO','6270','3','2'],
                ['RIO CUARTO','JOVITA','6127','3','2'],
                ['RIO CUARTO','L JAGUELES','5839','3','2'],
                ['RIO CUARTO','LA CALERA CALAMUCHIT','5819','3','2'],
                ['RIO CUARTO','LA CARLOTA','2670','3','2'],
                ['RIO CUARTO','LA CAUTIVA','6142','3','2'],
                ['RIO CUARTO','LA DORMIDA','5817','3','2'],
                ['RIO CUARTO','LABOULAYE','5120','3','2'],
                ['RIO CUARTO','LABOULAYE','6120','3','2'],
                ['RIO CUARTO','LAS HIGUERAS  ','5805','3','2'],
                ['RIO CUARTO','LAS VERTIENTES','5827','3','2'],
                ['RIO CUARTO','LOS CISNES','2684','3','2'],
                ['RIO CUARTO','LOS CONDORES','5823','3','2'],
                ['RIO CUARTO','MATTALDI','6271','3','3'],
                ['RIO CUARTO','MATTALDI','6272','3','3'],
                ['RIO CUARTO','MELO','6123','3','2'],
                ['RIO CUARTO','NICOLAS BRUZONE','6271','3','2'],
                ['RIO CUARTO','OLAETA','5824','3','2'],
                ['RIO CUARTO','ONAGOITY','6227','3','2'],
                ['RIO CUARTO','ONAGOITY','6271','3','2'],
                ['RIO CUARTO','Pueblo Italiano','2653','3','2'],
                ['RIO CUARTO','REDUCCION','5803','3','2'],
                ['RIO CUARTO','RIO BAMBA','6134','3','2'],
                ['RIO CUARTO','RIO CUARTO','5800','3','2'],
                ['RIO CUARTO','RIO CUARTO','5802','3','2'],
                ['RIO CUARTO','RIO CUARTO','5804','3','2'],
                ['RIO CUARTO','RIO CUARTO','5806','3','2'],
                ['RIO CUARTO','RIO CUARTO','5808','3','2'],
                ['RIO CUARTO','RIO DE LOS SAUCES','5821','3','2'],
                ['RIO CUARTO','SAN AMBROSIO','5848','3','2'],
                ['RIO CUARTO','SANTA EUFEMIA','2671','3','2'],
                ['RIO CUARTO','SERRANO','6125','3','2'],
                ['RIO CUARTO','SUCO','5837','3','2'],
                ['RIO CUARTO','TOSQUITA','6141','3','2'],
                ['RIO CUARTO','UCACHA','2677','3','2'],
                ['RIO CUARTO','VIAMONTE','2671','3','2'],
                ['RIO CUARTO','VICUÑA MACKENNA','6140','3','2'],
                ['RIO CUARTO','VILLA HUIDOBRO','6175','3','2'],
                ['RIO CUARTO','VILLA HUIDOBRO','6275','3','2'],
                ['RIO CUARTO','WASHINGTON','6144','3','2'],
                ['RIO CUARTO','PINCEN','6271','3','3'],
                ['RIO CUARTO','PINCEN','6274','3','2'],
                ['RIO CUARTO','ITALO','6271','3','3'],
                ['RIO CUARTO','ITALO','6278','3','2'],
                ['RIO CUARTO','RANQUELES','6271','3','2'],
                ['RIO TERCERO','ALMAFUERTE','5854','3','2'],
                ['RIO TERCERO','COLONIA SANTA CATALINA','5851','3','2'],
                ['RIO TERCERO','CORRALITO','5853','3','2'],
                ['RIO TERCERO','DESPEÑADEROS','5121','3','2'],
                ['RIO TERCERO','DIQUE LOS MOLINOS','5192','3','2'],
                ['RIO TERCERO','EMBALSE','5856','3','2'],
                ['RIO TERCERO','EMBALSE','5858','3','2'],
                ['RIO TERCERO','GENERAL FOTHERIGHAM','5933','3','2'],
                ['RIO TERCERO','HERNANDO','5929','3','2'],
                ['RIO TERCERO','HERNANDO','5968','3','2'],
                ['RIO TERCERO','LA CRUZ','5859','3','2'],
                ['RIO TERCERO','PAMPAYASTA SUD','5931','3','2'],
                ['RIO TERCERO','RIO TERCERO','5850','3','2'],
                ['RIO TERCERO','SAN AGUSTIN','5191','3','2'],
                ['RIO TERCERO','SAN IGNACIO','5199','3','2'],
                ['RIO TERCERO','SANTA ROSA DE CALAMUCHITA','5196','3','2'],
                ['RIO TERCERO','TANCACHA','5933','3','2'],
                ['RIO TERCERO','VILLA ASCASUBI','5935','3','2'],
                ['RIO TERCERO','VILLA DEL DIQUE','5862','3','2'],
                ['RIO TERCERO','VILLA GENERAL BELGRANO','5194','3','2'],
                ['RIO TERCERO','VILLA RUMIPAL','5864','3','2'],
                ['RIO TERCERO','VILLA SEGUNDA','5857','3','2'],
                ['RIO TERCERO','YANCANTO CALAMUCHITA','5197','3','2'],
                ['SAN FRANCISCO','ALICIA','5949','3','2'],
                ['SAN FRANCISCO','BALNEARIA','5141','3','2'],
                ['SAN FRANCISCO','BRINKMANN','2419','3','2'],
                ['SAN FRANCISCO','CARRILOBO','5915','3','2'],
                ['SAN FRANCISCO','COLONIA CASTELAR','2401','3','2'],
                ['SAN FRANCISCO','COLONIA VALTELINA','2414','3','2'],
                ['SAN FRANCISCO','COLONIA VIGNAUD','2420','3','2'],
                ['SAN FRANCISCO','DEVOTO  ','2424','3','2'],
                ['SAN FRANCISCO','EL FORTIN','5951','3','2'],
                ['SAN FRANCISCO','EL TIO','2432','3','2'],
                ['SAN FRANCISCO','FREYRE','2413','3','2'],
                ['SAN FRANCISCO','LA CAUTIBA','5142','3','2'],
                ['SAN FRANCISCO','LA FRANCIA','2425','3','2'],
                ['SAN FRANCISCO','LA FRANCIA','2426','3','2'],
                ['SAN FRANCISCO','LA PAQUITA','2416','3','2'],
                ['SAN FRANCISCO','LA PAQUITA','2417','3','2'],
                ['SAN FRANCISCO','LA PLAYOSA','5911','3','2'],
                ['SAN FRANCISCO','LAS VARAS','5941','3','2'],
                ['SAN FRANCISCO','LEXARDO','2411','3','2'],
                ['SAN FRANCISCO','MARIANA','2424','3','2'],
                ['SAN FRANCISCO','MIRAMAR','5143','3','2'],
                ['SAN FRANCISCO','PASCO','5953','3','2'],
                ['SAN FRANCISCO','PORTEÑA','2415','3','2'],
                ['SAN FRANCISCO','POZO DEL MOLLE','5913','3','2'],
                ['SAN FRANCISCO','QUEBRACHO HERRADO','2422','3','2'],
                ['SAN FRANCISCO','QUEBRACHO HERRADO','2423','3','2'],
                ['SAN FRANCISCO','RECREO VICTORIA','5144','3','2'],
                ['SAN FRANCISCO','SAN FRANCISCO','2400','3','2'],
                ['SAN FRANCISCO','SATURNINO M.LASPIUR','5943','3','2'],
                ['SAN FRANCISCO','SUARDI','2349','3','2'],
                ['SAN FRANCISCO','COLONIA SAN BARTOLOME','2426','3','3'],
                ['SAN FRANCISCO','COLONIA JOSEFINA','2404','21','2'],
                ['SAN FRANCISCO','COLONIA ROSA','2347','21','2'],
                ['SAN FRANCISCO','COLONIA ROSA','2351','21','2'],
                ['SAN FRANCISCO','CRISPI','2441','21','2'],
                ['SAN FRANCISCO','ESMERALDA','2456','21','2'],
                ['SAN FRANCISCO','ESTACION CLUCELLAS','2408','21','2'],
                ['SAN FRANCISCO','ESTACION JOSEFINA','2403','21','2'],
                ['SAN FRANCISCO','FRONTERA','2438','21','2'],
                ['SAN FRANCISCO','GARIBALDI','2443','21','2'],
                ['SAN FRANCISCO','HERSILLA','2352','21','2'],
                ['SAN FRANCISCO','LA RUBIA','2342','21','2'],
                ['SAN FRANCISCO','LA RUBIA','2350','21','2'],
                ['SAN FRANCISCO','LANDETA','2531','21','2'],
                ['SAN FRANCISCO','MARIA JUANA','2445','21','2'],
                ['SAN FRANCISCO','MONIGOTES','2342','21','2'],
                ['SAN FRANCISCO','MONIGOTES','2348','21','2'],
                ['SAN FRANCISCO','NUEVA LEHMANN','2326','21','2'],
                ['SAN FRANCISCO','PUEBLO MARIA JUANA','2446','21','2'],
                ['SAN FRANCISCO','TACURAL','2324','SANTA FE','2'],
                ['SAN FRANCISCO','TRAILL','2456','SANTA FE','2'],
                ['SAN FRANCISCO','VILLA TRINIDAD','2345','21','2'],
                ['SAN FRANCISCO','COLONIIA BOSSI','2326','21','3'],
                ['VILLA CARLOS PAZ','ALTA GRACIA','5186','3','2'],
                ['VILLA CARLOS PAZ','ANISACATE','5189','3','2'],
                ['VILLA CARLOS PAZ','BIALET MASSE','5158','3','2'],
                ['VILLA CARLOS PAZ','CAPILLA DEL MONTE','5184','3','2'],
                ['VILLA CARLOS PAZ','CASA SERRANA','5175','3','2'],
                ['VILLA CARLOS PAZ','COSQUIN','5166','3','2'],
                ['VILLA CARLOS PAZ','LA CUMBRE','5178','3','2'],
                ['VILLA CARLOS PAZ','LA CUMBRE','5180','3','2'],
                ['VILLA CARLOS PAZ','LA FALDA','5172','3','2'],
                ['VILLA CARLOS PAZ','LOS COCOS','5182','3','2'],
                ['VILLA CARLOS PAZ','SAN CLEMENTE','5187','3','2'],
                ['VILLA CARLOS PAZ','SAN ESTEBAN','5182','3','2'],
                ['VILLA CARLOS PAZ','SANATORIO SANTA MARIA','5165','3','2'],
                ['VILLA CARLOS PAZ','VILLA COSTA AZUL','5153','3','2'],
                ['VILLA CARLOS PAZ','VILLA SUIZA ARGENTINA','5156','3','2'],
                ['VILLA MARIA','ALTO ALEGRE','5907','3','2'],
                ['VILLA MARIA','ANA ZURMARAN','5905','3','2'],
                ['VILLA MARIA','ARROYO CABRAL','5917','3','2'],
                ['VILLA MARIA','AUSONIA','5924','3','2'],
                ['VILLA MARIA','BALLESTEROS','2572','3','2'],
                ['VILLA MARIA','BALLESTEROS SUD','2573','3','2'],
                ['VILLA MARIA','BELL VILLE','2550','3','2'],
                ['VILLA MARIA','CHILIBROSTE','2561','3','2'],
                ['VILLA MARIA','COLONIA BISMARK','2652','3','2'],
                ['VILLA MARIA','COLONIA SILVIO PELLICO','5910','3','2'],
                ['VILLA MARIA','DALMACIO VELEZ SARSFIELD  ','5919','3','2'],
                ['VILLA MARIA','ETRURIA','2681','3','2'],
                ['VILLA MARIA','GENERAL BALDISSERA','2583','3','2'],
                ['VILLA MARIA','GENERAL DEHEZA','5923','3','2'],
                ['VILLA MARIA','GENERAL FOTHERINGHAM','5982','3','2'],
                ['VILLA MARIA','GENERAL ROCA','2592','3','2'],
                ['VILLA MARIA','IDIAZABAL','2557','3','2'],
                ['VILLA MARIA','ISLA VERDE','2661','3','2'],
                ['VILLA MARIA','JUSTINIANO POSSE','2553','3','2'],
                ['VILLA MARIA','LA LAGUNA','5901','3','2'],
                ['VILLA MARIA','LABORDE','2657','3','2'],
                ['VILLA MARIA','LAS PERDICES','5921','3','2'],
                ['VILLA MARIA','LEONES','2594','3','2'],
                ['VILLA MARIA','LOS PATOS','2551','3','2'],
                ['VILLA MARIA','LUCA','5919','3','2'],
                ['VILLA MARIA','MARCOS JUAREZ','2580','3','2'],
                ['VILLA MARIA','MONTE BUEY','2589','3','2'],
                ['VILLA MARIA','ORDOÑEZ','2555','3','2'],
                ['VILLA MARIA','PAMPAYASTA NORTE','5973','3','2'],
                ['VILLA MARIA','PAMPAYASTA SUR','5975','3','2'],
                ['VILLA MARIA','PASCANAS','2679','3','2'],
                ['VILLA MARIA','PASCO','5925','3','2'],
                ['VILLA MARIA','SAIRA','2525','3','2'],
                ['VILLA MARIA','SAN ANTONIO DEL LITIN','2560','3','2'],
                ['VILLA MARIA','SAN MARCOS SUD','2566','3','2'],
                ['VILLA MARIA','TICINO','5927','3','2'],
                ['VILLA MARIA','TICINO','5966','3','2'],
                ['VILLA MARIA','TIO PUJIO','5936','3','2'],
                ['VILLA MARIA','VILLA MARIA','5900','3','2'],
                ['VILLA MARIA','VILLA NUEVA','5903','3','2'],
                ['VILLA MARIA','WENCESLAO ESCALANTE','2655','3','2'],
                ['VILLA MARIA','TORTUGAS','2512','21','2'],
                ['CATAMARCA','ALIJILAN','4723','7','2'],
                ['CATAMARCA','BAÑADO DE OVANTA','4721','7','2'],
                ['CATAMARCA','BAVIANO','5265','7','2'],
                ['CATAMARCA','BELEN','4750','7','2'],
                ['CATAMARCA','CATAMARCA','4700','7','2'],
                ['CATAMARCA','CATAMARCA','4702','7','2'],
                ['CATAMARCA','CATAMARCA','4703','7','2'],
                ['CATAMARCA','CATAMARCA','4704','7','2'],
                ['CATAMARCA','CHUMBICHA','4728','7','2'],
                ['CATAMARCA','COLPES','4711','7','2'],
                ['CATAMARCA','COLPES','5319','7','2'],
                ['CATAMARCA','EL ALAMITO','4743','7','2'],
                ['CATAMARCA','EL BAÑADO','4732','7','2'],
                ['CATAMARCA','EL EJE','4754','7','2'],
                ['CATAMARCA','EL HUECO','4710','7','2'],
                ['CATAMARCA','FRAY MAMERTO ESQUIU','4709','7','3'],
                ['CATAMARCA','FRAY MAMERTO ESQUIU','5261','7','2'],
                ['CATAMARCA','HUILLAPIMA','4726','7','2'],
                ['CATAMARCA','LA CARRERA','4711','7','3'],
                ['CATAMARCA','LA CARRERA','4712','7','2'],
                ['CATAMARCA','LA FALDA DE SAN ANTONIO','4731','7','2'],
                ['CATAMARCA','LA MERCED','4718','7','2'],
                ['CATAMARCA','LA TERCENA','4717','7','2'],
                ['CATAMARCA','LAS PIRQUITAS','4713','7','2'],
                ['CATAMARCA','LAVALLE','5347','7','2'],
                ['CATAMARCA','LONDRES','4753','7','2'],
                ['CATAMARCA','MIRAFLORES','4724','7','2'],
                ['CATAMARCA','PIEDRA BLANCA','4709','7','2'],
                ['CATAMARCA','POMAN','5315','7','2'],
                ['CATAMARCA','QUIROS','5266','7','2'],
                ['CATAMARCA','RINCON','5317','7','2'],
                ['CATAMARCA','SALADO','5332','7','2'],
                ['CATAMARCA','SAN ANTONIO DE LA PAZ','5264','7','2'],
                ['CATAMARCA','SAN ANTONIO FRAY M ESQUIU','4708','7','2'],
                ['CATAMARCA','SANTA ROSA','5343','7','2'],
                ['CATAMARCA','SAUJIL','5321','7','2'],
                ['CATAMARCA','SUMALAO','4705','7','2'],
                ['CATAMARCA','SUMALAO','4714','7','2'],
                ['CATAMARCA','TACO','4701','7','2'],
                ['CATAMARCA','TAPSO','4234','7','2'],
                ['CATAMARCA','TATON','5342','7','2'],
                ['CATAMARCA','TERMAS VILLA VIL','4755','7','2'],
                ['CATAMARCA','TRES PUENTES','4707','7','2'],
                ['CATAMARCA','VALLE VIEJO','4707','7','2'],
                ['CATAMARCA','VILLA MAZAN','5314','7','2'],
                ['CATAMARCA','ESTACION MAZAN','5313','11','2'],
                ['JUJUY','ABRA PAMPA','4640','10','2'],
                ['JUJUY','CAIMANCITO','4516','10','2'],
                ['JUJUY','CALILEGUA','4514','10','2'],
                ['JUJUY','CARAHUASI','4643','10','2'],
                ['JUJUY','CASPALA','4631','10','2'],
                ['JUJUY','CIENEGUILLAS','4653','10','2'],
                ['JUJUY','COCHINONGA','4641','10','2'],
                ['JUJUY','EL PIQUETE','4501','10','2'],
                ['JUJUY','EL QUEMADO','4504','10','2'],
                ['JUJUY','EL TALAR','4541','10','2'],
                ['JUJUY','EL TALAR  ','4542','10','2'],
                ['JUJUY','HIPOLITO YRIGOYEN ESTACION ITURBE','4635','10','2'],
                ['JUJUY','HUACALERA','4626','10','2'],
                ['JUJUY','JUJUY','4600','10','2'],
                ['JUJUY','JUJUY','4602','10','2'],
                ['JUJUY','JUJUY','4604','10','2'],
                ['JUJUY','JUJUY','4607','10','2'],
                ['JUJUY','LA ESPERANZA','4503','10','2'],
                ['JUJUY','LEDESMA  ','4512','10','2'],
                ['JUJUY','LIBERTADOR GENERAL SAN MARTIN','4512','10','2'],
                ['JUJUY','MAIMARA','4622','10','2'],
                ['JUJUY','MONTERRICO','4608','10','2'],
                ['JUJUY','ORATORIO','4656','10','2'],
                ['JUJUY','PALMA SOLA','4510','10','2'],
                ['JUJUY','PALPALA','4612','10','2'],
                ['JUJUY','PAMPICHUELA','4513','10','2'],
                ['JUJUY','PURMAMARCA','4618','10','2'],
                ['JUJUY','PURMAMARCA','4619','10','2'],
                ['JUJUY','RAWSON','4523','10','2'],
                ['JUJUY','RINCONADA','4645','10','2'],
                ['JUJUY','SAN  PEDRO DE JUJUY','4500','10','2'],
                ['JUJUY','SAN  PEDRO DE JUJUY','4502','10','2'],
                ['JUJUY','SAN  PEDRO DE JUJUY','4609','10','2'],
                ['JUJUY','SANTA CATALINA','4655','10','2'],
                ['JUJUY','SANTA VICTORIA','4652','10','2'],
                ['JUJUY','SUSQUES','4642','10','2'],
                ['JUJUY','TOBA','4648','10','2'],
                ['JUJUY','TRES CRUCES','4636','10','2'],
                ['JUJUY','YALA','4616','10','2'],
                ['JUJUY','YALA','4617','10','2'],
                ['JUJUY','YAVI','4651','10','2'],
                ['JUJUY','YUTO','4518','10','2'],
                ['JUJUY','PALPALA','4613','SANTIAGO DEL ESTERO','2'],
                ['LA RIOJA','AGUA BLANCA','5356','11','2'],
                ['LA RIOJA','AIMOGASTA','5310','11','2'],
                ['LA RIOJA','ALPASINCHE','5325','11','2'],
                ['LA RIOJA','ANTINACO','5361','11','2'],
                ['LA RIOJA','ANTINACO','5362','11','2'],
                ['LA RIOJA','ATILES','5388','11','2'],
                ['LA RIOJA','BANDA FLORIDA','5351','11','2'],
                ['LA RIOJA','BARRIO DE GALLI','5375','11','2'],
                ['LA RIOJA','BELLA VISTA','5381','11','2'],
                ['LA RIOJA','CAMPANAS','5366','11','2'],
                ['LA RIOJA','CARRIZAL TAMA','5376','11','2'],
                ['LA RIOJA','CASTRO BARROS','5276','11','2'],
                ['LA RIOJA','CHAMICAL','5380','11','2'],
                ['LA RIOJA','CHEPES','5470','11','2'],
                ['LA RIOJA','CHILA','5378','11','2'],
                ['LA RIOJA','CHILECITO','5360','11','2'],
                ['LA RIOJA','CILLA NIDIA','5473','11','2'],
                ['LA RIOJA','DESIDERIO TELLO','5474','11','2'],
                ['LA RIOJA','DISTRITO PUEBLO','5359','11','2'],
                ['LA RIOJA','DISTRITO PUEBLO','5371','11','2'],
                ['LA RIOJA','FAMATINA','5365','11','2'],
                ['LA RIOJA','GUADACOL','5353','11','2'],
                ['LA RIOJA','LOMA BLANCA','5382','11','2'],
                ['LA RIOJA','LOS AGUIRRES','5275','11','2'],
                ['LA RIOJA','LOS PALACIOS','5352','11','2'],
                ['LA RIOJA','LOS SARMIENTOS','5373','11','2'],
                ['LA RIOJA','MACHIGASTA','5311','11','2'],
                ['LA RIOJA','MALANZAN','5389','11','2'],
                ['LA RIOJA','MASCASIN','5471','11','2'],
                ['LA RIOJA','MILAGRO','5274','11','2'],
                ['LA RIOJA','NONOGASTA','5372','11','2'],
                ['LA RIOJA','PAGANCILLO','5369','11','2'],
                ['LA RIOJA','PATQUIA','5386','11','2'],
                ['LA RIOJA','PITUIL','5370','11','2'],
                ['LA RIOJA','PUNTA DE LOS LLANOS','5384','11','2'],
                ['LA RIOJA','SALICAS','5327','11','2'],
                ['LA RIOJA','SAN JOSE DE TINOGASTA','5339','11','2'],
                ['LA RIOJA','SAN PEDRO','5309','11','2'],
                ['LA RIOJA','SAÑOGASTA','5367','11','2'],
                ['LA RIOJA','POZO DE PIEDRA','5473','11','3'],
                ['LA RIOJA','SAN NICOLAS','5301','11','2'],
                ['LA RIOJA','TAMA','5385','11','2'],
                ['ORAN','URUNDEL','4542','10','2'],
                ['ORAN','EL TABACAL','4533','17','2'],
                ['ORAN','HIPOLITO YRIGOYEN','4532','17','2'],
                ['ORAN','LA ESTRELLA','4537','17','2'],
                ['ORAN','LOS BLANCOS','4555','17','2'],
                ['ORAN','LOS TOLDOS','4531','17','2'],
                ['ORAN','ORAN','4530','17','2'],
                ['ORAN','PEDRO LOZANO','4554','17','2'],
                ['ORAN','POZO CERCADO','4535','17','2'],
                ['ORAN','COLONIA SANTA ROSA','4531','17','2'],
                ['ORAN','AGUAS BLANCAS','4531','17','2'],
                ['ORAN','LA UNIÑN','4535','17','2'],
                ['ORAN','CORONEL SOLA','4554','17','2'],
                ['ORAN','MORILLO','4554','17','2'],
                ['ORAN','HICKMAN','4554','17','2'],
                ['ORAN','PLUMA DE PATO','4554','17','2'],
                ['ORAN','DRAGONES','4554','17','2'],
                ['ORAN','CAPITAN PAGES','4554','17','2'],
                ['SALTA','CAUTA','4413','10','2'],
                ['SALTA','ANGASTACO','4426','17','2'],
                ['SALTA','ARENAL','4198','17','2'],
                ['SALTA','CACHI','4417','17','2'],
                ['SALTA','CAFAYATE','4427','17','2'],
                ['SALTA','CAMPO QUIJANO','4407','17','2'],
                ['SALTA','CAMPO SANTO','4432','17','2'],
                ['SALTA','CERRILLOS','4403','17','2'],
                ['SALTA','CHICOANA','4423','17','2'],
                ['SALTA','CORONEL MOLLINEDO','4451','17','2'],
                ['SALTA','EL ALISAL','4409','17','2'],
                ['SALTA','EL BORDO','4449','17','2'],
                ['SALTA','EL GALPON','4444','17','2'],
                ['SALTA','EL QUEBRACHAL','4453','17','2'],
                ['SALTA','EL TUÑAL','4446','17','2'],
                ['SALTA','GENERAL GUEMES','4430','17','2'],
                ['SALTA','GUACHIPAS','4425','17','2'],
                ['SALTA','JOAQUIN V.GONZALEZ','4448','17','2'],
                ['SALTA','LA POMA','4416','17','2'],
                ['SALTA','METAN VIEJO','4441','17','2'],
                ['SALTA','MOLINOS','4420','17','2'],
                ['SALTA','PALOMITAS','4434','17','2'],
                ['SALTA','PAYOGASTA','4415','17','2'],
                ['SALTA','PAYOGASTA','4418','17','2'],
                ['SALTA','PIQUIRENDA','4564','17','2'],
                ['SALTA','ROSARIO DE LA FRONTERA','4190','17','2'],
                ['SALTA','ROSARIO DE LERMA','4405','17','2'],
                ['SALTA','SALTA','4400','17','2'],
                ['SALTA','SALTA','4402','17','2'],
                ['SALTA','SALTA','4404','17','2'],
                ['SALTA','SALTA','4406','17','2'],
                ['SALTA','SALTA','4408','17','2'],
                ['SALTA','SALTA','4410','17','2'],
                ['SALTA','SALTA','4412','17','2'],
                ['SALTA','SALTA','4414','17','2'],
                ['SALTA','SAN CARLOS','4428','17','2'],
                ['SALTA','SAUCELITO','4538','17','2'],
                ['SALTA','SECLANTAS','4419','17','2'],
                ['SALTA','VAQUEROS','4401','17','2'],
                ['SALTA','TOLOMBON','4154','TUCUMAN','2'],
                ['SANTIAGO DEL ESTERO','LAS PEÑAS','4237','7','2'],
                ['SANTIAGO DEL ESTERO','LOS PIRPINTOS','3712','CHACO','2'],
                ['SANTIAGO DEL ESTERO','SACHAYOJ','3731','CHACO','2'],
                ['SANTIAGO DEL ESTERO','TACO POZO','3713','CHACO','2'],
                ['SANTIAGO DEL ESTERO','TACO POZO','3717','CHACO','2'],
                ['SANTIAGO DEL ESTERO','URUTAU','3714','CHACO','2'],
                ['SANTIAGO DEL ESTERO','MATARA','4357','17','2'],
                ['SANTIAGO DEL ESTERO','25 DE MAYO','4352','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','ABRA GRANDE','4336','SANTIAGO DEL ESTERO','3'],
                ['SANTIAGO DEL ESTERO','AÑATUYA','3760','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','BANDERA','3064','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','BARRIO HUACO HONDO','4202','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','BELTRAN','4308','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','CAMPO GALLO','3747','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','COLONIA DORA','4332','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','EL AIBE','4302','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','EL COLORADO','3741','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','FERNANDEZ','4322','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','GRAMILLA','4304','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','GUANACO SOMBRIANA','4212','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','HERRERA','4328','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','HUAJIA','4318','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','LA BANDA','4300','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','LA INVERNADA','4353','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','LA REPRESA','4233','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','LAPRIDA','4205','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','LAS TINAJAS','4351','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','LOMITAS BLANCAS','5251','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','LOS QUIROGA','4201','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','LUGONES','4326','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','MONTE RICO','3749','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','NEGRA MUERTA','5258','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','NUEVA ESPERANZA','4197','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','NUEVA FRANCIA','4207','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','OJO DE AGUA','5250','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','PINTO','2356','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','QUILUMPA','3745','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','QUIMILI','3740','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','RAMIREZ DE VELAZCO','5257','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','SAN PEDRO','4238','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','SAN ROQUE','4320','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','SANTIAGO DEL ESTERO','4200','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','SANTIAGO DEL ESTERO','4204','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','SANTO DOMINGO','4301','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','SIMBOLAR','4340','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','TACANITAS','3766','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','TERMAS DEL RIO HONDO','4220','SANTIAGO DEL ESTERO','3'],
                ['SANTIAGO DEL ESTERO','TINTINA','3743','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','TIUN PUNCO','4356','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','VILELAS','3752','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','VILLA ATAMISQUI','4317','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','VIZCACHERAL','4203','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','ZANJON','4206','SANTIAGO DEL ESTERO','2'],
                ['SANTIAGO DEL ESTERO','TACO POZO','3714','SANTIAGO DEL ESTERO','3'],
                ['TARTAGAL','SANTA MARIA','4561','10','2'],
                ['TARTAGAL','AGUARAY','4566','17','2'],
                ['TARTAGAL','GENERAL BALLIVIAN','4552','17','2'],
                ['TARTAGAL','GENERALMOSCONI','4562','17','2'],
                ['TARTAGAL','TARTAGAL','4560','17','2'],
                ['TARTAGAL','CORONEL CORNEJO','4552','17','2'],
                ['TUCUMAN','EL CRUCE','4138','7','2'],
                ['TUCUMAN','SANTA MARIA','4139','7','3'],
                ['TUCUMAN','7 DE ABRIL','4194','17','2'],
                ['TUCUMAN','LA RAMADA','4123','17','2'],
                ['TUCUMAN','SIETE DE ABRIL','4195','17','2'],
                ['TUCUMAN','TALA','4126','17','2'],
                ['TUCUMAN','TRES POZOS','4178','SANTIAGO DEL ESTERO','2'],
                ['TUCUMAN','ALDERETE  ','4178','SANTIAGO DEL ESTERO','2'],
                ['TUCUMAN','BOBADAL','4187','SANTIAGO DEL ESTERO','2'],
                ['TUCUMAN','ESTACION ARAOZ','4177','SANTIAGO DEL ESTERO','2'],
                ['TUCUMAN','GOBERNADOR PIEDRABUENA','4188','SANTIAGO DEL ESTERO','2'],
                ['TUCUMAN','LAS CEJAS','4186','SANTIAGO DEL ESTERO','2'],
                ['TUCUMAN','POZO HONDO','4184','SANTIAGO DEL ESTERO','2'],
                ['TUCUMAN','RAPELLI','4189','SANTIAGO DEL ESTERO','2'],
                ['TUCUMAN','ROVERSI','3736','SANTIAGO DEL ESTERO','2'],
                ['TUCUMAN','VILLA RIO HONDO','4226','SANTIAGO DEL ESTERO','2'],
                ['TUCUMAN','AGUA DULCE','4115','TUCUMAN','2'],
                ['TUCUMAN','BELLA VISTA','4168','TUCUMAN','2'],
                ['TUCUMAN','CAMPO HERRERA','4110','TUCUMAN','2'],
                ['TUCUMAN','DELFIN GALLO','4117','TUCUMAN','2'],
                ['TUCUMAN','EL BRACHO','4112','TUCUMAN','2'],
                ['TUCUMAN','EL MANANTIAL','4104','TUCUMAN','2'],
                ['TUCUMAN','FAMAILLA','4132','TUCUMAN','2'],
                ['TUCUMAN','FUERTE QUEMADO','4150','TUCUMAN','2'],
                ['TUCUMAN','GARMENDIA','4181','TUCUMAN','2'],
                ['TUCUMAN','HUASA PAMPA','4163','TUCUMAN','2'],
                ['TUCUMAN','LA RAMADA','4119','TUCUMAN','2'],
                ['TUCUMAN','LASTENIA','4111','TUCUMAN','2'],
                ['TUCUMAN','LASTENIA','4116','TUCUMAN','2'],
                ['TUCUMAN','LEALES','4113','TUCUMAN','2'],
                ['TUCUMAN','LOS POCITOS','4101','TUCUMAN','2'],
                ['TUCUMAN','LOS POCITOS','4102','TUCUMAN','2'],
                ['TUCUMAN','LOS ZAZOS','4125','TUCUMAN','2'],
                ['TUCUMAN','MANANTIAL','4105','TUCUMAN','2'],
                ['TUCUMAN','NUEVA BAVIERA','4136','TUCUMAN','2'],
                ['TUCUMAN','PACARA','4003','TUCUMAN','2'],
                ['TUCUMAN','PADILLA','4133','TUCUMAN','2'],
                ['TUCUMAN','RANCHILLOS','4179','TUCUMAN','2'],
                ['TUCUMAN','RIO COLORADO','4173','TUCUMAN','2'],
                ['TUCUMAN','SAN JAVIER','4108','TUCUMAN','2'],
                ['TUCUMAN','SAN MIGUEL DE TUCUMAN','4167','TUCUMAN','2'],
                ['TUCUMAN','SANTA MARIA','4011','TUCUMAN','3'],
                ['TUCUMAN','SARGENTO MOYA','4165','TUCUMAN','2'],
                ['TUCUMAN','TAFI DEL VALLE','4127','TUCUMAN','2'],
                ['TUCUMAN','TAFI DEL VALLE','4137','TUCUMAN','2'],
                ['TUCUMAN','TAFI DEL VALLE','4140','TUCUMAN','2'],
                ['TUCUMAN','TAFI VIEJO','4103','TUCUMAN','2'],
                ['TUCUMAN','TAPIA','4122','TUCUMAN','2'],
                ['TUCUMAN','TRANCAS','4124','TUCUMAN','2'],
                ['TUCUMAN','TUCUMAN','4000','TUCUMAN','2'],
                ['TUCUMAN','TUCUMAN','4001','TUCUMAN','2'],
                ['TUCUMAN','TUCUMAN','4002','TUCUMAN','2'],
                ['TUCUMAN','VILLA CARMELA','4005','TUCUMAN','2'],
                ['TUCUMAN','VILLA CARMELA','4010','TUCUMAN','2'],
                ['TUCUMAN','VILLA FIAD','4118','TUCUMAN','2'],
                ['TUCUMAN','YERBA BUENA','4107','TUCUMAN','2'],
                ['9 DE JULIO','9 DE JULIO','6500','2','2'],
                ['9 DE JULIO','ALAGON','6463','2','2'],
                ['9 DE JULIO','BACACAY','6516','2','2'],
                ['9 DE JULIO','BELLOCQ','6535','2','2'],
                ['9 DE JULIO','BERUTTI','6424','2','2'],
                ['9 DE JULIO','BOLIVAR','6550','2','2'],
                ['9 DE JULIO','BRAGADO','6640','2','2'],
                ['9 DE JULIO','CARLOS CASARES','6530','2','2'],
                ['9 DE JULIO','CARLOS MARIA NAON','6515','2','2'],
                ['9 DE JULIO','DEL VALLE','6509','2','2'],
                ['9 DE JULIO','ENRIQUE LAVALLE','6467','2','2'],
                ['9 DE JULIO','ESPIGAS','6561','2','2'],
                ['9 DE JULIO','FRANCISCO MADERO','6472','2','2'],
                ['9 DE JULIO','FRENCH','6516','2','2'],
                ['9 DE JULIO','HALE','6511','2','2'],
                ['9 DE JULIO','LA COTORRA','6461','2','2'],
                ['9 DE JULIO','LA DORITA','6538','2','2'],
                ['9 DE JULIO','LUCAS MONTEVERDE','6661','2','2'],
                ['9 DE JULIO','NUEVA PLATA','6451','2','2'],
                ['9 DE JULIO','ORDOQUI','6537','2','2'],
                ['9 DE JULIO','PEHUAJO','6450','2','2'],
                ['9 DE JULIO','PIROVANO','6551','2','2'],
                ['9 DE JULIO','PRIMERA JUNTA','6422','2','2'],
                ['9 DE JULIO','RECALDE','6559','2','2'],
                ['9 DE JULIO','SALAZAR','6471','2','2'],
                ['9 DE JULIO','SAN BERNARDO','6476','2','2'],
                ['9 DE JULIO','SMITH','6531','2','2'],
                ['9 DE JULIO','TROPEZON','6501','2','2'],
                ['9 DE JULIO','URDAMPILLETA','6553','2','2'],
                ['9 DE JULIO','VALDES','6667','2','2'],
                ['GENERAL PICO','GONZALEZ MORENO','6239','2','2'],
                ['GENERAL PICO','BATAVIA','6279','3','2'],
                ['GENERAL PICO','AGUSTONI','6361','9','2'],
                ['GENERAL PICO','ALTA ITALIA','6207','9','2'],
                ['GENERAL PICO','ARATA','6385','9','2'],
                ['GENERAL PICO','BERNARDO LARROUDE','6220','9','2'],
                ['GENERAL PICO','CHAMAICO','6214','9','2'],
                ['GENERAL PICO','CORONEL HILARIO LAGOS','6228','9','2'],
                ['GENERAL PICO','DORILA','6365','9','2'],
                ['GENERAL PICO','FALUCHO','6215','9','2'],
                ['GENERAL PICO','GENERAL PICO','6360','9','2'],
                ['GENERAL PICO','INGENIERO LUIGGI','6205','9','2'],
                ['GENERAL PICO','INTENDENTE ALVEAR','6221','9','2'],
                ['GENERAL PICO','LUAN TORO','6317','9','2'],
                ['GENERAL PICO','QUETREQUEN','6212','9','2'],
                ['GENERAL PICO','RANCUL','6214','9','2'],
                ['GENERAL PICO','REALICO','6200','9','2'],
                ['GENERAL PICO','SANTA ISABEL','6323','9','2'],
                ['GENERAL PICO','SARAH','6228','9','2'],
                ['GENERAL PICO','SIMSON','6212','9','2'],
                ['GENERAL PICO','TELEN','6321','9','2'],
                ['GENERAL PICO','TREBOLARES','6361','9','2'],
                ['GENERAL PICO','VERTIZ','6365','9','2'],
                ['GENERAL PICO','VICTORICA','6319','9','2'],
                ['GENERAL PICO','ANCHORENA','6389','SAN LUIS','2'],
                ['GENERAL PICO','ARIZONA','6389','SAN LUIS','2'],
                ['GENERAL PICO','BUENA ESPERANZA','6277','SAN LUIS','2'],
                ['GENERAL PICO','UNION','6216','SAN LUIS','2'],
                ['JUNIN','AGUSTIN ROCA','6026','2','2'],
                ['JUNIN','AMEGHINO','6064','2','2'],
                ['JUNIN','ARAUJO','6643','2','2'],
                ['JUNIN','BAIGORRITA','6013','2','2'],
                ['JUNIN','BERMUDEZ','6071','2','2'],
                ['JUNIN','CARLOS TEJEDOR','6455','2','2'],
                ['JUNIN','CHACABUCO','6740','2','2'],
                ['JUNIN','COMODORO PY','6641','2','2'],
                ['JUNIN','CORONEL CHARLONE','6223','2','2'],
                ['JUNIN','CORONEL MOM','6628','2','2'],
                ['JUNIN','EL TRIUNFO','6073','2','2'],
                ['JUNIN','ELORDI','6242','2','2'],
                ['JUNIN','EMILIO AYARZA','6628','2','2'],
                ['JUNIN','EXALTACION DE LA CRUZ','6730','2','2'],
                ['JUNIN','FORTIN OLAVARRIA','6403','2','2'],
                ['JUNIN','GENERAL O`BRIEN','6646','2','2'],
                ['JUNIN','GENERAL PINTO','6050','2','2'],
                ['JUNIN','GENERAL VIAMONTE','6015','2','2'],
                ['JUNIN','GENERAL VILLEGAS','6230','2','2'],
                ['JUNIN','GOBERNADOR UGARTE','6621','2','2'],
                ['JUNIN','GOROSTIAGA','6632','2','2'],
                ['JUNIN','INGENIERO DE MADRID','6651','2','2'],
                ['JUNIN','IRALA','6047','2','2'],
                ['JUNIN','JUNIN','6000','2','2'],
                ['JUNIN','JUNIN','6002','2','2'],
                ['JUNIN','LA LIMPIA','6645','2','2'],
                ['JUNIN','LA ORIENTAL','6022','2','2'],
                ['JUNIN','LA RICA','6623','2','2'],
                ['JUNIN','LAPLACETTE','6013','2','2'],
                ['JUNIN','LAS TOSCAS','6453','2','2'],
                ['JUNIN','LINCOLN','6070','2','2'],
                ['JUNIN','LOS TOLDOS','6015','2','2'],
                ['JUNIN','MECHITA','6648','2','2'],
                ['JUNIN','O`HIGGINS','6748','2','2'],
                ['JUNIN','OLASCOAGA','6652','2','2'],
                ['JUNIN','PASTEUR','6077','2','2'],
                ['JUNIN','PAZOS KANKI','6058','2','2'],
                ['JUNIN','PICHINCHA','6051','2','2'],
                ['JUNIN','PIEDRITAS','6241','2','2'],
                ['JUNIN','PIEDRITAS','6249','2','2'],
                ['JUNIN','RAWSON','6734','2','2'],
                ['JUNIN','RIVAS','6614','2','2'],
                ['JUNIN','ROBERTS','6075','2','2'],
                ['JUNIN','ROBERTS','6079','2','2'],
                ['JUNIN','SANSINENA','6233','2','2'],
                ['JUNIN','TIMOTE','6457','2','2'],
                ['JUNIN','TRES ALGARROBOS','6231','2','2'],
                ['JUNIN','VILLA SAUZE','6235','2','2'],
                ['MERCEDES','25 DE MAYO','6660','2','2'],
                ['MERCEDES','ALBERTI','6634','2','2'],
                ['MERCEDES','DEL CARRIL','7265','2','2'],
                ['MERCEDES','ELVIRA','7243','2','2'],
                ['MERCEDES','EMPALME LOBOS','7249','2','2'],
                ['MERCEDES','GENERAL ALVEAR','7263','2','2'],
                ['MERCEDES','JUAN TRONCONI','7247','2','2'],
                ['MERCEDES','KM 117 APEADERO FCGB','6603','2','2'],
                ['MERCEDES','LA PAZ','7245','2','2'],
                ['MERCEDES','LA VERDE','6601','2','2'],
                ['MERCEDES','LAS MARIANAS','6607','2','2'],
                ['MERCEDES','LOBOS','7240','2','2'],
                ['MERCEDES','LOBOS','7420','2','2'],
                ['MERCEDES','MERCEDES','6600','2','2'],
                ['MERCEDES','MERCEDES','6602','2','2'],
                ['MERCEDES','MERCEDES','6604','2','2'],
                ['MERCEDES','NAVARRO','6605','2','2'],
                ['MERCEDES','NORBERTO DE LA RIESTRA','6663','2','2'],
                ['MERCEDES','OLIVERA','6608','2','2'],
                ['MERCEDES','ROQUE PEREZ ','7245','2','2'],
                ['MERCEDES','SALADILLO','7260','2','2'],
                ['MERCEDES','SALADILLO NORTE','7261','2','2'],
                ['MERCEDES','SALVADOR MARIA','7241','2','2'],
                ['MERCEDES','URIBELARREA','1815','2','2'],
                ['PERGAMINO','ARRECIFES','2740','2','3'],
                ['PERGAMINO','ARRIBEÑOS','6007','2','2'],
                ['PERGAMINO','ARROYO DULCE','2743','2','2'],
                ['PERGAMINO','CARMEN DE ARECO','6725','2','3'],
                ['PERGAMINO','COLON','2720','2','2'],
                ['PERGAMINO','CONESA','2907','2','2'],
                ['PERGAMINO','DESVIO EL CHINGOLO','6031','2','2'],
                ['PERGAMINO','FERRE','6003','2','2'],
                ['PERGAMINO','FERRE','6027','2','2'],
                ['PERGAMINO','FORTIN TIBURCIO','6001','2','2'],
                ['PERGAMINO','GENERAL ARENALES','6005','2','2'],
                ['PERGAMINO','GENERAL ROJO','2905','2','2'],
                ['PERGAMINO','HUNTER','2707','2','2'],
                ['PERGAMINO','INES INDART','2747','2','2'],
                ['PERGAMINO','JUANA A DE LA PEÑA','2717','2','2'],
                ['PERGAMINO','LA LUISA','2753','2','2'],
                ['PERGAMINO','LOS INDIOS','2709','2','2'],
                ['PERGAMINO','MANUEL OCAMPO','2713','2','2'],
                ['PERGAMINO','MARIANO H. ALFONZO','2718','2','2'],
                ['PERGAMINO','PERGAMINO','2700','2','2'],
                ['PERGAMINO','PERGAMINO','2702','2','2'],
                ['PERGAMINO','PINZON','2703','2','2'],
                ['PERGAMINO','RUEDA','2923','2','2'],
                ['PERGAMINO','SALTO','2741','2','3'],
                ['PERGAMINO','SARASA','2721','2','2'],
                ['PERGAMINO','TODD','2754','2','2'],
                ['PERGAMINO','TRES SARGENTOS','6727','2','2'],
                ['PERGAMINO','URQUIZA ','2718','2','2'],
                ['PERGAMINO','CORONEL BOGADO','2103','SANTA FE','2'],
                ['PERGAMINO','JUAN B. MOLINA','2104','SANTA FE','2'],
                ['PERGAMINO','JUAN B. MOLINA ','2103','SANTA FE','2'],
                ['PERGAMINO','LA VANGUARDIA','2133','SANTA FE','2'],
                ['PERGAMINO','PEYRANO','2113','SANTA FE','2'],
                ['PERGAMINO','URQUIZA','2724','SANTA FE','2'],
                ['PERGAMINO','CAÑADA RICA','2105','SANTA FE','2'],
                ['PERGAMINO','LA VANGUARDIA','2105','SANTA FE','2'],
                ['SAN NICOLAS','BARRIO D. ANTONIO','2914','2','2'],
                ['SAN NICOLAS','EL PARAISO','2916','2','2'],
                ['SAN NICOLAS','GOBERNADOR CASTRO','2946','2','2'],
                ['SAN NICOLAS','LA EMILIA','2901','2','2'],
                ['SAN NICOLAS','LA VIOLETA','2751','2','2'],
                ['SAN NICOLAS','PEREZ MILLAN','2933','2','2'],
                ['SAN NICOLAS','RAMALLO','2915','2','2'],
                ['SAN NICOLAS','SAN NICOLAS','2900','2','2'],
                ['SAN NICOLAS','SAN NICOLAS','2902','2','2'],
                ['SAN NICOLAS','THEOBALD','2922','2','2'],
                ['SAN NICOLAS','VILLA GENERAL SAVIO','2913','2','2'],
                ['SAN NICOLAS','VILLA RAMALLO','2914','2','2'],
                ['SAN NICOLAS','PAVON','2118','SANTA FE','2'],
                ['SAN NICOLAS','PAVON','2920','SANTA FE','2'],
                ['SAN NICOLAS','THEOBALD','2118','SANTA FE','2'],
                ['SAN NICOLAS','THEOBALD','2918','SANTA FE','3'],
                ['SAN PEDRO','SAN PEDRO','2930','2','2'],
                ['SANTA ROSA','LEUBUCO','6338','2','2'],
                ['SANTA ROSA','PELLEGRINI','6346','2','2'],
                ['SANTA ROSA','ANGUIL','6326','9','2'],
                ['SANTA ROSA','ATALIVA ROCA','6301','9','2'],
                ['SANTA ROSA','BAGUAL','6218','9','2'],
                ['SANTA ROSA','BERNASCONI','8204','9','2'],
                ['SANTA ROSA','CATRILO','6330','9','2'],
                ['SANTA ROSA','CONHELLO','6381','9','2'],
                ['SANTA ROSA','DOBLAS','6305','9','2'],
                ['SANTA ROSA','EDUARDO CASTEX','6380','9','2'],
                ['SANTA ROSA','GUATRACHE','6311','9','2'],
                ['SANTA ROSA','LONQUIMAY','6352','9','2'],
                ['SANTA ROSA','MACACHIN','6307','9','2'],
                ['SANTA ROSA','MAISONNAVE','6211','9','2'],
                ['SANTA ROSA','MIGUEL RIGLOS','6301','9','2'],
                ['SANTA ROSA','MONTE NIEVAS','6383','9','2'],
                ['SANTA ROSA','NAICO','6325','9','2'],
                ['SANTA ROSA','NUEVA GALIA','6217','9','2'],
                ['SANTA ROSA','PERU','8212','9','2'],
                ['SANTA ROSA','QUEHUE','8203','9','2'],
                ['SANTA ROSA','QUEMU QUEMU','6333','9','2'],
                ['SANTA ROSA','RELMO','6331','9','2'],
                ['SANTA ROSA','RICARDO LAVALLE','6312','9','2'],
                ['SANTA ROSA','SANTA ROSA','6300','9','2'],
                ['SANTA ROSA','SANTA ROSA','6302','9','2'],
                ['SANTA ROSA','SANTA ROSA','6304','9','2'],
                ['SANTA ROSA','TOAY','6303','9','2'],
                ['SANTA ROSA','UNANUE','8214','9','2'],
                ['SANTA ROSA','URIBURU','6354','9','2'],
                ['ZARATE','ALTO VERDE','2801','2','2'],
                ['ZARATE','AZCUENAGA','6721','2','2'],
                ['ZARATE','BARADERO','2942','2','3'],
                ['ZARATE','BLONDEAU','2805','2','2'],
                ['ZARATE','CAPITAN SARMIENTO','2752','2','2'],
                ['ZARATE','FLAMENCO','2763','2','2'],
                ['ZARATE','IRENEO PORTELA','2943','2','2'],
                ['ZARATE','LA LUISA','2752','2','2'],
                ['ZARATE','LOS CARDALES','2814','2','2'],
                ['ZARATE','OTAMENDI','2802','2','2'],
                ['ZARATE','SALTA LUCIA','2935','2','2'],
                ['ZARATE','SAN ANDRES DE GILES','6720','2','2'],
                ['ZARATE','SOLIS','2764','2','2'],
                ['ZARATE','VILLA LIA','2761','2','2'],
                ['ZARATE','VILLA RUIZ','6705','2','2'],
                ['RECONQUISTA','FLORENCIA','3516','CHACO','2'],
                ['RECONQUISTA','LAS MERCEDES','3516','5','2'],
                ['RECONQUISTA','ALEJANDRA','3051','21','2'],
                ['RECONQUISTA','BERNA','3569','21','2'],
                ['RECONQUISTA','EL RABON','3592','21','2'],
                ['RECONQUISTA','EL SOMBRERITO','3585','21','2'],
                ['RECONQUISTA','GUADALUPE NORTELAS GARZAS','3574','21','2'],
                ['RECONQUISTA','LA LOLA','3567','21','2'],
                ['RECONQUISTA','LAS PALMAS','3555','21','2'],
                ['RECONQUISTA','LAS TOSCAS','3586','21','2'],
                ['RECONQUISTA','LOS LAPACHOS','3575','21','2'],
                ['RECONQUISTA','MOUSSY','3561','21','2'],
                ['RECONQUISTA','RECONQUISTA','3560','21','2'],
                ['RECONQUISTA','ROMANG','3555','21','2'],
                ['RECONQUISTA','SANTA LUCIA','3553','21','2'],
                ['RECONQUISTA','TACUARENDI','3587','21','2'],
                ['RECONQUISTA','TARTAGAL','3565','21','2'],
                ['RECONQUISTA','TOBA','3551','21','2'],
                ['RECONQUISTA','VILLA ANA','3583','21','2'],
                ['RECONQUISTA','VILLA GUILLERMINA','3589','21','2'],
                ['RECONQUISTA','VILLA OCAMPO','3580','21','2'],
                ['CERRO LAS ROSAS','ARGUELLO','5147','3','2'],
                ['CERRO LAS ROSAS','ASCOCHINGA','5117','3','2'],
                ['CERRO LAS ROSAS','3','5009','3','2'],
                ['CERRO LAS ROSAS','3','5022','3','2'],
                ['CERRO LAS ROSAS','3','5148','3','2'],
                ['CERRO LAS ROSAS','LA CALERA','5151','3','2'],
                ['CERRO LAS ROSAS','MENDIOLAZA','5107','3','2'],
                ['CERRO LAS ROSAS','MENDIOLAZA','5108','3','2'],
                ['CERRO LAS ROSAS','RIO CEBALLOS','5111','3','2'],
                ['CERRO LAS ROSAS','SALDAN','5150','3','2'],
                ['CERRO LAS ROSAS','SALSIPUEDES','5113','3','2'],
                ['CERRO LAS ROSAS','VILLA ALLENDE','5021','3','2'],
                ['CERRO LAS ROSAS','VILLA ALLENDE','5105','3','2'],
                ['TIGRE','BENAVIDEZ','1621','2','1'],
                ['TIGRE','EL TALAR','1618','2','1'],
                ['TIGRE','RICARDO ROJAS','1610','2','1'],
                ['TIGRE','RINCON DE MILBERG','1624','2','1'],
                ['TIGRE','TIGRE','1648','2','1'],
                ['VIEDMA','CARDENAL CAGLIERO','8506','2','2'],
                ['VIEDMA','CARMEN DE PATAGONES','8504','2','2'],
                ['VIEDMA','GUARDIA MITRE','8505','2','2'],
                ['VIEDMA','VILLALONGA','8512','2','2'],
                ['VIEDMA','JOSE B. CASAS','8506','2','2'],
                ['VIEDMA','GENERAL CONESA','8503','16','3'],
                ['VIEDMA','SIERRA COLORADA','8534','16','3'],
                ['VIEDMA','VALCHETA','8536','16','3'],
                ['VIEDMA','SAN ANTONIO ESTE','8521','16','3'],
                ['VIEDMA','SAN JAVIER','8501','16','2'],
                ['VIEDMA','AGUADA CECILIO','8534','16','3'],
                ['CABALLITO','C.A.B.A.','1170','2','1'],
                ['CABALLITO','C.A.B.A.','1171','2','1'],
                ['CABALLITO','C.A.B.A.','1172','2','1'],
                ['CABALLITO','C.A.B.A.','1173','2','1'],
                ['CABALLITO','C.A.B.A.','1174','2','1'],
                ['CABALLITO','C.A.B.A.','1175','2','1'],
                ['CABALLITO','C.A.B.A.','1177','2','1'],
                ['CABALLITO','C.A.B.A.','1178','2','1'],
                ['CABALLITO','C.A.B.A.','1179','2','1'],
                ['CABALLITO','C.A.B.A.','1180','2','1'],
                ['CABALLITO','C.A.B.A.','1181','2','1'],
                ['CABALLITO','C.A.B.A.','1182','2','1'],
                ['CABALLITO','C.A.B.A.','1183','2','1'],
                ['CABALLITO','C.A.B.A.','1184','2','1'],
                ['CABALLITO','C.A.B.A.','1185','2','1'],
                ['CABALLITO','C.A.B.A.','1186','2','1'],
                ['CABALLITO','C.A.B.A.','1187','2','1'],
                ['CABALLITO','C.A.B.A.','1188','2','1'],
                ['CABALLITO','C.A.B.A.','1189','2','1'],
                ['CABALLITO','C.A.B.A.','1190','2','1'],
                ['CABALLITO','C.A.B.A.','1191','2','1'],
                ['CABALLITO','C.A.B.A.','1192','2','1'],
                ['CABALLITO','C.A.B.A.','1193','2','1'],
                ['CABALLITO','C.A.B.A.','1194','2','1'],
                ['CABALLITO','C.A.B.A.','1195','2','1'],
                ['CABALLITO','C.A.B.A.','1196','2','1'],
                ['CABALLITO','C.A.B.A.','1197','2','1'],
                ['CABALLITO','C.A.B.A.','1198','2','1'],
                ['CABALLITO','C.A.B.A.','1199','2','1'],
                ['CABALLITO','C.A.B.A.','1200','2','1'],
                ['CABALLITO','C.A.B.A.','1201','2','1'],
                ['CABALLITO','C.A.B.A.','1202','2','1'],
                ['CABALLITO','C.A.B.A.','1203','2','1'],
                ['CABALLITO','C.A.B.A.','1204','2','1'],
                ['CABALLITO','C.A.B.A.','1205','2','1'],
                ['CABALLITO','C.A.B.A.','1206','2','1'],
                ['CABALLITO','C.A.B.A.','1207','2','1'],
                ['CABALLITO','C.A.B.A.','1208','2','1'],
                ['CABALLITO','C.A.B.A.','1209','2','1'],
                ['CABALLITO','C.A.B.A.','1210','2','1'],
                ['CABALLITO','C.A.B.A.','1211','2','1'],
                ['CABALLITO','C.A.B.A.','1212','2','1'],
                ['CABALLITO','C.A.B.A.','1213','2','1'],
                ['CABALLITO','C.A.B.A.','1214','2','1'],
                ['CABALLITO','C.A.B.A.','1215','2','1'],
                ['CABALLITO','C.A.B.A.','1216','2','1'],
                ['CABALLITO','C.A.B.A.','1217','2','1'],
                ['CABALLITO','C.A.B.A.','1218','2','1'],
                ['CABALLITO','C.A.B.A.','1219','2','1'],
                ['CABALLITO','C.A.B.A.','1220','2','1'],
                ['CABALLITO','C.A.B.A.','1221','2','1'],
                ['CABALLITO','C.A.B.A.','1222','2','1'],
                ['CABALLITO','C.A.B.A.','1223','2','1'],
                ['CABALLITO','C.A.B.A.','1224','2','1'],
                ['CABALLITO','C.A.B.A.','1225','2','1'],
                ['CABALLITO','C.A.B.A.','1226','2','1'],
                ['CABALLITO','C.A.B.A.','1227','2','1'],
                ['CABALLITO','C.A.B.A.','1228','2','1'],
                ['CABALLITO','C.A.B.A.','1229','2','1'],
                ['CABALLITO','C.A.B.A.','1230','2','1'],
                ['CABALLITO','C.A.B.A.','1231','2','1'],
                ['CABALLITO','C.A.B.A.','1232','2','1'],
                ['CABALLITO','C.A.B.A.','1233','2','1'],
                ['CABALLITO','C.A.B.A.','1234','2','1'],
                ['CABALLITO','C.A.B.A.','1235','2','1'],
                ['CABALLITO','C.A.B.A.','1236','2','1'],
                ['CABALLITO','C.A.B.A.','1405','2','1'],
                ['CABALLITO','C.A.B.A.','1413','2','1'],
                ['CABALLITO','C.A.B.A.','1423','2','1'],
                ['CONCEPCION (TUCUMAN)','LA MADRID','4176','22','2'],
                ['CONCEPCION (TUCUMAN)','ACHERAL','4134','TUCUMAN','2'],
                ['CONCEPCION (TUCUMAN)','ALTO VERDE','4153','TUCUMAN','2'],
                ['CONCEPCION (TUCUMAN)','ARCADIA','4147','TUCUMAN','2'],
                ['CONCEPCION (TUCUMAN)','CAMPO BELLO','4159','TUCUMAN','2'],
                ['CONCEPCION (TUCUMAN)','CONCEPCION','4146','TUCUMAN','2'],
                ['CONCEPCION (TUCUMAN)','DONATO ALVAREZ','4161','TUCUMAN','2'],
                ['CONCEPCION (TUCUMAN)','EL MOLINO','4149','TUCUMAN','2'],
                ['CONCEPCION (TUCUMAN)','INDEPENDENCIA','4143','TUCUMAN','2'],
                ['CONCEPCION (TUCUMAN)','INGENIO SANTA ANA','4155','TUCUMAN','2'],
                ['CONCEPCION (TUCUMAN)','LA COCHA','4162','TUCUMAN','2'],
                ['CONCEPCION (TUCUMAN)','LA TIPA','4157','TUCUMAN','2'],
                ['CONCEPCION (TUCUMAN)','MANUEL GARCIA','4166','TUCUMAN','2'],
                ['CONCEPCION (TUCUMAN)','RUMI PUNCO','4164','TUCUMAN','2'],
                ['CONCEPCION (TUCUMAN)','SOLDADO MALDONADO','4156','TUCUMAN','2'],
                ['CONCEPCION (TUCUMAN)','TACO RALO','4242','TUCUMAN','2'],
                ['CONCEPCION (TUCUMAN)','VILLA LA TRINIDAD','4151','TUCUMAN','2'],
                ['CONCEPCION (TUCUMAN)','VILLA QUINTEROS','4144','TUCUMAN','2'],
                ['CONCEPCION (TUCUMAN)','LOS SARMIENTOS','4157','TUCUMAN','3'],
                ['LUJAN','CARLOS KEEN','6701','2','1'],
                ['LUJAN','CORTINEZ','6712','2','1'],
                ['LUJAN','JAUREGUI','6706','2','1'],
                ['LUJAN','OPEN DOOR','6708','2','1'],
                ['LUJAN','TORRES','6703','2','1'],
                ['PINAMAR','COSTA DEL ESTE','7104','2','2'],
                ['PINAMAR','GENERAL LAVALLE','7103','2','2'],
                ['PINAMAR','LAS TONINAS','7106','2','3'],
                ['PINAMAR','MAR DE AJO','7109','2','3'],
                ['PINAMAR','OSTENDE','7164','2','2'],
                ['PINAMAR','PINAMAR','7167','2','2'],
                ['PINAMAR','SAN BERNARDO','7111','2','3'],
                ['PINAMAR','SAN CLEMENTE DEL TUYU','7105','2','3'],
                ['PINAMAR','SANTA TERESITA','7107','2','3'],
                ['PINAMAR','VALERIA DEL MAR','7166','2','2'],
                ['PINAMAR','VILLA GESELL','7165','2','3'],
                ['PINAMAR','GENERAL MADARIAGA','7163','2','3'],
                ['PINAMAR','MAR DE LAS PAMPAS','7165','2','3'],
                ['PINAMAR','MAR AZUL','7165','2','3'],
                ['BURZACO','ADROGUE','1846','2','1'],
                ['BURZACO','ALEJANDRO KORN','1864','2','1'],
                ['BURZACO','BARRIO DON ORIONE','1850','2','1'],
                ['BURZACO','CLAYPOLE','1849','2','1'],
                ['BURZACO','GLEW','1856','2','1'],
                ['BURZACO','GUERNICA','1862','2','1'],
                ['BURZACO','JOSE MARMOL','1845','2','1'],
                ['BURZACO','LONGCHAMPS','1854','2','1'],
                ['BURZACO','MINISTRO RIVADAVIA','1848','2','1'],
                ['BURZACO','SAN VICENTE','1865','2','1'],
                ['BURZACO','VILLA NUMANCIA','1858','2','1'],
                ['ESCOBAR','DIQUE LUJAN','1622','2','1'],
                ['ESCOBAR','ESCOBAR','1625','2','1'],
                ['ESCOBAR','LOMA VERDE','1628','2','1'],
                ['ESCOBAR','MAQUINISTA SAVIO','1620','2','1'],
                ['ESCOBAR','ZELAYA','1627','2','1'],
                ['LAFERRERE','CAÑUELAS','1814','2','1'],
                ['LAFERRERE','LAFERRERE','1757','2','1'],
                ['LAFERRERE','RAFAEL CASTILLO','1755','2','1'],
                ['LAFERRERE','VIRREY DEL PINO','1763','2','1'],
                ['FLORENCIO VARELA','VILLA VATTEONE','1853','2','1'],
                ['LA PLATA','JEPPENER','1986','2','1'],
                ['LA PLATA','MELCHOR ROMERO','1903','2','1'],
                ['LA PLATA','SAN MIGUEL DEL MONTE','7220','2','1'],
                ['LANUS','MONTE CHINGOLO','1825','2','1'],
                ['MONTE GRANDE','EL JAGUEL','1842','2','1'],
                ['MONTE GRANDE','TEMPERLEY','1809','2','1'],
                ['TEMPERLEY','TEMPERLEY','1834','2','1'],
                ['TEMPERLEY','TEMPERLEY','1835','2','1'],
                ['MORENO','GENERAL RODRIGUEZ','1717','2','1'],
                ['MORENO','HORNOS','1739','2','1'],
                ['MORENO','M PAZ BARRIO URIOSTE','1788','2','1'],
                ['MORENO','PLOMER','1733','2','1'],
                ['MORON','CIUDADELA','1701','2','1'],
                ['MORON','CIUDADELA','1702','2','1'],
                ['PILAR','KM. 45','1635','2','1'],
                ['SAN JUSTO','BARRIO SANTA CATALINA','1781','2','1'],
                ['SAN JUSTO','ISIDRO CASANOVA','1765','2','1'],
                ['SAN JUSTO','MERCADO CENTRAL','1771','2','1'],
                ['SAN JUSTO','VILLA LUZURIAGA','1753','2','1'],
                ['SAN MARTIN','CASEROS','1678','2','1'],
                ['SAN MARTIN','CHURRUCA','1691','2','1'],
                ['SAN MARTIN','VILLA LYNCH','1672','2','1'],
                ['SAN MIGUEL','GRAND BOURG','1615','2','1'],
                ['SAN MIGUEL','LAS MALVINAS','1750','2','1'],
                ['VICTORIA','BOULOGNE','1609','2','1'],
                ['VICTORIA','SAN ISIDRO','1642','2','1'],
                ['BARILOCHE','JOSE DE SAN MARTIN','9220','4','2'],
                ['BARILOCHE','LAGO PUELO','8431','4','2'],
                ['BARILOCHE','RIO PICO','9225','4','2'],
                ['BARILOCHE','LOS MENUCOS','8424','16','2'],
                ['COMODORO RIVADAVIA','COMODORO RIVADAVIA','9000','4','2'],
                ['COMODORO RIVADAVIA','COMODORO RIVADAVIA','9002','4','2'],
                ['COMODORO RIVADAVIA','COMODORO RIVADAVIA','9004','4','2'],
                ['COMODORO RIVADAVIA','COMODORO RIVADAVIA','9006','4','2'],
                ['COMODORO RIVADAVIA','COMODORO RIVADAVIA','9008','4','2'],
                ['COMODORO RIVADAVIA','COMODORO RIVADAVIA','9010','4','2'],
                ['COMODORO RIVADAVIA','GARAYALDE','9007','4','2'],
                ['COMODORO RIVADAVIA','RIO MAYO','9030','4','2'],
                ['COMODORO RIVADAVIA','PICO TRUNCADO','9015','20','2'],
                ['GENERAL ROCA','CHIMPAY','8364','16','2'],
                ['GENERAL ROCA','DARWIN','8364','16','2'],
                ['GENERAL ROCA','DARWIN','8369','16','2'],
                ['GENERAL ROCA','INGENIERO HUERGO','8334','16','2'],
                ['GENERAL ROCA','PASO CORDOVA','8333','16','2'],
                ['NUEQUEN','VILLA EL CHOCON','8311','15','2'],
                ['NUEQUEN','CIPOLETTI','8324','16','2'],
                ['RIO GALLEGOS','CAÑADA DE LAS VACAS','9305','20','2'],
                ['TRELEW','PUERTO MADRYN','9120','4','2'],
                ['BAHIA BLANCA','BATERIAS','8113','2','2'],
                ['BAHIA BLANCA','CORONEL PRINGLES','7530','2','2'],
                ['BAHIA BLANCA','DUFAUR','8164','2','2'],
                ['BAHIA BLANCA','GARRE','6411','2','2'],
                ['BAHIA BLANCA','GONZALES CHAVES','7513','2','2'],
                ['BAHIA BLANCA','INGENIERO WHITE','8103','2','2'],
                ['BAHIA BLANCA','LA VITICOLA','8122','2','2'],
                ['BAHIA BLANCA','MAYOR BURATOVICH','8146','2','2'],
                ['BAHIA BLANCA','PONTAUT','7535','2','2'],
                ['BAHIA BLANCA','PUAN','8120','2','2'],
                ['BAHIA BLANCA','PUAN','8180','2','2'],
                ['BAHIA BLANCA','PUNTA ALTA','8109','2','2'],
                ['BAHIA BLANCA','LA BILBAINA','6345','9','2'],
                ['AZUL','ARROYO DE LOS HUESOS','7301','2','2'],
                ['MAR DEL PLATA','RAMOS OTERO','7621','2','2'],
                ['MAR DEL PLATA','SAN CAYETANO','7521','2','2'],
                ['MAR DEL PLATA','SIERRA DE LOS PADRES','7601','2','2'],
                ['TANDIL','FAIR','7153','2','2'],
                ['TANDIL','LOS PINOS','7623','2','3'],
                ['MENDOZA','ALGARROBAL','5541','12','2'],
                ['MENDOZA','BARRIO QUIMICOS M.D.C.1','5520','12','2'],
                ['MENDOZA','BERMEJO','5533','12','2'],
                ['MENDOZA','BUENA NUEVA','5523','12','2'],
                ['MENDOZA','CACHUETA','5549','12','2'],
                ['MENDOZA','CHAPANAY','5589','12','2'],
                ['MENDOZA','COLONIA SEGOVIA','5646','12','2'],
                ['MENDOZA','EL CEIBO','5644','12','2'],
                ['MENDOZA','ESPINO','5657','12','2'],
                ['MENDOZA','ISLA GRANDE','5672','12','2'],
                ['MENDOZA','LA DORMIDA','5592','12','2'],
                ['MENDOZA','LAS CUEVAS','5557','12','2'],
                ['MENDOZA','LAS HERAS','5539','12','2'],
                ['MENDOZA','LAS HERAS','5540','12','2'],
                ['MENDOZA','LAS LAGUNAS','5680','12','2'],
                ['MENDOZA','LOS ALAMOS','5656','12','2'],
                ['MENDOZA','MAYOR DRUMMOND','5534','12','2'],
                ['MENDOZA','MONTE CASEROS','5571','12','2'],
                ['MENDOZA','MONTE CASEROS','5671','12','2'],
                ['MENDOZA','PARAJE BEBIDA','5677','12','2'],
                ['MENDOZA','REDUCCION','5645','12','2'],
                ['MENDOZA','SAN CARLOS  ','5569','12','2'],
                ['MENDOZA','SAN JOSE DE GUAYMALLEN','5519','12','2'],
                ['SAN JUAN','ABERASTAIN','5428','18','2'],
                ['SAN JUAN','BARREAL','5405','18','2'],
                ['SAN JUAN','CARPINTERIA','5437','18','2'],
                ['SAN JUAN','TUPELI','5445','18','2'],
                ['SAN JUAN','VILLA EL SALVADOR','5415','18','2'],
                ['SAN JUAN','ZONDA','5401','18','2'],
                ['SAN JUAN','ZONDA','5451','18','2'],
                ['SAN LUIS','AVIADOR ORIGONE','5741','19','2'],
                ['SAN LUIS','CABEZA DE NOVILLO','5779','19','2'],
                ['SAN LUIS','NOGOLI','5720','19','2'],
                ['SAN LUIS','PASO GRANDE','5753','19','2'],
                ['SAN LUIS','PASO GRANDE','5761','19','2'],
                ['SAN LUIS','SALADILLO','5751','19','2'],
                ['SAN LUIS','SAN ROQUE','5718','19','2'],
                ['SAN LUIS','LA PUNTA','5710','19','3'],
                ['SAN LUIS','LAS LAGUNAS','5719','19','3'],
                ['SAN LUIS','LA FIRMEZA','5714','22','2'],
                ['SAN RAFAEL','COLONIA ALVEAR','5632','12','2'],
                ['SAN RAFAEL','CUADRO NACIONAL','5607','12','2'],
                ['SAN RAFAEL','JAIME PRATS','5623','12','2'],
                ['SAN RAFAEL','SANTA TERESA','5605','12','2'],
                ['SAN RAFAEL','SANTA TERESA','5640','12','2'],
                ['VILLA MERCEDES','LAVAISSE','5730','19','2'],
                ['CAÑADA DE GOMEZ','CLASON','2146','21','2'],
                ['CAÑADA DE GOMEZ','LAS ROSAS','2520','21','2'],
                ['CAÑADA DE GOMEZ','SALTO GRANDE','2142','21','2'],
                ['CASILDA','CRUZ ALTA','2189','3','2'],
                ['CASILDA','ACEBAL','2109','21','2'],
                ['CASILDA','CAÑADA DE UCLE','2635','21','2'],
                ['CASILDA','LOS MOLINOS','2181','21','2'],
                ['CASILDA','SANTA TERESA','2111','21','2'],
                ['ROSARIO','ARROYO SECO','2128','21','2'],
                ['ROSARIO','BARRIO CAIMA','2245','21','2'],
                ['ROSARIO','CHARIGUE','2100','21','2'],
                ['ROSARIO','CLASON','2151','21','2'],
                ['ROSARIO','FRAY LUIS BELTRAN','2158','21','2'],
                ['SAN LORENZO','ANDINO','2215','21','2'],
                ['SAN LORENZO','BARRANCAS','2246','21','2'],
                ['SAN LORENZO','BERNARDO DE IRIGOYEN','2248','21','2'],
                ['SAN LORENZO','CAPITAN BERMUDEZ','2154','21','2'],
                ['SAN LORENZO','FRAY LUIS BELTRAN','2156','21','2'],
                ['SAN LORENZO','GENERALLOPEZ','2255','21','2'],
                ['SAN LORENZO','LARRECHEA','2241','21','2'],
                ['VENADO TUERTO','CAÑADA SECA','6105','2','2'],
                ['VENADO TUERTO','SANTA REGINA','6105','2','2'],
                ['VENADO TUERTO','ROSALES    ','6128','3','2'],
                ['VENADO TUERTO','VILLA ROSSI','6126','3','2'],
                ['VENADO TUERTO','CARMEN','2618','21','2'],
                ['VENADO TUERTO','MURPHY','2601','21','2'],
                ['VENADO TUERTO','SAN FRANCISCO DE 21','2604','21','2'],
                ['VENADO TUERTO','SAN GREGORIO','2613','21','2'],
                ['VENADO TUERTO','SANTA REGINA','6107','21','2'],
                ['VENADO TUERTO','SANTA TERESA','2609','21','3'],
                ['VENADO TUERTO','VILLA CAÑAS','2607','21','2'],
                ['CONCORDIA','BOMPLAND','3234','6','2'],
                ['CONCORDIA','CAPITAN JOAQUIN','3465','6','2'],
                ['CONCORDIA','CARLOS PELLEGRIN','3471','6','2'],
                ['CONCORDIA','MONTE CASEROS','3220','6','2'],
                ['CONCORDIA','PARADA LABOUGLE','3222','6','2'],
                ['CONCORDIA','COLONIA HOCKER','3270','8','2'],
                ['CONCORDIA','COLONIA VAZQUEZ','3267','8','2'],
                ['CONCORDIA','FEDERACION','3206','8','2'],
                ['CONCORDIA','PUERTO YERUA','3201','8','2'],
                ['CORRIENTES','ABALO','3436','6','2'],
                ['CORRIENTES','ISABEL VICTORIA - IFRAN','3453','6','2'],
                ['CORRIENTES','LAVALLE','3443','6','2'],
                ['CORRIENTES','MANUEL FLORENCIO','3446','6','2'],
                ['CORRIENTES','PEDRO R FERNANDEZ','3452','6','2'],
                ['FORMOSA','9 DE JULIO','3643','5','2'],
                ['FORMOSA','COLONIA CAMPO VILLAFAÑE','3609','13','2'],
                ['FORMOSA','DR E R.MEJIA CHIRIGUANOS','3625','13','2'],
                ['FORMOSA','EL COLORADO  ','3603','13','2'],
                ['FORMOSA','INGENIERO JUARES','3636','13','2'],
                ['FORMOSA','LAGUNA BLANCA','3613','13','2'],
                ['PARANA','PARAJE EL CARMEN','3199','6','2'],
                ['PARANA','DIAMANTE','3105','8','2'],
                ['PARANA','MACIA','3177','8','3'],
                ['PARANA','PUIGARI','3103','8','3'],
                ['PARANA','S ELENA','3192','8','2'],
                ['PARANA','VICTORIA','3153','8','2'],
                ['POSADAS','AZARA','3351','6','2'],
                ['POSADAS','COLONIA LIEBIG','3358','6','3'],
                ['POSADAS','ISLA APPIPE','3302','6','2'],
                ['POSADAS','COLONIA ALBERDI','3393','14','2'],
                ['POSADAS','JARDIN AMERICA','3328','14','2'],
                ['POSADAS','MONTECARLO','3384','14','2'],
                ['POSADAS','SANTIAGO DE LINIERS','3380','14','3'],
                ['POSADAS','SANTIAGO DE LINIERS','3387','14','2'],
                ['POSADAS','GOBERNADOR LOPEZ','3315','14','2'],
                ['RAFAELA','AGUARA GRANDE','3071','21','2'],
                ['RAFAELA','COLONIA SAN ANTONIO','2312','21','2'],
                ['RAFAELA','CONSTANZA','2311','21','2'],
                ['RAFAELA','ESTACION SAGUIER','2315','21','2'],
                ['RAFAELA','LHEMANN','2305','21','2'],
                ['RAFAELA','SAGUIER','2321','21','2'],
                ['RESISTENCIA','CABO ADRIANO AYALA','3519','5','2'],
                ['RESISTENCIA','COLONIA BENITEZ','3505','5','2'],
                ['RESISTENCIA','COLONIA BENITEZ','3510','5','2'],
                ['RESISTENCIA','PUERTO TIROL','3521','5','2'],
                ['RESISTENCIA','PUERTO TIROL','3535','5','2'],
                ['RESISTENCIA','RESISTENCIA','3500','5','2'],
                ['RESISTENCIA','RESISTENCIA','3501','5','2'],
                ['RESISTENCIA','RESISTENCIA','3502','5','2'],
                ['RESISTENCIA','RESISTENCIA','3504','5','2'],
                ['RESISTENCIA','RESISTENCIA','3506','5','2'],
                ['RESISTENCIA','RESISTENCIA','3508','5','2'],
                ['RESISTENCIA','SAMUHU','3543','5','2'],
                ['RESISTENCIA','SANTA SYLVINA','3541','5','2'],
                ['RESISTENCIA','SANTA SYLVINA','3542','5','2'],
                ['SANTA FE','CABAL','3091','21','2'],
                ['SANTA FE','COSTA DEL TOBA','2494','21','2'],
                ['SANTA FE','ESTEBAN RAMS','3077','21','2'],
                ['SANTA FE','FRANCK','3009','21','2'],
                ['SANTA FE','HELVECIA','3003','21','2'],
                ['SANTA FE','LA CRIOLLA','3052','21','2'],
                ['SANTA FE','LA GALLARETA','3057','21','2'],
                ['SANTA FE','LA GERALDINA','2343','21','2'],
                ['SANTA FE','LOGROÑO','3066','21','2'],
                ['SANTA FE','LOGROÑO','3075','21','2'],
                ['SANTA FE','SAN AGUSTIN','3019','21','2'],
                ['SANTA FE','SAN JOSE DEL RINCON','3001','21','2'],
                ['SANTA FE','SAN JOSE DEL RINCON','3089','21','2'],
                ['SANTA FE','SAN VICENTE','2447','21','2'],
                ['SANTA FE','21LICIA','2495','21','2'],
                ['CORDOBA','LA GUARDIA','5262','7','2'],
                ['CORDOBA','CHUCHIRAS','5875','3','3'],
                ['CORDOBA','CHUÑA','5218','3','2'],
                ['CORDOBA','COSTA SACATE','5961','3','2'],
                ['CORDOBA','EL FUERTECITO','2428','3','2'],
                ['CORDOBA','LA TORDILLA NORTE','2429','3','2'],
                ['CORDOBA','LUQUE','5967','3','2'],
                ['CORDOBA','OBISPO TREJO','5225','3','2'],
                ['CORDOBA','SALSACATE','5295','3','2'],
                ['CORDOBA','SAN LORENZO','5893','3','2'],
                ['CORDOBA','SANTIAGO TEMPLE','5124','3','2'],
                ['CORDOBA','SANTIAGO TEMPLE','5125','3','2'],
                ['CORDOBA','SARMIENTO','5212','3','2'],
                ['CORDOBA','TOTORALEJOS','5216','3','2'],
                ['CORDOBA','VILLA CERRO AZUL','5106','3','2'],
                ['CORDOBA','LA CORTADERA','2334','21','2'],
                ['RIO CUARTO','BULNES','5845','3','2'],
                ['RIO CUARTO','MONTE DE LOS GAUCHOS','5831','3','2'],
                ['RIO CUARTO','MONTE DE LOS GAUCHOS','5832','3','2'],
                ['RIO CUARTO','DEL CAMPILLO','6271','3','2'],
                ['SAN FRANCISCO','EL ARAÑADO','5947','3','2'],
                ['SAN FRANCISCO','SACANTA','5945','3','2'],
                ['SAN FRANCISCO','VILLA CONCEPCION','2433','3','2'],
                ['SAN FRANCISCO','Altos de Chipion','2417','3','2'],
                ['SAN FRANCISCO','AMBROSETTI','2339','21','2'],
                ['SAN FRANCISCO','AMBROSETTI','2352','21','2'],
                ['SAN FRANCISCO','CLUCELLAS','2407','21','2'],
                ['SAN FRANCISCO','PALACIOS','2326','21','2'],
                ['SAN FRANCISCO','SASTRE','2440','21','2'],
                ['VILLA MARIA','MONTE LEÑA','2564','7','2'],
                ['VILLA MARIA','CINTRA','2559','3','2'],
                ['VILLA MARIA','LA PALESTINA','5957','3','2'],
                ['VILLA MARIA','LAS MOJARRAS','5909','3','2'],
                ['VILLA MARIA','MORRISON','2568','3','2'],
                ['VILLA MARIA','NOETINGER','2563','3','2'],
                ['VILLA MARIA','SAN ANTONIO DE LITIN','2559','3','2'],
                ['VILLA MARIA','ORDOÑEZ','2595','21','2'],
                ['CATAMARCA','ANDALGALA','4740','7','2'],
                ['CATAMARCA','BALCOSNA','4719','7','2'],
                ['CATAMARCA','CHAQUIAGO','4742','7','2'],
                ['CATAMARCA','COLONIA DEL VALLE','4725','7','2'],
                ['CATAMARCA','EL PUESTO','5341','7','2'],
                ['CATAMARCA','HUALFIN','4751','7','2'],
                ['CATAMARCA','LA GUARDIA','5263','7','2'],
                ['CATAMARCA','POLCOS','4730','7','2'],
                ['CATAMARCA','RECREO','5260','7','3'],
                ['CATAMARCA','TINOGASTA','5340','7','2'],
                ['CATAMARCA','VILLA SAN ROQUE','5344','7','2'],
                ['CATAMARCA','VILLA VIL','4741','7','2'],
                ['JUJUY','AGUAS CALIENTES','4431','24','3'],
                ['JUJUY','AGUAS CALIENTES','4606','24','2'],
                ['JUJUY','BARRIOS','4649','24','2'],
                ['JUJUY','EL AGUILAR','4634','24','2'],
                ['JUJUY','HUMAHUACA','4630','24','2'],
                ['JUJUY','IRUYA','4633','24','2'],
                ['JUJUY','ITURBE','4629','24','2'],
                ['JUJUY','ITURBE','4632','24','2'],
                ['JUJUY','LA MENDIETA','4522','24','2'],
                ['JUJUY','LA QUIACA','4650','24','2'],
                ['JUJUY','OCLOYAS','4601','24','2'],
                ['JUJUY','SAN FRANCISCO DE ALFARCITO','4639','24','2'],
                ['LA RIOJA','ANJULLON','5303','11','2'],
                ['LA RIOJA','CHAÑAR','5277','11','2'],
                ['LA RIOJA','LOS MOLINOS','5301','11','3'],
                ['LA RIOJA','LOS MOLINOS','5328','11','2'],
                ['LA RIOJA','PINCHAS','5307','11','2'],
                ['LA RIOJA','SAN MIGUEL','5308','11','2'],
                ['LA RIOJA','SAN MIGUEL','5364','11','2'],
                ['LA RIOJA','SCHAQUI','5329','11','2'],
                ['LA RIOJA','VILLA CASTELLI','5355','11','2'],
                ['LA RIOJA','VILLA UNION','5350','11','2'],
                ['LA RIOJA','MIRANDA','5367','11','3'],
                ['LA RIOJA','OLPAS','5275','11','3'],
                ['ORAN','EMBARCACION','4550','17','2'],
                ['ORAN','ISLA DE CAÑAS','4531','17','2'],
                ['SALTA','SEY','4411','10','2'],
                ['SALTA','LA SILLETA','4433','17','2'],
                ['SANTIAGO DEL ESTERO','FRIAS SDE','4230','7','2'],
                ['SANTIAGO DEL ESTERO','LAS PALMITAS','4231','7','2'],
                ['SANTIAGO DEL ESTERO','SOL DE JULIO','5255','3','2'],
                ['SANTIAGO DEL ESTERO','CLODOMIRA','4338','22','2'],
                ['SANTIAGO DEL ESTERO','EL CHARCO','4306','22','3'],
                ['SANTIAGO DEL ESTERO','ESTACION ATAMISQUI','4315','22','2'],
                ['SANTIAGO DEL ESTERO','FORRES','4312','22','2'],
                ['SANTIAGO DEL ESTERO','GUARDIA ESCOLTA','3062','22','2'],
                ['SANTIAGO DEL ESTERO','LA CAÑADA','4354','22','2'],
                ['SANTIAGO DEL ESTERO','LA DONOSA','4221','22','3'],
                ['SANTIAGO DEL ESTERO','LORETO','4208','22','2'],
                ['SANTIAGO DEL ESTERO','SUMAMPA','5253','22','2'],
                ['SANTIAGO DEL ESTERO','TABOADA','4324','22','2'],
                ['SANTIAGO DEL ESTERO','CHOYA','4233','22','3'],
                ['TARTAGAL','CAMINERA SAN PEDRITO','4563','17','2'],
                ['TARTAGAL','PARAJE KM6','4560','17','2'],
                ['TARTAGAL','CAMPO DURAN','4566','17','2'],
                ['TARTAGAL','SANTA VICTORIA ESTE','4561','17','2'],
                ['TARTAGAL','MISION ALTO LA SIERRA','4561','17','2'],
                ['TUCUMAN','FUERTE QUEMADO','4141','7','2'],
                ['TUCUMAN','LULES','4128','17','2'],
                ['TUCUMAN','LA REDUCCION','4129','24','2'],
                ['TUCUMAN','LA REDUCCION','4131','24','2'],
                ['TUCUMAN','LOS RALOS','4182','24','2'],
                ['9 DE JULIO','ARBOLEDAS','6557','2','2'],
                ['9 DE JULIO','BLANCAGRANDE','6561','2','2'],
                ['9 DE JULIO','ESTEBAN DE LUCA','6475','2','2'],
                ['9 DE JULIO','GUANACO','6476','2','2'],
                ['9 DE JULIO','JUAN JOSE PASO','6474','2','2'],
                ['9 DE JULIO','MOREA','6507','2','2'],
                ['9 DE JULIO','PATRICIOS','6503','2','2'],
                ['GENERAL PICO','PARERA','6213','9','2'],
                ['JUNIN','AGUSTINA','6023','2','2'],
                ['JUNIN','BLAQUIER','6065','2','2'],
                ['JUNIN','COLONIA SERE','6459','2','2'],
                ['JUNIN','CORONEL GRANADA','6062','2','2'],
                ['JUNIN','CUCHA CUCHA','6746','2','2'],
                ['JUNIN','MORSE','6049','2','2'],
                ['JUNIN','PORVENIR','6063','2','2'],
                ['JUNIN','QUIROGA','6533','2','2'],
                ['MERCEDES','PEDERNALES','6665','2','2'],
                ['MERCEDES','VILLA MOQUEHUA','6625','2','2'],
                ['PERGAMINO','AGUSTINA','6001','2','2'],
                ['PERGAMINO','ASCENSION','6003','2','2'],
                ['PERGAMINO','GUERRICO','2706','2','2'],
                ['PERGAMINO','GUERRICO','2717','2','2'],
                ['PERGAMINO','VEDIA','6030','2','2'],
                ['PERGAMINO','RUEDA','2921','21','2'],
                ['SAN NICOLAS','SANCHEZ','2912','2','2'],
                ['SAN NICOLAS','VILLA CONSTITUCION','2919','21','2'],
                ['SANTA ROSA','TRENQUE LAUQUEN','6400','2','2'],
                ['SANTA ROSA','COLONIA SAN JOSE','6315','9','2'],
                ['SANTA ROSA','GENERAL ACHA','8200','9','2'],
                ['SANTA ROSA','METILEO','6367','9','2'],
                ['SANTA ROSA','ROLÑN','6305','9','2'],
                ['ZARATE','ALSINA','2938','2','2'],
                ['ZARATE','SAN ANTONIO DE ARECO','2760','2','2'],
                ['ZARATE','ZARATE','2800','2','2'],
                ['RECONQUISTA','AVELLANEDA','3561','21','2'],
                ['RECONQUISTA','EL TAJAMAR','3565','21','2'],
                ['CERRO LAS ROSAS','LA GRANJA','5115','3','2'],
                ['CERRO LAS ROSAS','UNQUILLO','5109','3','2'],
                ['TIGRE','DELTA','1621','2','1'],
                ['TIGRE','GENERAL PACHECO','1617','2','1'],
                ['VIEDMA','BALNEARIO EL CONDOR','8501','16','2'],
                ['VIEDMA','SIERRA PAILEMAN','8521','16','3'],
                ['VIEDMA','RAMOS MEXIA','8534','16','3'],
                ['CONCEPCION (TUCUMAN)','LOS AGUIRRE','4174','24','2'],
                ['CONCEPCION (TUCUMAN)','MONTEROS','4142','24','2'],
                ['LUJAN','GENERAL RODRIGUEZ','1747','2','1'],
                ['LUJAN','GENERAL RODRIGUEZ','1748','2','1'],
                ['LUJAN','GENERAL RODRIGUEZ','1749','2','1'],
                ['PINAMAR','MAR DEL TUYU','7108','2','3'],
                ['BURZACO','RAFAEL CALZADA','1847','2','1'],
                ['ESCOBAR','GARIN','1619','2','1'],
                ['ESCOBAR','MATHEU','1627','2','1'],
                ['FLORENCIO VARELA','CRUCE VARELA','1887','2','1'],
                ['LA PLATA','BARTOLOME BAVIO','1911','2','1'],
                ['LA PLATA','GONNET','1897','2','1'],
                ['LA PLATA','GONNET','1898','2','1'],
                ['LA PLATA','MAGDALENA','1913','2','1'],
                ['LA PLATA','RANCHOS','1987','2','1'],
                ['LA PLATA','VILELA','7208','2','1'],
                ['LANUS','SANTA ROSA','1867','2','1'],
                ['LANUS','VALENTIN ALSINA','1822','2','1'],
                ['MONTE GRANDE','CARLOS SPEGAZZINI','1812','2','1'],
                ['MONTE GRANDE','CARLOS SPEGAZZINI','1813','2','1'],
                ['MONTE GRANDE','LA UNION FERROVIARIA','1803','2','1'],
                ['MONTE GRANDE','LLAVALLOL','1836','2','1'],
                ['MONTE GRANDE','TRISTAN SUAREZ','1806','2','1'],
                ['QUILMES','DON BOSCO','1877','2','1'],
                ['QUILMES','SOLANO','1843','2','1'],
                ['TEMPERLEY','VILLA ALBERTINA','1829','2','1'],
                ['MORENO','M PAZ BARRIO EL MORO','1787','2','1'],
                ['MORENO','MORENO','1743','2','1'],
                ['MORENO','MORENO','1744','2','1'],
                ['MORON','BARRIO PARQUE LELOIR','1715','2','1'],
                ['MORON','PARQUE SAN MARTIN','1721','2','1'],
                ['MORON','VILLA TESSEI','1689','2','1'],
                ['PILAR','MANZANARES','1632','2','1'],
                ['SAN JUSTO','BARRIO SARMIENTO DON ROLANDO','1789','2','1'],
                ['SAN JUSTO','NUESTRA SEÑORA','1780','2','1'],
                ['SAN JUSTO','TABLADA','1766','2','1'],
                ['SAN JUSTO','VILLA CELINA','1772','2','1'],
                ['SAN MARTIN','EL PALOMAR','1684','2','1'],
                ['SAN MARTIN','EL PALOMAR','1685','2','1'],
                ['SAN MARTIN','VILLA RAFFO','1675','2','1'],
                ['SAN MIGUEL','MUÑIZ','1662','2','1'],
                ['SAN MIGUEL','TORTUGUITAS','1667','2','1'],
                ['SAN MIGUEL','VILLA DE MAYO','1614','2','1'],
                ['VICENTE LOPEZ','FLORIDA','1602','2','1'],
                ['VICENTE LOPEZ','FLORIDA','1604','2','1'],
                ['VICENTE LOPEZ','VILLA MARTELLI','1603','2','1'],
                ['BARILOCHE','CERRO CATEDRAL','8401','15','2'],
                ['BARILOCHE','SAN MARTIN DE LOS ANDES','8370','15','2'],
                ['BARILOCHE','DINA HUAPI','8402','16','2'],
                ['BARILOCHE','INGENIERO JACOBACCI','8418','16','2'],
                ['BARILOCHE','LAGUNA FRIA','8411','16','2'],
                ['BARILOCHE','LLAO LLAO','8409','16','2'],
                ['COMODORO RIVADAVIA','ALTO RIO SENGUER','9033','4','2'],
                ['COMODORO RIVADAVIA','DON BOSCO','9003','4','2'],
                ['COMODORO RIVADAVIA','FACUNDO','9031','4','2'],
                ['COMODORO RIVADAVIA','GRAL MOSCONI','9005','4','2'],
                ['COMODORO RIVADAVIA','RADA TILLY','9001','4','2'],
                ['COMODORO RIVADAVIA','UZCUDUM','9007','4','2'],
                ['COMODORO RIVADAVIA','CALETA OLIVIA','9011','20','2'],
                ['GENERAL ROCA','ALLEN','8328','16','2'],
                ['GENERAL ROCA','EL CUY','8333','16','2'],
                ['GENERAL ROCA','MAINQUE','8326','16','2'],
                ['RIO GALLEGOS','CALAFATE','9405','20','2'],
                ['TRELEW','DOLAVON','9107','4','2'],
                ['TRELEW','FLORENTINO AMEGHINO','9113','4','2'],
                ['TRELEW','GAIMAN','9105','4','2'],
                ['BAHIA BLANCA','17 DE AGOSTO','8129','2','2'],
                ['BAHIA BLANCA','17 DE AGOSTO','8225','2','2'],
                ['BAHIA BLANCA','ALFEREZ SAN MARTIN','8117','2','2'],
                ['BAHIA BLANCA','ARGERICH','8134','2','2'],
                ['BAHIA BLANCA','BERRAONDO','8124','2','2'],
                ['BAHIA BLANCA','CURUMALAN','7548','2','2'],
                ['BAHIA BLANCA','ESTELA  ','8127','2','2'],
                ['BAHIA BLANCA','FRANCISCO MURATURE','6341','2','2'],
                ['BAHIA BLANCA','LA PORTEÑA','6407','2','2'],
                ['BAHIA BLANCA','MASUREL','6438','2','2'],
                ['BAHIA BLANCA','MAZA','6343','2','2'],
                ['BAHIA BLANCA','ORIENTE','7509','2','2'],
                ['BAHIA BLANCA','PALANTELEN','6434','2','2'],
                ['BAHIA BLANCA','PUEBLO SAN JOSE','7541','2','2'],
                ['BAHIA BLANCA','QUENUMA','6335','2','2'],
                ['BAHIA BLANCA','SANTA TRINIDAD','7570','2','2'],
                ['AZUL','CROTTO','7307','2','2'],
                ['AZUL','LAPRIDA','7414','2','2'],
                ['AZUL','LAS NIEVES','7316','2','2'],
                ['AZUL','SIERRA CHICA','7401','2','2'],
                ['AZUL','VELLOSO','7305','2','2'],
                ['MAR DEL PLATA','BARRIO BATAN','7601','2','2'],
                ['MAR DEL PLATA','COSTA AZUL','7102','2','2'],
                ['MAR DEL PLATA','EL SOCORRO','7215','2','2'],
                ['MAR DEL PLATA','GENERAL PIRAN','7172','2','2'],
                ['MAR DEL PLATA','LOBERIA','7635','2','2'],
                ['MAR DEL PLATA','MAIPU','7160','2','2'],
                ['MAR DEL PLATA','MAIPU','7162','2','2'],
                ['MAR DEL PLATA','TAMANGUEYU','7633','2','2'],
                ['MAR DEL PLATA','VIVORATA','7612','2','2'],
                ['MENDOZA','ASTICA','5681','12','2'],
                ['MENDOZA','CHILECITO','5652','12','2'],
                ['MENDOZA','COLONIA RICHET','5678','12','2'],
                ['MENDOZA','COQUIMBITO','5522','12','2'],
                ['MENDOZA','COQUIMBITO  ','5513','12','2'],
                ['MENDOZA','DESAGUADERO','5598','12','2'],
                ['MENDOZA','IGLESIA','5684','12','2'],
                ['MENDOZA','LA LEGUA','5679','12','2'],
                ['MENDOZA','LA LIBERTAD','5558','12','2'],
                ['MENDOZA','LAS CATITAS','5594','12','2'],
                ['MENDOZA','LAS COLONIAS','5659','12','2'],
                ['MENDOZA','LUJAN DE CUYO','5507','12','2'],
                ['MENDOZA','LUZURIAGA','5513','12','2'],
                ['MENDOZA','LUZURIAGA','5516','12','2'],
                ['MENDOZA','PALMIRA','5584','12','2'],
                ['MENDOZA','POTRERILLOS','5552','12','2'],
                ['MENDOZA','RODRIGUEZ PEÑA','5669','12','2'],
                ['MENDOZA','SAN FRANCISCO DEL MONTE','5503','12','2'],
                ['MENDOZA','SAN ROQUE','5587','12','2'],
                ['MENDOZA','SAN ROQUE','5650','12','2'],
                ['MENDOZA','SANTA BLANCA','5655','12','2'],
                ['MENDOZA','VILLA HIPODROMO','5547','12','2'],
                ['SAN JUAN','CAMPO DE BATALLA','5434','18','2'],
                ['SAN JUAN','COLONIA FISCAL SARMIENTO','5450','18','2'],
                ['SAN JUAN','COLONIA RODAS','5421','18','2'],
                ['SAN JUAN','HUACO','5459','18','2'],
                ['SAN JUAN','HUACO','5463','18','2'],
                ['SAN JUAN','MOGNA','5412','18','2'],
                ['SAN JUAN','SAN JUAN','5400','18','2'],
                ['SAN JUAN','SAN JUAN','5402','18','2'],
                ['SAN JUAN','SAN JUAN','5406','18','2'],
                ['SAN JUAN','SAN JUAN','5422','18','2'],
                ['SAN JUAN','SAN JUAN','5452','18','2'],
                ['SAN LUIS','BOTIJAS','5707','19','2'],
                ['SAN LUIS','EL CHORRILLO','5723','19','2'],
                ['SAN LUIS','EL MILAGRO','5703','19','2'],
                ['SAN LUIS','LA TOMA','5750','19','2'],
                ['SAN LUIS','LA VERTIENTE','5760','19','2'],
                ['SAN LUIS','LOS HORNILLOS','5884','19','2'],
                ['SAN LUIS','POTRERO DE LOS FUNES','5704','19','2'],
                ['SAN LUIS','SAN PABLO','5774','19','2'],
                ['SAN LUIS','VILLA LARCA','5882','19','2'],
                ['SAN RAFAEL','EL TROPEZON','5629','12','2'],
                ['SAN RAFAEL','GOUDGE','5631','12','2'],
                ['SAN RAFAEL','LA ESCANDINAVA','5647','12','2'],
                ['SAN RAFAEL','RODOLFO ISELIN','5628','12','2'],
                ['CAÑADA DE GOMEZ','BOUQUET','2523','21','2'],
                ['CAÑADA DE GOMEZ','BUSTINZA','2504','21','2'],
                ['CAÑADA DE GOMEZ','CARCARAÑA','2138','21','2'],
                ['CAÑADA DE GOMEZ','EL TREBOL','2535','21','2'],
                ['CAÑADA DE GOMEZ','LOS CARDOS','2533','21','2'],
                ['CAÑADA DE GOMEZ','SAN JORGE','2451','21','2'],
                ['CAÑADA DE GOMEZ','SAN JORGE','2452','21','2'],
                ['CASILDA','BIGAND','2177','21','2'],
                ['CASILDA','PAVON ARRIBA','2110','21','2'],
                ['CASILDA','PUEBLO MUÑOZ','2119','21','3'],
                ['CASILDA','PUJATO','2122','21','2'],
                ['CASILDA','PUJATO','2123','21','2'],
                ['CASILDA','URANGA','2105','21','2'],
                ['CASILDA','URANGA','2112','21','2'],
                ['CASILDA','VILLA MUGUETA','2175','21','2'],
                ['CASILDA','ZAVALLA','2123','21','2'],
                ['CASILDA','ARMINDA','2119','21','3'],
                ['ROSARIO','GAHAN','2744','2','2'],
                ['ROSARIO','KM 57','2619','3','2'],
                ['ROSARIO','EL TREBOL','2534','21','2'],
                ['ROSARIO','PUEBLO MUÑOZ','2120','21','2'],
                ['SAN LORENZO','GALVEZ','2252','21','2'],
                ['SAN LORENZO','RICARDONE','2201','21','2'],
                ['VENADO TUERTO','HUGHES','2725','21','3'],
                ['VENADO TUERTO','RUNCIMAN','2611','21','2'],
                ['CONCORDIA','COLONIA LIBERTAD','3224','6','2'],
                ['CONCORDIA','FELIPE JOFRE','3472','6','2'],
                ['CONCORDIA','PASO DE LOS LIBRES','3230','6','2'],
                ['CONCORDIA','SOLARI','3476','6','2'],
                ['CONCORDIA','YAPEYU','3231','6','2'],
                ['CONCORDIA','BERDUC','3285','8','2'],
                ['CONCORDIA','COLONIA HUGHES','3275','8','2'],
                ['CONCORDIA','FEDERAL','3080','8','2'],
                ['CONCORDIA','FEDERAL','3180','8','2'],
                ['CONCORDIA','JUBILEO','3254','8','2'],
                ['CONCORDIA','LAGUNA LARGA','3241','8','2'],
                ['CONCORDIA','LUCAS NORTE','3241','8','2'],
                ['CONCORDIA','PUEBLO DOMINGUEZ','3246','8','2'],
                ['CONCORDIA','SANTA ANITA','3249','8','2'],
                ['CONCORDIA','TAPEBICUA','3236','8','2'],
                ['CONCORDIA','VILLA DEL ROSARIO','3229','8','2'],
                ['CONCORDIA','VILLA ELISA','3265','8','2'],
                ['CORRIENTES','BELLA VISTA','3432','6','2'],
                ['CORRIENTES','CAVIA CUE','3403','6','2'],
                ['CORRIENTES','PAGO DE LOS DESEOS','3425','6','2'],
                ['CORRIENTES','PASO DE LA PATRIA','3409','6','2'],
                ['FORMOSA','BARRIO JOSE DE SAN MARTIN','3698','5','2'],
                ['FORMOSA','VILLA ESCOLAR - G.LUCIO','3526','5','2'],
                ['FORMOSA','LAS LOMITAS','3630','13','2'],
                ['GUALEGUAYCHU','BASAVILVASO','3170','8','2'],
                ['GUALEGUAYCHU','GENERAL ALMADA','2824','8','2'],
                ['GUALEGUAYCHU','LARROQUE','2854','8','2'],
                ['GUALEGUAYCHU','URDINARRAIN','2826','8','2'],
                ['PARANA','COLONIA LOMA NEGRA','3112','8','2'],
                ['PARANA','CRUCESITAS','3151','8','2'],
                ['PARANA','LAGUNA DEL PESCADO','3155','8','2'],
                ['PARANA','MARIA GRANDE','3133','8','2'],
                ['PARANA','TEZADOS PINTO','3131','8','2'],
                ['POSADAS','SANTO TOME','3340','6','2'],
                ['POSADAS','SANTO TOME','3341','6','2'],
                ['POSADAS','GARRUCHO','3351','6','3'],
                ['POSADAS','25 DE MAYO','3379','14','2'],
                ['POSADAS','BARRIO DON SANTIAGO','3312','14','2'],
                ['POSADAS','BOMPLAND','3317','14','2'],
                ['POSADAS','CAMPO RAMON','3361','14','3'],
                ['POSADAS','CAMPO RAMON','3367','14','2'],
                ['POSADAS','CATARATAS DEL IGUAZU','3372','14','2'],
                ['POSADAS','CERRO AZUL','3313','14','2'],
                ['POSADAS','GARUPA','3304','14','2'],
                ['POSADAS','PUERTO IGUAZU','3370','14','2'],
                ['POSADAS','SAN IGNACIO','3322','14','2'],
                ['POSADAS','WANDA','3376','14','2'],
                ['POSADAS','FACHINAL','3304','14','2'],
                ['POSADAS','ITACUARARE','3357','14','3'],
                ['RAFAELA','LA PELADA','3027','21','2'],
                ['RAFAELA','MARIA LUISA','3028','21','2'],
                ['RAFAELA','PERICOTA','3025','21','2'],
                ['RAFAELA','PUJATO','2323','21','2'],
                ['RAFAELA','SUNCHALES','2322','21','2'],
                ['RESISTENCIA','LA PASTORIL','3517','5','2'],
                ['RESISTENCIA','LOS FRENTONES','3712','5','2'],
                ['RESISTENCIA','LOS FRENTONES','3715','5','2'],
                ['RESISTENCIA','MACHAGAY','3534','5','2'],
                ['RESISTENCIA','PUERTO VILELAS','3503','5','2'],
                ['RESISTENCIA','PUERTO VILELAS','3512','5','2'],
                ['RESISTENCIA','QUITILIPI','3530','5','2'],
                ['SANTA FE','VILLA TRINIDAD','2346','2','2'],
                ['SANTA FE','COLONIA ALPINA','2341','3','2'],
                ['SANTA FE','ANGELICA','2303','21','2'],
                ['SANTA FE','AURELIA SUD','2318','21','2'],
                ['SANTA FE','CARAGUATAY','3557','21','2'],
                ['SANTA FE','COLASTINE','3001','21','2'],
                ['SANTA FE','COLONIA LA PENCA','3045','21','2'],
                ['SANTA FE','EL TROPEZON','3010','21','2'],
                ['SANTA FE','GREGORIA PEREZ DE DENIS','3069','21','2'],
                ['SANTA FE','HUMBOLDT','3081','21','2'],
                ['SANTA FE','LAS GARZAS','3577','21','2'],
                ['SANTA FE','LAS TUNAS','3033','21','2'],
                ['SANTA FE','MARGARITA','3056','21','2'],
                ['SANTA FE','MONTE VERA','3014','21','2'],
                ['SANTA FE','MONTE VERA','3030','21','2'],
                ['SANTA FE','PERICOTA','3053','21','2'],
                ['SANTA FE','SAN JUSTO','3040','21','2'],
                ['SANTA FE','VIDELA','3048','21','2'],
                ['SANTA FE','VILLA ADELA','3596','21','2'],
                ['SANTA FE','SELVA','2354','22','2'],
                ['CORDOBA','CAPILLA LA CANDELARIA','5287','3','3'],
                ['CORDOBA','CIENAGA DEL CORO','5289','3','2'],
                ['CORDOBA','CRUZ DEL EJE','5280','3','2'],
                ['CORDOBA','EL CHACHO','5272','3','2'],
                ['CORDOBA','ESTACION CALCHIN','5969','3','2'],
                ['CORDOBA','LA TOTORILLA  ','5209','3','2'],
                ['CORDOBA','LOS AGUIRRES','5278','3','2'],
                ['CORDOBA','MALVINAS ARGENTINAS','5126','3','2'],
                ['CORDOBA','OJO DE AGUA','5293','3','2'],
                ['CORDOBA','RIO PRIMERO','5127','3','2'],
                ['CORDOBA','RIO SEGUNDO','5960','3','2'],
                ['CORDOBA','SAN FRANCISCO DEL CHAÑAR','5209','3','2'],
                ['CORDOBA','SAN PEDRO','5871','3','3'],
                ['CORDOBA','TIROLESA','5104','3','2'],
                ['CORDOBA','SAN MARCOS SIERRA','5282','3','3'],
                ['CORDOBA','EL MEDANITO','5272','11','3'],
                ['RIO CUARTO','ALEJO LEDESMA','2662','3','2'],
                ['RIO CUARTO','ALEJO LEDESMA','2663','3','2'],
                ['RIO CUARTO','BERROTARAN','5817','3','2'],
                ['RIO CUARTO','CHAZON','2675','3','2'],
                ['RIO CUARTO','EL BAÑADO RIO CUARTO','5801','3','2'],
                ['RIO CUARTO','LAS ACEQUIAS','5848','3','2'],
                ['RIO CUARTO','MALENA','5838','3','2'],
                ['RIO CUARTO','SAMPACHO','5829','3','2'],
                ['RIO CUARTO','SAN BASILIO','5841','3','2'],
                ['RIO CUARTO','VILLA VALERIA','6273','3','2'],
                ['RIO CUARTO','VILLA VALERIA','6373','3','2'],
                ['RIO TERCERO','COLONIA ALMADA','5987','3','2'],
                ['SAN FRANCISCO','LAS VARILLAS','5940','3','2'],
                ['SAN FRANCISCO','LAS VARILLAS','5992','3','2'],
                ['SAN FRANCISCO','MONTE REDONDO','2423','3','2'],
                ['SAN FRANCISCO','MORTEROS','2421','3','2'],
                ['SAN FRANCISCO','SAN PEDRO','2430','3','2'],
                ['SAN FRANCISCO','SEEBER','2418','3','2'],
                ['SAN FRANCISCO','SEEBER','2419','3','2'],
                ['SAN FRANCISCO','ARRUFO','2344','21','2'],
                ['SAN FRANCISCO','SAN GUILLERMO','2347','21','2'],
                ['SAN FRANCISCO','SANTA CLARA DE SAGUI','2405','21','2'],
                ['SAN FRANCISCO','ZENON PEREYRA','2409','21','2'],
                ['VILLA CARLOS PAZ','HUERTA GRANDE','5174','3','2'],
                ['VILLA CARLOS PAZ','SANTA MARIA DE PUNILLA','5164','3','2'],
                ['VILLA CARLOS PAZ','TANTI','5155','3','2'],
                ['VILLA CARLOS PAZ','VALLE HERMOSO','5168','3','2'],
                ['VILLA CARLOS PAZ','VILLA CARLOS PAZ','5152','3','2'],
                ['VILLA CARLOS PAZ','VILLA CARLOS PAZ','5154','3','2'],
                ['VILLA CARLOS PAZ','VILLA GIARDINO','5176','3','2'],
                ['VILLA MARIA','CHUCUL','5822','3','2'],
                ['VILLA MARIA','GENERAL CABRERA','5809','3','2'],
                ['VILLA MARIA','INRIVILLE','2587','3','2'],
                ['VILLA MARIA','MONTE MAIZ','2659','3','2'],
                ['VILLA MARIA','SALADILLO','2588','3','2'],
                ['CATAMARCA','COPACABANA','5333','7','2'],
                ['CATAMARCA','EL PUEBLITO','5331','7','2'],
                ['CATAMARCA','EL RODEO','4715','7','2'],
                ['CATAMARCA','FIAMBALA','5345','7','2'],
                ['CATAMARCA','LA PUERTA DE SAN JOSE','4748','7','2'],
                ['CATAMARCA','LA VIÑA','4722','7','2'],
                ['CATAMARCA','LAS CAÑAS','4235','7','2'],
                ['CATAMARCA','PORTEZUELO','4716','7','2'],
                ['CATAMARCA','POZO DEL MISTOL','4706','7','2'],
                ['JUJUY','FRAILE PINTADO','4506','10','2'],
                ['JUJUY','PERICO','4608','10','2'],
                ['JUJUY','PERICO','4610','10','2'],
                ['JUJUY','PERICO DEL CARMEN','4603','10','2'],
                ['JUJUY','PERICO DEL CARMEN','4605','10','2'],
                ['JUJUY','PUESTO DEL MARQUEZ','4644','10','2'],
                ['JUJUY','TILCARA','4624','10','2'],
                ['LA RIOJA','ANGUINAN','5363','11','2'],
                ['LA RIOJA','ANILLACO','5301','11','2'],
                ['LA RIOJA','ANILLACO','5330','11','2'],
                ['LA RIOJA','LA RIOJA','5300','11','2'],
                ['LA RIOJA','LA RIOJA','5302','11','2'],
                ['LA RIOJA','LA RIOJA','5305','11','2'],
                ['LA RIOJA','OLTA','5383','11','2'],
                ['LA RIOJA','SANTA VERA CRUZ','5326','11','2'],
                ['LA RIOJA','TALAMUYUNA','5304','11','2'],
                ['LA RIOJA','TUIZON','5385','11','2'],
                ['LA RIOJA','VICHIGASTA','5374','11','2'],
                ['LA RIOJA','VILLA SANAGASTA','5379','11','2'],
                ['LA RIOJA','VINCHINA','5357','11','2'],
                ['ORAN','PICHANAL','4534','17','2'],
                ['ORAN','SANTA ROSA','4535','17','2'],
                ['SALTA','ANIMANA','4429','17','2'],
                ['SALTA','EL CARRIL','4421','17','2'],
                ['SALTA','GAONA','4452','17','2'],
                ['SALTA','LAS LAJITAS','4450','17','2'],
                ['SALTA','LOS BAÑOS','4193','17','2'],
                ['SALTA','METAN','4440','17','2'],
                ['SALTA','LA VIÑA','4424','24','2'],
                ['SANTIAGO DEL ESTERO','RIO MUERTO','3719','5','2'],
                ['SANTIAGO DEL ESTERO','EL NARANJO','4191','17','2'],
                ['SANTIAGO DEL ESTERO','BARRIALITO','4225','22','3'],
                ['SANTIAGO DEL ESTERO','CHILCA JULIANA','4319','22','2'],
                ['SANTIAGO DEL ESTERO','EL MALACARA','3761','22','2'],
                ['SANTIAGO DEL ESTERO','EL VEINTICINCO DE MAYO','4283','22','2'],
                ['SANTIAGO DEL ESTERO','ICAÑO','4334','22','2'],
                ['SANTIAGO DEL ESTERO','LA NENA','3765','22','2'],
                ['SANTIAGO DEL ESTERO','LAS DELICIAS','4339','22','2'],
                ['SANTIAGO DEL ESTERO','LOS JURIES','3763','22','2'],
                ['SANTIAGO DEL ESTERO','REAL SAYANA','4335','22','2'],
                ['SANTIAGO DEL ESTERO','ROBLES','4313','22','2'],
                ['SANTIAGO DEL ESTERO','SUNCHO CORRAL','4350','22','2'],
                ['SANTIAGO DEL ESTERO','VILMER','4303','22','2'],
                ['SANTIAGO DEL ESTERO','VINARA','4223','22','3'],
                ['TARTAGAL','POCITOS','4568','17','2'],
                ['TUCUMAN','TACO ISLA','4321','22','2'],
                ['TUCUMAN','BANDA DEL RIO SALI','4109','24','2'],
                ['TUCUMAN','EL CHAÑAR','4120','24','2'],
                ['TUCUMAN','LOS SARMIENTOS','4065','24','2'],
                ['TUCUMAN','MERCEDES','4130','24','2'],
                ['TUCUMAN','SAN IGNACIO','4171','24','2'],
                ['TUCUMAN','VILLA BURRUYACU','4121','24','2'],
                ['9 DE JULIO','DAIREAUX','6555','2','2'],
                ['9 DE JULIO','DUDIGNAC','6505','2','2'],
                ['9 DE JULIO','HENDERSON','6465','2','2'],
                ['9 DE JULIO','LA NIÑA','6513','2','2'],
                ['9 DE JULIO','MONES CAZON','6469','2','2'],
                ['GENERAL PICO','BANDERALO','6244','2','2'],
                ['GENERAL PICO','CALEUFU','6387','9','2'],
                ['GENERAL PICO','CHANILAO','6201','9','2'],
                ['GENERAL PICO','EMBAJADOR MARTINI','6203','9','2'],
                ['GENERAL PICO','LA MARUJA','6386','9','2'],
                ['GENERAL PICO','SPELUZZI','6365','9','2'],
                ['GENERAL PICO','TRENEL','6369','9','2'],
                ['GENERAL PICO','LA COLINA','6269','19','2'],
                ['JUNIN','ACHUPALLAS','6627','2','2'],
                ['JUNIN','AMERICA','6237','2','2'],
                ['JUNIN','BAYAUCA','6078','2','2'],
                ['JUNIN','CASTILLA','6616','2','2'],
                ['JUNIN','COLIQUEO','6743','2','2'],
                ['JUNIN','LEANDRO N ALEM','6032','2','2'],
                ['JUNIN','MARTINEZ DE HOZ  ','6533','2','2'],
                ['JUNIN','SAN EMILIO','6017','2','2'],
                ['JUNIN','SAN EMILIO','6019','2','2'],
                ['JUNIN','ZABALIA','6018','2','2'],
                ['MERCEDES','CHIVILCOY','6620','2','2'],
                ['MERCEDES','POLVAREDAS','7267','2','2'],
                ['MERCEDES','SUIPACHA','6612','2','2'],
                ['PERGAMINO','CAMPO SALLES','2903','2','2'],
                ['PERGAMINO','CARABELAS','2703','2','2'],
                ['PERGAMINO','EL SOCORRO','2715','2','2'],
                ['PERGAMINO','GAHAN','2745','2','2'],
                ['PERGAMINO','RANCAGUA','2701','2','2'],
                ['PERGAMINO','RANCAGUA','2704','2','2'],
                ['PERGAMINO','ROJAS','2705','2','2'],
                ['PERGAMINO','VIÑA','2755','2','2'],
                ['PERGAMINO','PINZON','2223','21','2'],
                ['PERGAMINO','SARGENTO CABRAL','2105','21','2'],
                ['PERGAMINO','SARGENTO CABRAL','2106','21','2'],
                ['SAN NICOLAS','ISLA DE LOS LAURELES','2931','2','2'],
                ['SAN NICOLAS','EMPALME VILLA CONSTITUCION','2118','21','2'],
                ['SAN NICOLAS','EMPALME VILLA CONSTITUCION','2918','21','2'],
                ['SANTA ROSA','ALPACHIRI','6309','9','2'],
                ['SANTA ROSA','GENERAL SAN MARTIN','8206','9','2'],
                ['SANTA ROSA','PUEBLO ALASSA','6208','9','2'],
                ['SANTA ROSA','WINIFREDA','6313','9','2'],
                ['ZARATE','ATUCHA','2808','2','2'],
                ['ZARATE','CAMPANA','2804','2','2'],
                ['ZARATE','KILOMETRO 108','6723','2','2'],
                ['ZARATE','LIMA','2806','2','2'],
                ['ZARATE','RIO TALA','2944','2','2'],
                ['RECONQUISTA','ARROYO CEIBAL','3575','21','2'],
                ['RECONQUISTA','COLONIA SAN MANUEL','3563','21','2'],
                ['RECONQUISTA','MALABRIGO','3572','21','2'],
                ['RECONQUISTA','VILLA ADELA','3581','21','2'],
                ['CERRO LAS ROSAS','VILLA WARCARDE','5149','3','2'],
                ['TIGRE','NORDELTA TIGRE','1670','2','1'],
                ['TIGRE','TRONCOS DEL TALAR','1608','2','1'],
                ['VIEDMA','STROEDER','8508','2','3'],
                ['VIEDMA','SAN ANTONIO OESTE','8520','16','3'],
                ['VIEDMA','VIEDMA','8500','16','2'],
                ['CONCEPCION (TUCUMAN)','SIMOCA','4172','10','2'],
                ['CONCEPCION (TUCUMAN)','AGUILARES','4152','24','2'],
                ['CONCEPCION (TUCUMAN)','ALBERDI','4158','24','2'],
                ['CONCEPCION (TUCUMAN)','CAPITAN CACERES','4160','24','2'],
                ['CONCEPCION (TUCUMAN)','INGENIO SANTA LUCIA','4135','24','2'],
                ['CONCEPCION (TUCUMAN)','RIO SECO','4145','24','2'],
                ['LUJAN','LUJAN','6700','2','1'],
                ['LUJAN','LUJAN','6702','2','1'],
                ['LUJAN','VILLA FLANDRIA','6706','2','1'],
                ['PINAMAR','AGUAS VERDES','7112','2','3'],
                ['PINAMAR','GENERAL CONESA ','7101','2','2'],
                ['PINAMAR','LA LUCILA DEL MAR','7113','2','3'],
                ['PINAMAR','CARILO','7167','2','2'],
                ['BURZACO','BURZACO','1852','2','1'],
                ['ESCOBAR','INGENIERO MASCHWITZ','1623','2','1'],
                ['LAFERRERE','GONZALEZ CATAN','1758','2','1'],
                ['LAFERRERE','GONZALEZ CATAN','1759','2','1']
            ];

        $setup->getConnection()
            ->insertArray($setup->getTable('ids_andreani_codigo_postal'),
                ['sucursal','localidad','codigo_postal','provincia_id','zona_id'], $infoIdsAndreniCodigoPostal);

        $infoIdsAndreniSucursal =
            [
                ['Barracas ','Santo Domingo 3220 ','1293','Tel: (011) 5289-6700 ','1','3'],
                ['Belgrano ','Av. Cabildo 1386 ','1426','Tel: 4787-4949 ','1',''],
                ['Caballito ','Av. Díaz Vélez 4190 ','1185','Tel: (011) 4981-9330','1','109'],
                ['Colegiales ','Charlone 532 ','1427','Tel: (011) 4553-7577 ','1','4'],
                ['Flores ','Av. Juan Bautista Alberdi 3138 ','1406','Tel: (011) 4613-0809 ','1','15'],
                ['Palermo ','Bulnes 1422 ','1413','Tel: (011) 4861-7783','1','10'],
                ['Suipacha ','Suipacha 272 ','1008','Tel: (011) 4326-5185/ 0099 ','1','12'],
                ['Burzaco ','Av. Hipólito Yrigoyen 14343 ','1852','Tel: (011) 4238-4356','2','125'],
                ['Escobar','Belgrano 732','1625','Tel: (0348) 4422804','2','126'],
                ['Florencio Varela ','Av. San Martín 1471 ','1888','Tel: (011) 4287-5474 ','2','5'],
                ['Laferrere','Piedrabuena 6028','1757','Tel: 4626-8171','2','127'],
                ['Lanús ','Av. H. Irigoyen 2828','1824','Tel: (011) 4247-3140','2','7'],
                ['Monte Grande ','Boulevard Bs. As. 311','1842','Tel: (011) 4296-2445 ','2','9'],
                ['Morón ','H. Irigoyen 1293 ','1708','Tel: (011) 4483-4169 ','2','18'],
                ['Pilar ','Pedro Lagrave 282 Loc. 6 ','1629','Tel: (02322) 404347 / 427435','2','19'],
                ['Quilmes ','Andrés Baranda 625 ','1878','Tel: (011) 4257-3993','2','11'],
                ['San Isidro','Ituzaingó 375','1642','Tel: (011) 4732 3675','2',''],
                ['San Justo ','Pte. Perón 3368 ','1754','Tel: (011) 4441-2226','2','20'],
                ['San Martín ','Carrillo 2176','1650','Teléfono:4754-2466 / 4754 - 9449','2','21'],
                ['San Miguel ','Serrano 1695 ','1663','Tel: (011) 4667-6027','2','22'],
                ['Temperley ','Almirante Brown 3180','1834','Tel: (011) 4243-4882','2','13'],
                ['Tigre ','Gelly y Obes 1680','1648','Teléfono: 03327 - 472435','2','96'],
                ['Vicente López','Av. San Martín 4747','1602','Tel: (011) 4761-0864','2','23'],
                ['Victoria','Pte. Perón 2880 ','1644','Tel: (011) 4890-0175 ','2','24'],
                ['9 de Julio ','Bme. Mitre 1668 ','6500','Tel: (02317) 427813','2','71'],
                ['Azul','Perón 441 ','7300','Tel: (02281) 429952 ','2','34'],
                ['Bahía Blanca ','San Martín 351 PB  ','8000','Tel: (0291) 4518716 / 4557825','2','33'],
                ['Junín ','Capitán Yanquelén 190 ','6000','Tel: (023642) 421370','2','73'],
                ['La Plata','Av. 13 nº 1262 e/ 58 y 59','1900','Tel: (0221) 423 3875','2','6'],
                ['Luján','Mariano Moreno 856','6700','Tel: (02323) 438 652','2','120'],
                ['Mar del Plata ','Independencia 1946 ','7600','Tel: (0223) 4921081','2','35'],
                ['Mercedes','Calle 29 nº 482 E/20 y 22','6600','Tel: (02324) 435217/259','2','74'],
                ['Pergamino ','San Nicolás 13 ','2700','Tel: (02477) 431265 ','2','75'],
                ['Pinamar','Shaw 137 (7167) ','7167','','2','121'],
                ['San Nicolás ','Italia 67 ','2900','Tel: (0336) 4450580 /4453149','2','76'],
                ['Tandil ','San Martín 512 ','7000','Tel: (0249) 4440474 ','2','36'],
                ['Zárate ','Justa Lima 1626 ','2800','Tel: (03487) 446 464','2','79'],
                ['Río Cuarto ','Gral. Paz 1175 ','5800','Tel: (0358) 4642878 ','3','57'],
                ['Río Tercero ','Vélez Sarsfield 240','5850','Tel: (03571) 426674 ','3','58'],
                ['San Francisco ','Bv. 9 de Julio 1852 ','2400','Tel: (03564) 428886 ','3','59'],
                ['Villa Carlos Paz ','San Martín 555 ','5152','Te: (03541) 486648','3','60'],
                ['Villa María ','Bv. M. T. de Alvear 335 ','5900','Tel: (0353) 4528852 ','3','62'],
                ['Comodoro Rivadavia ','Belgrano 628','9000','Tel: (0297) 4443562 / 4463113','4','26'],
                ['Trelew ','San Martín 1045','9100','Tel: (0280) 4438289 ','4','31'],
                ['Resistencia ','Juan Domingo Perón 455 (3500) ','3500','Tel: (0362) 4432464 /4432340 ','5','54'],
                ['Corrientes ','Belgrano 275 (3400) ','3400','Tel: (0379) 4420332 ','6','48'],
                ['Catamarca ','Tucumán 665 (4700) ','4700','Tel: (03833) 454233 ','7','63'],
                ['Concordia ','Sarmiento 741 (3200) ','3200','Tel: (0345) 4230396 ','8','47'],
                ['Gualeguaychú ','Boulevard Pedro Jurado 458','2820','Tel: (03446) 442335','8','50'],
                ['Paraná ','Río Gualeguaychú 357 ','3100','Tel: (0343) 4223425 /4231207','8','51'],
                ['Santa Rosa ','Av. Perón 1935','6300','Tel: (02954) 432458 ','9','78'],
                ['General Pico','Calle 32 Nro: 45 Entre 1 y 3','6360','Tel: (02302) 432375','9','72'],
                ['Jujuy ','San Martín 979 (4600) ','4600','Tel: (0388) 4240607 /154122035 ','10','64'],
                ['La Rioja ','Belgrano 202 (5300) ','5300','Tel: (0380) 4424648','11','65'],
                ['Mendoza ','Zapata José Vicente 236 (5500) ','5500','Tel: (0261) 4202425/ 2697/ 2841 ','12','37'],
                ['San Rafael ','Barcala 180 (5600) ','5600','Tel: (0260) 4428997','12','40'],
                ['Formosa ','Saavedra 423 (3600) ','3600','Tel: (0370) 4422040 /4422042 ','13','49'],
                ['Posadas ','San Martín 2130 (3300) ','3300','Tel: (0376) 429796 ','14','52'],
                ['Neuquén ','Antártica Argentina 1880 (8300) ','8300','Tel: (0299) 4428609 /4425842','15','28'],
                ['Bariloche ','Villegas 369 (8400) ','8400','Tel: (0294) 4426810 /4436193 ','16','25'],
                ['General Roca ','Tucumán 137 (8332) ','8332','Tel: (0298) 433140 ','16','27'],
                ['Viedma','Juan José Biedma 339 (8500) ','8500','Tel: (02920) 424222','16','107'],
                ['Orán ','Coronel Egües 460  (4530) ','4530','Tel: (03878) 424868 ','17','66'],
                ['Salta ','España 1097 (4400) ','4400','Tel: (0387) 4212804 /4321028','17','67'],
                ['Tartagal ','Warnes 701 Esq. Gorriti (4561) ','4561','Tel: (03873) 425112 ','17','69'],
                ['San Juan ','Av. España 405 Sur','5400','Tel: (0264) 4241277 /4285021 ','18','38'],
                ['San Luis ','Av. Lafinur 63 extremo Sur','5700','Tel: (02664) 428326 ','19','39'],
                ['Villa Mercedes ','Belgrano 452','5730','Tel: (02657) 427169 ','19','41'],
                ['Río Gallegos ','Avellaneda 162 (9400) ','9400','Tel: (02966) 431397 ','20','29'],
                ['Cañada de Gómez ','Lavalle 1246 (2500) ','2500','Tel: (03471) 429017','21','42'],
                ['Casilda ','Buenos Aires 2447 (2170) ','2170','Tel: (03464) 424840 / 51','21','43'],
                ['Rosario ','Av. San Martín 2127 (2000) ','2000','Tel: (0341) 4728000 /1 ','21','44'],
                ['San Lorenzo ','Av. San Martin 737 (2200) ','2200','Tel: (03476) 435282 ','21','45'],
                ['Santa Fe','Av. 25 de Mayo 3340','3340','Tel: (0342) 4571030/1','21','55'],
                ['Venado Tuerto ','Belgrano 136 (2600) ','2600','Tel: (03462) 428581 ','21','46'],
                ['Reconquista','Iturraspe 1058','3560','Tel: (03482) 449230','21','80'],
                ['Santiago del Estero ','Sarmiento 77 (4200) ','4200','Tel: (0385) 4241524 / 4283425','22','68'],
                ['Río Grande ','Beauvoir 76 (9420) ','9420','Tel: (02964) 423862 ','23','30'],
                ['Ushuaia ','Gdor. Godoy 175 (9410) ','9410','Tel: (02901) 432434 ','23','32'],
                ['Tucumán ','San Martín 1175 (4000) ','4000','Tel: (0381) 4214580 /4306674 ','24','43'],
                ['Concepción de Tucumán ','Belgrano 119','4119','Tel: (03865) 422949 / 425003','24','110']
            ];

        $setup->getConnection()
            ->insertArray($setup->getTable('ids_andreani_sucursal'),
                ['nombre','direccion','codigo_postal','telefono','provincia_id','codigo_sucursal'], $infoIdsAndreniSucursal);

        $setup->endSetup();
    }
}
